@@ -361,6 +361,7 @@ bool GdalAdapter::loadImage(const QString& fn)
                 theImg.setPixel(px, py, qRgba(*r, *g, *b, a));
                 break;
             }
+#if QT_VERSION >= 0x040600
             case GdalAdapter::Hsl:
             {
                 float* h = lineBuf + (col*bandCount) + ixH;
@@ -375,6 +376,7 @@ bool GdalAdapter::loadImage(const QString& fn)
                 theImg.setPixel(px, py, C.rgba());
                 break;
             }
+#endif
             case GdalAdapter::Cmyk:
             {
                 float* c = lineBuf + (col*bandCount) + ixC;
@@ -422,6 +424,7 @@ bool GdalAdapter::loadImage(const QString& fn)
                 theImg.setPixel(px, py, qRgba(color->c1, color->c2, color->c3, color->c4));
                 break;
             }
+#if QT_VERSION >= 0x040600
             case GdalAdapter::Palette_HLS:
             {
                 float* ix = (lineBuf + (col*bandCount));
@@ -430,6 +433,7 @@ bool GdalAdapter::loadImage(const QString& fn)
                 theImg.setPixel(px, py, C.rgba());
                 break;
             }
+#endif
             case GdalAdapter::Palette_CMYK:
             {
                 float* ix = (lineBuf + (col*bandCount));
@@ -584,49 +588,56 @@ void GdalAdapter::cleanup()
     theProjection = QString();
 }
 
-bool GdalAdapter::toXML(QDomElement xParent)
+bool GdalAdapter::toXML(QXmlStreamWriter& stream)
 {
     bool OK = true;
 
-    QDomElement fs = xParent.ownerDocument().createElement("Images");
-    xParent.appendChild(fs);
+    stream.writeStartElement("Images");
 
-    fs.setAttribute("projection", theProjection);
+    stream.writeAttribute("projection", theProjection);
     if (!theSourceTag.isEmpty())
-        fs.setAttribute("source", theSourceTag);
+        stream.writeAttribute("source", theSourceTag);
     for (int i=0; i<theImages.size(); ++i) {
-        QDomElement f = xParent.ownerDocument().createElement("Image");
-        fs.appendChild(f);
-        f.setAttribute("filename", theImages[i].theFilename);
+        stream.writeStartElement("Image");
+        stream.writeAttribute("filename", theImages[i].theFilename);
+        stream.writeEndElement();
     }
 
+    stream.writeEndElement();
+
     return OK;
 }
 
-void GdalAdapter::fromXML(const QDomElement xParent)
+void GdalAdapter::fromXML(QXmlStreamReader& stream)
 {
     theBbox = QRectF();
     theImages.clear();
 
-    QDomElement fs = xParent.firstChildElement();
-    while(!fs.isNull()) {
-        if (fs.tagName() == "Images") {
-            if (fs.hasAttribute("projection"))
-                theProjection = fs.attribute("projection");
-            if (fs.hasAttribute("source"))
-                theSourceTag = fs.attribute("source");
-            QDomElement f = fs.firstChildElement();
-            while(!f.isNull()) {
-                if (f.tagName() == "Image") {
-                    QString fn = f.attribute("filename");
+    while(!stream.atEnd() && !stream.isEndElement()) {
+        if (stream.name() == "Images") {
+            if (stream.attributes().hasAttribute("projection"))
+                theProjection = stream.attributes().value("projection").toString();
+            if (stream.attributes().hasAttribute("source"))
+                theSourceTag = stream.attributes().value("source").toString();
+            stream.readNext();
+            while(!stream.atEnd() && !stream.isEndElement()) {
+                if (stream.name() == "Image") {
+                    QString fn = stream.attributes().value("filename").toString();
                     if (!fn.isEmpty())
                         loadImage(fn);
+                    stream.readNext();
+                } else if (!stream.isWhitespace()) {
+                    qDebug() << "gdalimage: logic error: " << stream.name() << " : " << stream.tokenType() << " (" << stream.lineNumber() << ")";
+                    stream.skipCurrentElement();
                 }
-                f = f.nextSiblingElement();
+                stream.readNext();
             }
+        } else if (!stream.isWhitespace()) {
+            qDebug() << "gdalmain: logic error: " << stream.name() << " : " << stream.tokenType() << " (" << stream.lineNumber() << ")";
+            stream.skipCurrentElement();
         }
 
-        fs = fs.nextSiblingElement();
+        stream.readNext();
     }
 }
 
