@@ -87,13 +87,13 @@ public:
     /*!
      * @return the min zoom value
      */
-    virtual int 		getMinZoom	() const { return -1; }
+    virtual int 		getMinZoom	(const QRectF &) const { return -1; }
 
     //! returns the max zoom value
     /*!
      * @return the max zoom value
      */
-    virtual int		getMaxZoom	() const { return -1; }
+    virtual int		getMaxZoom	(const QRectF &) const { return -1; }
 
     //! returns the current zoom
     /*!
@@ -106,6 +106,8 @@ public:
      * @return the source tag
      */
     virtual QString	getSourceTag		() const;
+    virtual void setSourceTag (const QString& ) {};
+
     //! returns the Url of the usage license
     /*!
      * @return the Url of the usage license
@@ -113,8 +115,8 @@ public:
     virtual QString	getLicenseUrl() const {return "";}
 
     virtual int		getAdaptedZoom() const { return -1; }
-    virtual int 	getAdaptedMinZoom() const { return -1; }
-    virtual int		getAdaptedMaxZoom() const { return -1; }
+    virtual int 	getAdaptedMinZoom(const QRectF &) const { return -1; }
+    virtual int		getAdaptedMaxZoom(const QRectF &) const { return -1; }
 
     virtual void	zoom_in() {}
     virtual void	zoom_out() {}
@@ -138,8 +140,8 @@ public:
 
     virtual void cleanup();
 
-    virtual bool toXML(QDomElement xParent);
-    virtual void fromXML(const QDomElement xParent);
+    virtual bool toXML(QXmlStreamWriter& stream);
+    virtual void fromXML(QXmlStreamReader& stream);
     virtual QString toPropertiesHtml();
 
     virtual void setSettings(QSettings* /*aSet*/) {}
