@@ -370,54 +370,59 @@ void WalkingPapersAdapter::cleanup()
     theCoordBbox = QRectF();
 }
 
-bool WalkingPapersAdapter::toXML(QDomElement xParent)
+bool WalkingPapersAdapter::toXML(QXmlStreamWriter& stream)
 {
     bool OK = true;
 
-    QDomElement fs = xParent.ownerDocument().createElement("Images");
-    xParent.appendChild(fs);
-
+    stream.writeStartElement("Images");
     for (int i=0; i<theImages.size(); ++i) {
-        QDomElement f = xParent.ownerDocument().createElement("Image");
-        fs.appendChild(f);
-        f.setAttribute("filename", theImages[i].theFilename);
-        f.setAttribute("top", theImages[i].theBBox.top());
-        f.setAttribute("left", theImages[i].theBBox.left());
-        f.setAttribute("width", theImages[i].theBBox.width());
-        f.setAttribute("height", theImages[i].theBBox.height());
-        f.setAttribute("rotation", theImages[i].rotation);
+        stream.writeStartElement("Image");
+        stream.writeAttribute("filename", theImages[i].theFilename);
+        stream.writeAttribute("top", QString::number(theImages[i].theBBox.top()));
+        stream.writeAttribute("left", QString::number(theImages[i].theBBox.left()));
+        stream.writeAttribute("width", QString::number(theImages[i].theBBox.width()));
+        stream.writeAttribute("height", QString::number(theImages[i].theBBox.height()));
+        stream.writeAttribute("rotation", QString::number(theImages[i].rotation));
+        stream.writeEndElement();
     }
+    stream.writeEndElement();
 
     return OK;
 }
 
-void WalkingPapersAdapter::fromXML(const QDomElement xParent)
+void WalkingPapersAdapter::fromXML(QXmlStreamReader& stream)
 {
     theCoordBbox = QRectF();
     theImages.clear();
 
-    QDomElement fs = xParent.firstChildElement();
-    while(!fs.isNull()) {
-        if (fs.tagName() == "Images") {
-            QDomElement f = fs.firstChildElement();
-            while(!f.isNull()) {
-                if (f.tagName() == "Image") {
-                    QString fn = f.attribute("filename");
+    stream.readNext();
+    while(!stream.atEnd() && !stream.isEndElement()) {
+        if (stream.name() == "Images") {
+            stream.readNext();
+            while(!stream.atEnd() && !stream.isEndElement()) {
+                if (stream.name() == "Image") {
+                    QString fn = stream.attributes().value("filename").toString();
                     if (!fn.isEmpty()) {
-                        double x = f.attribute("left").toDouble();
-                        double y = f.attribute("top").toDouble();
-                        double w = f.attribute("width").toDouble();
-                        double h = f.attribute("height").toDouble();
-                        int r = f.attribute("rotation").toInt();
+                        double x = stream.attributes().value("left").toString().toDouble();
+                        double y = stream.attributes().value("top").toString().toDouble();
+                        double w = stream.attributes().value("width").toString().toDouble();
+                        double h = stream.attributes().value("height").toString().toDouble();
+                        int r = stream.attributes().value("rotation").toString().toInt();
                         QRectF bbox(x, y, w, h);
                         loadImage(fn, bbox, r);
                     }
+                    stream.readNext();
+                } else if (!stream.isWhitespace()) {
+                    qDebug() << "wp: logic error: " << stream.name() << " : " << stream.tokenType() << " (" << stream.lineNumber() << ")";
+                    stream.skipCurrentElement();
                 }
-                f = f.nextSiblingElement();
+                stream.readNext();
             }
+        } else if (!stream.isWhitespace()) {
+            qDebug() << "wp: logic error: " << stream.name() << " : " << stream.tokenType() << " (" << stream.lineNumber() << ")";
+            stream.skipCurrentElement();
         }
-
-        fs = fs.nextSiblingElement();
+        stream.readNext();
     }
 }
 
