@@ -73,13 +73,13 @@ public:
     /*!
      * @return the min zoom value
      */
-    virtual int 		getMinZoom	() const { return -1; }
+    virtual int 		getMinZoom	(const QRectF &) const { return -1; }
 
     //! returns the max zoom value
     /*!
      * @return the max zoom value
      */
-    virtual int		getMaxZoom	() const { return -1; }
+    virtual int		getMaxZoom	(const QRectF &) const { return -1; }
 
     //! returns the current zoom
     /*!
@@ -92,6 +92,7 @@ public:
      * @return the source tag
      */
     virtual QString	getSourceTag		() const { return ""; }
+    virtual void setSourceTag (const QString& ) {};
 
     //! returns the Url of the usage license
     /*!
@@ -100,8 +101,8 @@ public:
     virtual QString	getLicenseUrl() const {return "";}
 
     virtual int		getAdaptedZoom() const { return -1; }
-    virtual int 	getAdaptedMinZoom() const { return -1; }
-    virtual int		getAdaptedMaxZoom() const { return -1; }
+    virtual int 	getAdaptedMinZoom(const QRectF &) const { return -1; }
+    virtual int		getAdaptedMaxZoom(const QRectF &) const { return -1; }
 
     virtual void	zoom_in() {}
     virtual void	zoom_out() {}
@@ -125,8 +126,8 @@ public:
 
     virtual void cleanup();
 
-    virtual bool toXML(QDomElement xParent);
-    virtual void fromXML(const QDomElement xParent);
+    virtual bool toXML(QXmlStreamWriter& stream);
+    virtual void fromXML(QXmlStreamReader& stream);
     virtual QString toPropertiesHtml();
 
     virtual void setSettings(QSettings* aSet) {theSets = aSet;}
