@@ -69,13 +69,13 @@ public:
     /*!
      * @return the min zoom value
      */
-    virtual int 		getMinZoom	() const { return -1; }
+    virtual int 		getMinZoom	(const QRectF &) const { return -1; }
 
     //! returns the max zoom value
     /*!
      * @return the max zoom value
      */
-    virtual int		getMaxZoom	() const { return -1; }
+    virtual int		getMaxZoom	(const QRectF &) const { return -1; }
 
     //! returns the current zoom
     /*!
@@ -88,6 +88,7 @@ public:
      * @return the source tag
      */
     virtual QString	getSourceTag		() const { return "Yahoo"; }
+    virtual void setSourceTag (const QString& ) {};
 
     //! returns the Url of the usage license
     /*!
@@ -96,8 +97,8 @@ public:
     virtual QString	getLicenseUrl() const {return "";}
 
     virtual int		getAdaptedZoom() const { return -1; }
-    virtual int 	getAdaptedMinZoom() const { return -1; }
-    virtual int		getAdaptedMaxZoom() const { return -1; }
+    virtual int 	getAdaptedMinZoom(const QRectF &) const { return -1; }
+    virtual int		getAdaptedMaxZoom(const QRectF &) const { return -1; }
 
     virtual void	zoom_in() {}
     virtual void	zoom_out() {}
@@ -121,8 +122,8 @@ public:
 
     virtual void cleanup() {}
 
-    virtual bool toXML(QDomElement /*xParent*/) { return true; }
-    virtual void fromXML(const QDomElement /*xParent*/) {}
+    virtual bool toXML(QXmlStreamWriter& /*stream*/) { return true; }
+    virtual void fromXML(QXmlStreamReader& /*xParent*/) {}
     virtual QString toPropertiesHtml() {return "";}
 
     virtual void setSettings(QSettings* /*aSet*/) {}
