@@ -78,13 +78,13 @@ public:
     /*!
      * @return the min zoom value
      */
-    virtual int 		getMinZoom	() const;
+    virtual int 		getMinZoom	(const QRectF &) const;
 
     //! returns the max zoom value
     /*!
      * @return the max zoom value
      */
-    virtual int		getMaxZoom	() const;
+    virtual int		getMaxZoom	(const QRectF &) const;
 
     //! returns the current zoom
     /*!
@@ -97,6 +97,7 @@ public:
      * @return the source tag
      */
     virtual QString	getSourceTag		() const { return "Yahoo"; }
+    virtual void setSourceTag (const QString& ) {};
 
     //! returns the Url of the usage license
     /*!
@@ -105,8 +106,8 @@ public:
     virtual QString	getLicenseUrl() const {return "";}
 
     virtual int		getAdaptedZoom()   const;
-    virtual int 	getAdaptedMinZoom	() const;
-    virtual int		getAdaptedMaxZoom	() const;
+    virtual int 	getAdaptedMinZoom	(const QRectF &) const;
+    virtual int		getAdaptedMaxZoom	(const QRectF &) const;
 
     virtual void	zoom_in();
     virtual void	zoom_out();
@@ -130,8 +131,8 @@ public:
 
     virtual void cleanup() {}
 
-    virtual bool toXML(QDomElement /*xParent*/) { return true; }
-    virtual void fromXML(const QDomElement /*xParent*/) {}
+    virtual bool toXML(QXmlStreamWriter& /*stream*/) { return true; }
+    virtual void fromXML(QXmlStreamReader& /*xParent*/) {}
     virtual QString toPropertiesHtml() {return "";}
 
     virtual void setSettings(QSettings* /*aSet*/) {}
