@@ -42,8 +42,8 @@ class SetTagCommand : public TagCommand
         virtual void redo();
         virtual bool buildDirtyList(DirtyList& theList);
 
-        virtual bool toXML(QDomElement& xParent) const;
-        static SetTagCommand* fromXML(Document* d,QDomElement e);
+        virtual bool toXML(QXmlStreamWriter& stream) const;
+        static SetTagCommand* fromXML(Document* d, QXmlStreamReader& stream);
 
     private:
         int theIdx;
@@ -61,8 +61,8 @@ class ClearTagsCommand : public TagCommand
         virtual void undo();
         virtual void redo();
 
-        virtual bool toXML(QDomElement& xParent) const;
-        static ClearTagsCommand* fromXML(Document* d,QDomElement e);
+        virtual bool toXML(QXmlStreamWriter& stream) const;
+        static ClearTagsCommand* fromXML(Document* d, QXmlStreamReader& stream);
 };
 
 class ClearTagCommand : public TagCommand
@@ -75,8 +75,8 @@ class ClearTagCommand : public TagCommand
         virtual void redo();
         virtual bool buildDirtyList(DirtyList& theList);
 
-        virtual bool toXML(QDomElement& xParent) const;
-        static ClearTagCommand* fromXML(Document* d,QDomElement e);
+        virtual bool toXML(QXmlStreamWriter& stream) const;
+        static ClearTagCommand* fromXML(Document* d, QXmlStreamReader& stream);
 
     private:
         int theIdx;
