@@ -5,6 +5,7 @@
 #include "DocumentCommands.h"
 #include "LayerWidget.h"
 #include "PropertiesDock.h"
+#include "Global.h"
 
 #ifdef USE_ZBAR
 #include <zbar.h>
@@ -91,6 +92,7 @@ GeoImageDock::GeoImageDock(MainWindow *aMain)
     setContextMenuPolicy(Qt::ActionsContextMenu);
 
     centerAction = new QAction(tr("Center map"), this);
+    centerAction->setShortcut(tr("c"));
     remImagesAction = new QAction(tr("Remove Images"), this);
     toClipboardAction = new QAction(tr("Copy filename to clipboard"), this);
     nextImageAction = new QAction(tr("Select next image"), this);
@@ -211,7 +213,7 @@ void GeoImageDock::setImage(int ImageId)
         updateByMe = true;
         if (!Main->properties()->isSelected(theFeature)) {
             Main->properties()->setSelection(theFeature);
-            Main->view()->invalidate(true, false);
+            Main->view()->invalidate(true, true, false);
         }
         updateByMe = false;
     }
@@ -231,7 +233,7 @@ void GeoImageDock::removeImages(void)
         }
         if (usedTrackPoints.at(i).inserted) {
             Pt->layer()->remove(Pt);
-            delete Pt;
+            g_backend.deallocFeature(Pt->layer(), Pt);
         }
         else
             Pt->clearTag("Picture");
@@ -241,7 +243,7 @@ void GeoImageDock::removeImages(void)
     curImage = -1;
     Image->setImage("");
 
-    Main->view()->invalidate(true, false);
+    Main->view()->invalidate(true, true, false);
 }
 
 void GeoImageDock::toClipboard(void)
@@ -381,18 +383,41 @@ void GeoImageDock::loadImage(QString file, Coord pos)
             break;
         else
             Pt = 0;
+
+    PhotoNode* phNode;
     if (!Pt) {
-        Pt = new Node(pos);
+        phNode = g_backend.allocPhotoNode(theLayer, pos);
         theLayer->add(Pt);
-        theLayer->indexAdd(Pt->boundingBox(), Pt);
+        g_backend.sync(Pt);
+    } else {
+        if (CAST_TRACKNODE(Pt))
+            phNode = g_backend.allocPhotoNode(theLayer, *CAST_TRACKNODE(Pt));
+        else
+            phNode = g_backend.allocPhotoNode(theLayer, *Pt);
+        theLayer->add(phNode);
+        for (int i=0; i<Pt->sizeParents(); ++i) {
+            Feature *P = CAST_FEATURE(Pt->getParent(i));
+            int idx = P->find(Pt);
+            if (Way* W = CAST_WAY(P)) {
+                W->add(phNode, idx);
+            } else if (Relation* R = CAST_RELATION(P)) {
+                R->add(R->getRole(idx), phNode, idx);
+            } else if (TrackSegment* S = CAST_SEGMENT(P)) {
+                S->add(phNode, idx);
+            }
+            P->remove(idx+1);
+            g_backend.deallocFeature(theLayer, Pt);
+
+        }
+
     }
 
     QDateTime time = QFileInfo(file).created();
 
     //Pt->setTag("_waypoint_", "true");
-    Pt->setTag("_picture_", "GeoTagged");
-    Pt->setPhoto(QPixmap(file));
-    addUsedTrackpoint(NodeData(Pt, file, time, i == theLayer->size()));
+    phNode->setTag("_picture_", "GeoTagged");
+    phNode->setPhoto(QPixmap(file));
+    addUsedTrackpoint(NodeData(phNode, file, time, i == theLayer->size()));
 }
 
 void GeoImageDock::loadImages(QStringList fileNames)
@@ -625,7 +650,7 @@ void GeoImageDock::loadImages(QStringList fileNames)
         }
 
         if (positionValid) {
-            Coord newPos(angToCoord(lat), angToCoord(lon));
+            Coord newPos(lon, lat);
             Node *Pt = 0;
             int i = 0;
             for (; i<theLayer->size(); ++i) { // use existing TrackPoint if there is one in small distance
@@ -635,16 +660,38 @@ void GeoImageDock::loadImages(QStringList fileNames)
                 else
                     Pt = 0;
             }
-            if (!Pt) {
-                Pt = new Node(newPos);
-                theLayer->add(Pt);
-                theLayer->indexAdd(Pt->boundingBox(), Pt);
-            }
 
-            //Pt->setTag("_waypoint_", "true");
-            Pt->setTag("_picture_", "GeoTagged");
-            Pt->setPhoto(QPixmap(file));
-            addUsedTrackpoint(NodeData(Pt, file, time, i == theLayer->size()));
+            PhotoNode* phNode;
+            if (!Pt) {
+                phNode = g_backend.allocPhotoNode(theLayer, newPos);
+                theLayer->add(phNode);
+                g_backend.sync(phNode);
+            } else {
+                if (CAST_TRACKNODE(Pt))
+                    phNode = g_backend.allocPhotoNode(theLayer, *CAST_TRACKNODE(Pt));
+                else
+                    phNode = g_backend.allocPhotoNode(theLayer, *Pt);
+                theLayer->add(phNode);
+                for (int i=0; i<Pt->sizeParents(); ++i) {
+                    Feature *P = CAST_FEATURE(Pt->getParent(i));
+                    int idx = P->find(Pt);
+                    if (Way* W = CAST_WAY(P)) {
+                        W->add(phNode, idx);
+                    } else if (Relation* R = CAST_RELATION(P)) {
+                        R->add(R->getRole(idx), phNode, idx);
+                    } else if (TrackSegment* S = CAST_SEGMENT(P)) {
+                        S->add(phNode, idx);
+                    }
+                    P->remove(idx+1);
+                    g_backend.deallocFeature(theLayer, Pt);
+
+                }
+
+            }
+                        //Pt->setTag("_waypoint_", "true");
+            phNode->setTag("_picture_", "GeoTagged");
+            phNode->setPhoto(QPixmap(file));
+            addUsedTrackpoint(NodeData(phNode, file, time, i == theLayer->size()));
         } else if (!time.isNull() && res == 2) {
 
             if (offset == -1) { // ask the user to specify an offset for the images
@@ -687,7 +734,7 @@ void GeoImageDock::loadImages(QStringList fileNames)
                     else
                         offset = 0;
                 } else {
-                    theView->invalidate(true, false);
+                    theView->invalidate(true, true, false);
                     if (photoLayer && !photoLayer->size()) {
                         theDocument->remove(photoLayer);
                         SAFE_DELETE(photoLayer);
@@ -699,13 +746,13 @@ void GeoImageDock::loadImages(QStringList fileNames)
             time = time.addSecs(offset);
 
             Feature *feature = NULL;
-            Node *Pt, *bestPt = NULL;
+            TrackNode *Pt, *bestPt = NULL;
             int a, secondsTo = INT_MAX;
             int u;
 
             for (u=0; u<theLayer->size(); u++) {
                 feature = theLayer->get(u);
-                if ((Pt = dynamic_cast<Node*>(feature))) {
+                if ((Pt = dynamic_cast<TrackNode*>(feature))) {
                     a = time.secsTo(Pt->time().toLocalTime());
                     if (abs(a) < abs(secondsTo)) {
                         secondsTo = a;
@@ -744,7 +791,7 @@ void GeoImageDock::loadImages(QStringList fileNames)
         }
 
         if (progress.wasCanceled()) {
-            theView->invalidate(true, false);
+            theView->invalidate(true, true, false);
             if (photoLayer && !photoLayer->size()) {
                 theDocument->remove(photoLayer);
                 SAFE_DELETE(photoLayer);
@@ -763,7 +810,7 @@ void GeoImageDock::loadImages(QStringList fileNames)
         theDocument->remove(photoLayer);
         SAFE_DELETE(photoLayer);
     }
-    theView->invalidate(true, false);
+    theView->invalidate(true, true, false);
 
 }
 
@@ -852,7 +899,7 @@ Coord GeoImageDock::getGeoDataFromImage(const QString & file)
         if (exifData["Exif.GPSInfo.GPSLongitudeRef"].toString() == "W")
             lon *= -1.0;
     }
-    pos = Coord(angToCoord(lat), angToCoord(lon));
+    pos = Coord(lon, lat);
     return pos;
 }
 
@@ -875,8 +922,8 @@ void GeoImageDock::addGeoDataToImage(Coord position, const QString & file)
     image->readMetadata();
     Exiv2::ExifData &exifData = image->exifData();
 
-    double lat = fabs(coordToAng(position.lat()));
-    double lon = fabs(coordToAng(position.lon()));
+    double lat = fabs(position.y());
+    double lon = fabs(position.x());
     int h, m, s;
 
     QString hourFormat("%1/1 %2/1 %3/100");
@@ -896,12 +943,12 @@ void GeoImageDock::addGeoDataToImage(Coord position, const QString & file)
     exifData["Exif.GPSInfo.GPSVersionID"] = "2 0 0 0";
 
     exifData["Exif.GPSInfo.GPSLatitude"] = vlat;
-    if (position.lat() < 0)
+    if (position.y() < 0)
         exifData["Exif.GPSInfo.GPSLatitudeRef"] = "S";
     else
         exifData["Exif.GPSInfo.GPSLatitudeRef"] = "N";
     exifData["Exif.GPSInfo.GPSLongitude"] = vlon;
-    if (position.lon() < 0)
+    if (position.x() < 0)
         exifData["Exif.GPSInfo.GPSLongitudeRef"] = "W";
     else
         exifData["Exif.GPSInfo.GPSLongitudeRef"] = "E";
