@@ -1,21 +1,23 @@
 #include "PropertiesDock.h"
 #include "InfoDock.h"
 #include "MainWindow.h"
+#ifndef _MOBILE
 #include "ui_MainWindow.h"
+#endif
 #include "MapView.h"
 #include "Interaction.h"
 #include "TagModel.h"
-#include "Utils/EditCompleterDelegate.h"
-#include "Utils/ShortcutOverrideFilter.h"
+#include "EditCompleterDelegate.h"
+#include "ShortcutOverrideFilter.h"
 #include "DocumentCommands.h"
 #include "FeatureCommands.h"
 #include "NodeCommands.h"
 #include "RelationCommands.h"
-#include "Maps/Coord.h"
+#include "Coord.h"
 #include "Document.h"
 #include "Features.h"
-#include "Maps/FeatureManipulations.h"
-#include "TagTemplate/TagTemplate.h"
+#include "FeatureManipulations.h"
+#include "TagTemplate.h"
 
 #ifdef GEOIMAGE
 #include "GeoImageDock.h"
@@ -28,6 +30,7 @@
 #include <QtGui/QTableView>
 #include <QClipboard>
 #include <QMessageBox>
+#include <QMenu>
 
 #include <algorithm>
 
@@ -135,6 +138,9 @@ void PropertiesDock::checkMenuStatus()
     int NumRelation = 0;
     int NumRelationChild = 0;
     int NumAreas = 0;
+    int NumParents = 0;
+    int NumChildren = 0;
+    int NumIncomplete = 0;
     if (Selection.size() == 1)
     {
         IsPoint = CAST_NODE(Selection[0]) != 0;
@@ -149,6 +155,13 @@ void PropertiesDock::checkMenuStatus()
     }
     for (int i=0; i<Selection.size(); ++i)
     {
+        if (Selection[i]->sizeParents())
+            ++NumParents;
+        if (Selection[i]->size())
+            ++NumChildren;
+        if (Selection[i]->notEverythingDownloaded())
+            ++NumIncomplete;
+
         if (CAST_NODE(Selection[i]))
             ++NumPoints;
         if (Way* R = dynamic_cast<Way*>(Selection[i]))
@@ -170,6 +183,7 @@ void PropertiesDock::checkMenuStatus()
         if (!Selection[i]->isUploadable() && !Selection[i]->isSpecial())
             ++NumCommitableFeature;
     }
+#ifndef _MOBILE
     Main->ui->createRelationAction->setEnabled(Selection.size());
     Main->ui->editRemoveAction->setEnabled(Selection.size());
     Main->ui->editMoveAction->setEnabled(true);
@@ -185,7 +199,9 @@ void PropertiesDock::checkMenuStatus()
     Main->ui->areaJoinAction->setEnabled(NumAreas > 1);
     Main->ui->areaSplitAction->setEnabled(NumAreas == 1 && NumPoints == 2 && canSplitArea(this));
     Main->ui->areaTerraceAction->setEnabled(NumAreas == 1 && NumRoads == 0 && canTerraceArea(this));
-    Main->ui->featureSelectChildrenAction->setEnabled(Selection.size());
+    Main->ui->featureSelectChildrenAction->setEnabled(NumChildren);
+    Main->ui->featureSelectParentsAction->setEnabled(NumParents);
+    Main->ui->featureDownloadMissingChildrenAction->setEnabled(NumIncomplete);
     Main->ui->featureDeleteAction->setEnabled((IsPoint || IsRoad || IsRelation) && !Selection[0]->isDirty());
     Main->ui->featureCommitAction->setEnabled(NumCommitableFeature);
     Main->ui->nodeMergeAction->setEnabled(NumPoints > 1);
@@ -194,13 +210,15 @@ void PropertiesDock::checkMenuStatus()
     Main->ui->nodeDetachAction->setEnabled(NumPoints && canDetachNodes(this));
     Main->ui->relationAddMemberAction->setEnabled(NumRelation && Selection.size() > 1);
     Main->ui->relationRemoveMemberAction->setEnabled((NumRelation && Selection.size() > 1 && NumRelationChild) || IsParentRelation);
+    Main->ui->relationAddToMultipolygonAction->setEnabled((NumAreas > 1) || (NumAreas >0 && NumRelation == 1));
     Main->ui->menuOpenStreetBugs->setEnabled(IsOpenStreetBug);
 
     Main->ui->editCopyAction->setEnabled(Selection.size());
     Main->clipboardChanged();
+#endif
 }
 
-int PropertiesDock::size() const
+int PropertiesDock::selectionSize() const
 {
     return Selection.size();
 }
@@ -374,7 +392,7 @@ void PropertiesDock::cleanUpUi()
 void PropertiesDock::switchUi()
 {
     if (CurrentTagView)
-        M_PREFS->setTagListFirstColumnWidth(CurrentTagView->columnWidth(0));
+        M_PREFS->setTagListFirstColumnWidth(qMax(CurrentTagView->columnWidth(0), 20));
 
     if (FullSelection.size() == 0)
         switchToNoUi();
@@ -503,8 +521,8 @@ void PropertiesDock::resetValues()
         if ((Pt) && (NowShowing == TrackPointUiShowing))
         {
             TrackPointUi.Id->setText(QString::number(Pt->id().numId));
-            TrackPointUi.Latitude->setText(COORD2STRING(coordToAng(Pt->position().lat())));
-            TrackPointUi.Longitude->setText(COORD2STRING(coordToAng(Pt->position().lon())));
+            TrackPointUi.Latitude->setText(COORD2STRING(Pt->position().y()));
+            TrackPointUi.Longitude->setText(COORD2STRING(Pt->position().x()));
             TrackPointUi.TagView->setModel(theModel);
             TrackPointUi.TagView->setItemDelegate(delegate);
 
@@ -596,7 +614,7 @@ void PropertiesDock::resetValues()
     /* first column is the width of the default text (Edit this to add...)  */
     /* And the rest of the space is assigned to the second column           */
     if (CurrentTagView) {
-        if (M_PREFS->getTagListFirstColumnWidth() && M_PREFS->getTagListFirstColumnWidth() > 0 && M_PREFS->getTagListFirstColumnWidth() < CurrentTagView->width())
+        if (M_PREFS->getTagListFirstColumnWidth() > 20 && M_PREFS->getTagListFirstColumnWidth() < CurrentTagView->width())
             CurrentTagView->setColumnWidth(
                 0, M_PREFS->getTagListFirstColumnWidth()
             );
@@ -624,7 +642,7 @@ void PropertiesDock::on_TrackPointLat_editingFinished()
     {
         Main->document()->addHistory(
             new MoveNodeCommand(Pt,
-                Coord(angToCoord(TrackPointUi.Latitude->text().toDouble()),Pt->position().lon()), Main->document()->getDirtyOrOriginLayer(Pt->layer()) ));
+                Coord(Pt->position().x(), TrackPointUi.Latitude->text().toDouble()), Main->document()->getDirtyOrOriginLayer(Pt->layer()) ));
         Main->invalidateView(false);
     }
 }
@@ -637,7 +655,7 @@ void PropertiesDock::on_TrackPointLon_editingFinished()
     {
         Main->document()->addHistory(
             new MoveNodeCommand(Pt,
-                Coord(Pt->position().lat(),angToCoord(TrackPointUi.Longitude->text().toDouble())), Main->document()->getDirtyOrOriginLayer(Pt->layer()) ));
+                Coord(TrackPointUi.Longitude->text().toDouble(), Pt->position().y()), Main->document()->getDirtyOrOriginLayer(Pt->layer()) ));
         Main->invalidateView(false);
     }
 }
