@@ -1,5 +1,6 @@
 #include "Global.h"
 
+#include "MainWindow.h"
 #include "Features.h"
 #include "Command.h"
 #include "DocumentCommands.h"
@@ -9,12 +10,12 @@
 #include "NodeCommands.h"
 #include "Document.h"
 #include "Layer.h"
-#include "PaintStyle/MasPaintStyle.h"
-#include "Utils/TagSelector.h"
+#include "MasPaintStyle.h"
+#include "TagSelector.h"
 #include "MapView.h"
 #include "PropertiesDock.h"
 
-#include "Utils/Utils.h"
+#include "Utils.h"
 
 #include <QApplication>
 #include <QUuid>
@@ -100,88 +101,99 @@ void copyTags(Feature* Dest, Feature* Src)
         Dest->setTag(Src->tagKey(i),Src->tagValue(i));
 }
 
-class MapFeaturePrivate
+
+class FeaturePrivate
 {
-    public:
-        MapFeaturePrivate()
-            :  LastActor(Feature::User),
-                PossiblePaintersUpToDate(false),
-                PixelPerMForPainter(-1), CurrentPainter(0), HasPainter(false),
-                theFeature(0), LastPartNotification(0),
-                Time(QDateTime::currentDateTime()), Deleted(false), Visible(true), Uploaded(false), ReadOnly(false), FilterRevision(-1)
-                , Virtual(false), Special(false), DirtyLevel(0)
-                , parentLayer(0)
-        {
-            initVersionNumber();
-        }
-        MapFeaturePrivate(const MapFeaturePrivate& other)
-            : Tags(other.Tags), LastActor(other.LastActor),
-                PossiblePaintersUpToDate(false),
-                PixelPerMForPainter(-1), CurrentPainter(0), HasPainter(false),
-                theFeature(0), LastPartNotification(0),
-                Time(other.Time), Deleted(false), Visible(true), Uploaded(false), ReadOnly(false), FilterRevision(-1)
-                , Virtual(other.Virtual), Special(other.Special), DirtyLevel(0)
-                , parentLayer(0)
-        {
-            initVersionNumber();
-        }
+public:
+    FeaturePrivate(Feature* aFeature)
+        :  LastActor(Feature::User)
+        , PossiblePaintersUpToDate(false)
+        , PixelPerMForPainter(-1), CurrentPainter(0), HasPainter(false)
+        , theFeature(aFeature), LastPartNotification(0)
+        , Deleted(false), Visible(true), Uploaded(false), FilterRevision(-1)
+        , Virtual(false), Special(false), DirtyLevel(0)
+        , parentLayer(0)
+    #ifndef FRISIUS_BUILD
+        , Time(QDateTime::currentDateTime().toTime_t()), User(0xffffffff)
+    #endif
+    {
+#ifndef FRISIUS_BUILD
+        initVersionNumber();
+        //            qDebug() << "MapFeaturePrivate size: " << sizeof(FeaturePrivate) << sizeof(IFeature::FId) << sizeof(RenderPriority);
+#endif
+    }
+    FeaturePrivate(const FeaturePrivate& other)
+        : Tags(other.Tags), LastActor(other.LastActor)
+        , PossiblePaintersUpToDate(false)
+        , PixelPerMForPainter(-1), CurrentPainter(0), HasPainter(false)
+        , theFeature(NULL), LastPartNotification(0)
+        , Deleted(false), Visible(true), Uploaded(false), FilterRevision(-1)
+        , Virtual(other.Virtual), Special(other.Special), DirtyLevel(0)
+        , parentLayer(0)
+    #ifndef FRISIUS_BUILD
+        , Time(other.Time), User(other.User)
+    #endif
+    {
+#ifndef FRISIUS_BUILD
+        initVersionNumber();
+#endif
+    }
 
-        void updatePossiblePainters();
-        void blankPainters();
-        void updatePainters(double PixelPerM);
-        void initVersionNumber();
+    void updatePossiblePainters();
+    void blankPainters();
+    void updatePainters(qreal PixelPerM);
+#ifndef FRISIUS_BUILD
+    void initVersionNumber()
+    {
+        //    static int VN = -1;
+        //    VersionNumber = VN--;
+        VersionNumber = 0;
+    }
+#endif
 
-        mutable IFeature::FId Id;
-        QList<QPair<quint32, quint32> > Tags;
-        Feature::ActorType LastActor;
-        QList<const FeaturePainter*> PossiblePainters;
-        bool PossiblePaintersUpToDate;
-        double PixelPerMForPainter;
-        const FeaturePainter* CurrentPainter;
-        bool HasPainter;
-        Feature* theFeature;
-        QList<Feature*> Parents;
-        int LastPartNotification;
-        QDateTime Time;
-        QString User;
-        int VersionNumber;
-        bool Deleted;
-        bool Visible;
-        bool Uploaded;
-        bool ReadOnly;
-        int FilterRevision;
-        RenderPriority theRenderPriority;
-        bool Virtual;
-        bool Special;
-        int DirtyLevel;
-        QList<FilterLayer*> FilterLayers;
-        double Alpha;
-        Layer* parentLayer;
+    mutable IFeature::FId Id; // 9 (16)
+    QList<QPair<quint32, quint32> > Tags; // 4
+    Feature::ActorType LastActor; // 4
+    QList<const FeaturePainter*> PossiblePainters; // 4
+    bool PossiblePaintersUpToDate; // 1
+    qreal PixelPerMForPainter; // 8
+    const FeaturePainter* CurrentPainter; // 4
+    bool HasPainter; // 1
+    Feature* theFeature; // 4
+    QList<Feature*> Parents; // 4
+    int LastPartNotification; // 4
+#ifndef FRISIUS_BUILD
+    uint Time; // 4
+    quint32 User; // 4
+    int VersionNumber; // 4
+#endif
+    bool Deleted; // 1
+    bool Visible; // 1
+    bool Uploaded; // 1
+    int FilterRevision; // 4
+    bool Virtual; // 1
+    bool Special; // 1
+    int DirtyLevel; // 4
+    QList<FilterLayer*> FilterLayers; // 4
+    qreal Alpha; // 8
+    Layer* parentLayer; // 4
 };
 
-void MapFeaturePrivate::initVersionNumber()
-{
-//    static int VN = -1;
-//    VersionNumber = VN--;
-    VersionNumber = 0;
-}
-
 Feature::Feature()
-: p(0), MetaUpToDate(false), m_references(0)
+: p(0), MetaUpToDate(false), m_references(0), ReadOnly(false)
 {
-     p = new MapFeaturePrivate;
-     p->theFeature = this;
-     p->Id = IFeature::FId(IFeature::Uninitialized, 0);
+    p = new FeaturePrivate(this);
+    p->Id = IFeature::FId(IFeature::Uninitialized, 0);
 
-//     qDebug() << "Feature size: " << sizeof(Feature) << sizeof(MapFeaturePrivate);
+    //     qDebug() << "Feature size: " << sizeof(Feature);
 }
 
 Feature::Feature(const Feature& other)
-: MetaUpToDate(false), m_references(0)
+: IFeature(other), MetaUpToDate(false), m_references(0), ReadOnly(other.ReadOnly)
 {
-    p = new MapFeaturePrivate(*other.p);
-    p->theFeature = this;
+    p = new FeaturePrivate(*other.p);
     p->Id = IFeature::FId(IFeature::Uninitialized, 0);
+    p->theFeature = this;
 }
 
 Feature::~Feature(void)
@@ -193,16 +205,6 @@ Feature::~Feature(void)
     delete p;
 }
 
-void Feature::setVersionNumber(int vn)
-{
-    p->VersionNumber = vn;
-}
-
-int Feature::versionNumber() const
-{
-    return p->VersionNumber;
-}
-
 void Feature::setLayer(Layer* aLayer)
 {
     p->parentLayer = aLayer;
@@ -213,18 +215,6 @@ Layer* Feature::layer() const
     return p->parentLayer;
 }
 
-const RenderPriority& Feature::renderPriority()
-{
-    if (!MetaUpToDate)
-        updateMeta();
-    return p->theRenderPriority;
-}
-
-void Feature::setRenderPriority(const RenderPriority& aPriority)
-{
-    p->theRenderPriority = aPriority;
-}
-
 void Feature::setLastUpdated(Feature::ActorType A)
 {
     p->LastActor = A;
@@ -259,8 +249,6 @@ void Feature::setId(const IFeature::FId& id)
     p->Id = id;
 
     // Make sure any new id is unique
-    if (id.numId < -12000)
-        qDebug("numid");
     if (id.numId < g_feat_rndId)
         g_feat_rndId = id.numId;
 }
@@ -289,26 +277,46 @@ bool Feature::hasOSMId() const
     return (p->Id.numId >= 0);
 }
 
-const QDateTime& Feature::time() const
+#ifndef FRISIUS_BUILD
+
+const QDateTime Feature::time() const
 {
-    return p->Time;
+    return QDateTime::fromTime_t(p->Time);
 }
 
 void Feature::setTime(const QDateTime& time)
 {
-    p->Time = time;
+    p->Time = time.toTime_t();
+}
+
+void Feature::setTime(uint epoch)
+{
+    p->Time = epoch;
 }
 
 const QString& Feature::user() const
 {
-    return p->User;
+    return g_getUser(p->User);
 }
 
 void Feature::setUser(const QString& user)
 {
-    p->User = user;
+    p->User = g_setUser(user);
 }
 
+void Feature::setVersionNumber(int vn)
+{
+    p->VersionNumber = vn;
+}
+
+int Feature::versionNumber() const
+{
+    return p->VersionNumber;
+}
+
+
+#endif
+
 int Feature::incDirtyLevel(int inc)
 {
     return p->DirtyLevel += inc;
@@ -367,28 +375,22 @@ bool Feature::isUploadable() const
 
 void Feature::setReadonly(bool val)
 {
-    p->ReadOnly = val;
+    ReadOnly = val;
 }
 
 bool Feature::isReadonly()
 {
     if (!MetaUpToDate)
         updateMeta();
-    return p->ReadOnly;
+    return ReadOnly;
 }
 
 void Feature::setDeleted(bool delState)
 {
     if (delState == p->Deleted)
         return;
-
-    if (layer()) {
-        if (delState)
-            layer()->indexRemove(boundingBox(false), this);
-        else
-            layer()->indexAdd(boundingBox(), this);
-    }
     p->Deleted = delState;
+    g_backend.sync(this);
 }
 
 bool Feature::isDeleted() const
@@ -421,10 +423,11 @@ void Feature::setVirtual(bool val)
 {
     if (val == p->Virtual)
         return;
-
     p->Virtual = val;
-    if (!p->Virtual)
+    if (!p->Virtual) {
         resetId();
+    }
+    g_backend.sync(this);
 }
 
 bool Feature::isVirtual() const
@@ -437,6 +440,10 @@ void Feature::setSpecial(bool val)
     p->Special = val;
 }
 
+void Feature::buildPath(const Projection &)
+{
+}
+
 bool Feature::isSpecial() const
 {
     return p->Special;
@@ -565,13 +572,17 @@ void Feature::invalidatePainter()
     p->PixelPerMForPainter = -1;
 }
 
+static QPainterPath painterPath;
+
 const QPainterPath& Feature::getPath() const
 {
-    return QPainterPath();
+    return painterPath;
 }
 
-void MapFeaturePrivate::updatePossiblePainters()
+void FeaturePrivate::updatePossiblePainters()
 {
+    QMutexLocker mutlock(&theFeature->featMutex);
+
     //still match features with no tags and no parent, i.e. "lost" trackpoints
     if ( (theFeature->layer()->isTrack()) && M_PREFS->getDisableStyleForTracks() ) return blankPainters();
 
@@ -602,12 +613,12 @@ void MapFeaturePrivate::updatePossiblePainters()
     HasPainter = (PossiblePainters.size() > 0);
 }
 
-void MapFeaturePrivate::updatePainters(double PixelPerM)
+void FeaturePrivate::updatePainters(qreal PixelPerM)
 {
     if (!PossiblePaintersUpToDate)
         updatePossiblePainters();
 
-    CurrentPainter = 0;
+    CurrentPainter = NULL;
     PixelPerMForPainter = PixelPerM;
     for (int i=0; i<PossiblePainters.size(); ++i)
         if (PossiblePainters[i]->matchesZoom(PixelPerM))
@@ -617,15 +628,15 @@ void MapFeaturePrivate::updatePainters(double PixelPerM)
         }
 }
 
-void MapFeaturePrivate::blankPainters()
+void FeaturePrivate::blankPainters()
 {
-    CurrentPainter = 0;
+    CurrentPainter = NULL;
     PossiblePainters.clear();
     PossiblePaintersUpToDate = true;
     HasPainter = false;
 }
 
-const FeaturePainter* Feature::getPainter(double PixelPerM) const
+const FeaturePainter* Feature::getPainter(qreal PixelPerM) const
 {
     if (p->PixelPerMForPainter != PixelPerM)
         p->updatePainters(PixelPerM);
@@ -634,7 +645,13 @@ const FeaturePainter* Feature::getPainter(double PixelPerM) const
 
 const FeaturePainter* Feature::getCurrentPainter() const
 {
-    return p->CurrentPainter;
+    if (p->CurrentPainter)
+        return p->CurrentPainter;
+    else {
+        if (p->PossiblePainters.size())
+            return p->PossiblePainters[0];
+        else return NULL;
+    }
 }
 
 bool Feature::hasPainter() const
@@ -645,8 +662,10 @@ bool Feature::hasPainter() const
     return p->HasPainter;
 }
 
-bool Feature::hasPainter(double PixelPerM) const
+bool Feature::hasPainter(qreal PixelPerM) const
 {
+    if (!layer())
+        return false;
     if (p->PixelPerMForPainter != PixelPerM)
         p->updatePainters(PixelPerM);
     return (p->CurrentPainter != NULL);
@@ -668,17 +687,6 @@ void Feature::unsetParentFeature(Feature* F)
         }
 }
 
-void Feature::updateIndex()
-{
-    if (isDeleted())
-        return;
-
-    if (layer()) {
-        layer()->indexRemove(boundingBox(false), this);
-        layer()->indexAdd(boundingBox(true), this);
-    }
-}
-
 void Feature::updateFilters()
 {
     p->FilterLayers.clear();
@@ -736,12 +744,12 @@ void Feature::updateMeta()
     }
 
     if (L->isReadonly())
-        setReadonly(true);
+        ReadOnly = true;
     else {
-        setReadonly(false);
+        ReadOnly = false;
         foreach(FilterLayer* Fl, p->FilterLayers) {
             if (Fl->isReadonly()) {
-                setReadonly(true);
+                ReadOnly = true;
                 break;
             }
 
@@ -800,7 +808,6 @@ void Feature::drawHover(QPainter& thePainter, MapView* theView)
 void Feature::drawFocus(QPainter& thePainter, MapView* theView)
 {
     QPen TP(M_PREFS->getFocusColor(),M_PREFS->getFocusWidth(),Qt::SolidLine);
-
     thePainter.setPen(TP);
 
     drawSpecial(thePainter, TP, theView);
@@ -818,6 +825,7 @@ void Feature::drawHighlight(QPainter& thePainter, MapView* theView)
 {
     QPen TP(M_PREFS->getHighlightColor(),M_PREFS->getHighlightWidth(),Qt::SolidLine);
     thePainter.setPen(TP);
+
     drawSpecial(thePainter, TP, theView);
 
     drawChildrenSpecial(thePainter, TP, theView, 1);
@@ -831,72 +839,82 @@ void Feature::drawHighlight(QPainter& thePainter, MapView* theView)
 
 QString Feature::toXML(int lvl, QProgressDialog * progress)
 {
-    QDomDocument doc;
-    QDomElement root = doc.createElement("root");
-    doc.appendChild(root);
-    if (toXML(root, progress)) {
-        QString ret;
-        QTextStream ts(&ret);
-        root.firstChild().save(ts, lvl);
-        return ret;
-//        return theXmlDoc.toString(lvl);
+    QString xml;
+    QXmlStreamWriter stream(&xml);
+    stream.setAutoFormatting(true);
+    stream.setAutoFormattingIndent(lvl*2);
+
+    stream.writeStartDocument();
+    stream.writeStartElement("root");
+    if (toXML(stream, progress)) {
+        stream.writeEndElement();
+        stream.writeEndDocument();
+        return xml;
     } else
         return "";
 }
 
-void Feature::fromXML(const QDomElement& e, Feature* F)
+void Feature::fromXML(QXmlStreamReader& stream, Feature* F)
 {
-    bool Deleted = (e.attribute("deleted") == "true");
-    int Dirty = (e.hasAttribute("dirtylevel") ? e.attribute("dirtylevel").toInt() : 0);
-    bool Uploaded = (e.attribute("uploaded") == "true");
-    bool Special = (e.attribute("special") == "true");
-    bool Selected = (e.attribute("selected") == "true");
+    bool Deleted = (stream.attributes().value("deleted") == "true");
+    int Dirty = (stream.attributes().hasAttribute("dirtylevel") ? stream.attributes().value("dirtylevel").toString().toInt() : 0);
+    bool Uploaded = (stream.attributes().value("uploaded") == "true");
+    bool Special = (stream.attributes().value("special") == "true");
+//    bool Selected = (stream.attributes().value("selected") == "true");
 
     QDateTime time;
-    time = QDateTime::fromString(e.attribute("timestamp").left(19), Qt::ISODate);
-    QString user = e.attribute("user");
-    int Version = e.attribute("version").toInt();
+    time = QDateTime::fromString(stream.attributes().value("timestamp").toString().left(19), Qt::ISODate);
+    QString user = stream.attributes().value("user").toString();
+    int Version = stream.attributes().value("version").toString().toInt();
     if (Version < 1)
         Version = 0;
-    Feature::ActorType A = (Feature::ActorType)(e.attribute("actor", "2").toInt());
+    Feature::ActorType A = (Feature::ActorType)(stream.attributes().hasAttribute("actor") ? stream.attributes().value("actor").toString().toInt() : 2);
 
     F->setLastUpdated(A);
     F->setDeleted(Deleted);
     F->setDirtyLevel(Dirty);
     F->setUploaded(Uploaded);
     F->setSpecial(Special);
+#ifndef FRISIUS_BUILD
     F->setTime(time);
     F->setUser(user);
     F->setVersionNumber(Version);
+#endif
 
     // TODO Manage selection at document level
 //    if(Selected)
 //        g_Merk_MainWindow->properties()->addSelection(F);
 }
 
-void Feature::toXML(QDomElement& e, bool strict)
+void Feature::toXML(QXmlStreamWriter& stream, bool strict, QString changetsetid)
 {
-    e.setAttribute("id", xmlId());
-    e.setAttribute("timestamp", time().toString(Qt::ISODate)+"Z");
-    e.setAttribute("version", versionNumber());
-    e.setAttribute("user", user());
+    stream.writeAttribute("id", xmlId());
+#ifndef FRISIUS_BUILD
+    stream.writeAttribute("timestamp", time().toString(Qt::ISODate)+"Z");
+    stream.writeAttribute("version", QString::number(versionNumber()));
+    stream.writeAttribute("user", user());
+#endif
+    if (!changetsetid.isEmpty())
+        stream.writeAttribute("changeset", changetsetid);
     if (!strict) {
-        e.setAttribute("actor", (int)lastUpdated());
+        stream.writeAttribute("actor", QString::number((int)lastUpdated()));
         if (isDeleted())
-            e.setAttribute("deleted","true");
+            stream.writeAttribute("deleted","true");
         if (getDirtyLevel())
-            e.setAttribute("dirtylevel", getDirtyLevel());
+            stream.writeAttribute("dirtylevel", QString::number(getDirtyLevel()));
         if (isUploaded())
-            e.setAttribute("uploaded","true");
+            stream.writeAttribute("uploaded","true");
         if (isSpecial())
-            e.setAttribute("special","true");
+            stream.writeAttribute("special","true");
         // TODO Manage selection at document level
-        if (g_Merk_MainWindow->properties()->isSelected(this))
-            e.setAttribute("selected","true");
+#ifndef _MOBILE
+        if (g_Merk_MainWindow && g_Merk_MainWindow->properties()->isSelected(this))
+            stream.writeAttribute("selected","true");
+#endif
     }
 }
 
-bool Feature::tagsToXML(QDomElement xParent, bool strict)
+bool Feature::tagsToXML(QXmlStreamWriter& stream, bool strict)
 {
     bool OK = true;
 
@@ -907,25 +925,24 @@ bool Feature::tagsToXML(QDomElement xParent, bool strict)
                 continue;
         }
 
-        QDomElement e = xParent.ownerDocument().createElement("tag");
-        xParent.appendChild(e);
-
-        e.setAttribute("k", tagKey(i));
-        e.setAttribute("v", tagValue(i));
+        stream.writeStartElement("tag");
+        stream.writeAttribute("k", tagKey(i));
+        stream.writeAttribute("v", tagValue(i));
+        stream.writeEndElement();
     }
 
     return OK;
 }
 
-void Feature::tagsFromXML(Document* d, Feature * f, QDomElement e)
+void Feature::tagsFromXML(Document* d, Feature * f, QXmlStreamReader& stream)
 {
     Q_UNUSED(d)
-    QDomElement c = e.firstChildElement();
-    while(!c.isNull()) {
-        if (c.tagName() == "tag") {
-            f->setTag(c.attribute("k"),c.attribute("v"));
+    while(!stream.atEnd() && !stream.isEndElement()) {
+        if (stream.name() == "tag") {
+            f->setTag(stream.attributes().value("k").toString(), stream.attributes().value("v").toString());
+            stream.readNext();
         }
-        c = c.nextSiblingElement();
+        stream.readNext();
     }
 }
 
@@ -955,16 +972,31 @@ Relation * Feature::GetSingleParentRelation(Feature * mapFeature)
 }
 
 //Static
-Node* Feature::getTrackPointOrCreatePlaceHolder(Document *theDocument, Layer *theLayer, const IFeature::FId& Id)
+Node* Feature::getNodeOrCreatePlaceHolder(Document *theDocument, Layer *theLayer, const IFeature::FId& Id)
 {
     Node* Part = CAST_NODE(theDocument->getFeature(Id));
     if (!Part)
     {
-        Part = new Node(Coord(0,0));
-        Part->setId(Id);
-        Part->setLastUpdated(Feature::NotYetDownloaded);
         if (!theLayer)
             theLayer = theDocument->getDirtyOrOriginLayer();
+        Part = g_backend.allocNode(theLayer, Coord(0,0));
+        Part->setId(Id);
+        Part->setLastUpdated(Feature::NotYetDownloaded);
+        theLayer->add(Part);
+    }
+    return Part;
+}
+
+TrackNode* Feature::getTrackNodeOrCreatePlaceHolder(Document *theDocument, Layer *theLayer, const IFeature::FId& Id)
+{
+    TrackNode* Part = CAST_TRACKNODE(theDocument->getFeature(Id));
+    if (!Part)
+    {
+        if (!theLayer)
+            theLayer = theDocument->getDirtyOrOriginLayer();
+        Part = g_backend.allocTrackNode(theLayer, Coord(0,0));
+        Part->setId(Id);
+        Part->setLastUpdated(Feature::NotYetDownloaded);
         theLayer->add(Part);
     }
     return Part;
@@ -975,11 +1007,11 @@ Way* Feature::getWayOrCreatePlaceHolder(Document *theDocument, Layer *theLayer,
     Way* Part = CAST_WAY(theDocument->getFeature(Id));
     if (!Part)
     {
-        Part = new Way;
-        Part->setId(Id);
-        Part->setLastUpdated(Feature::NotYetDownloaded);
         if (!theLayer)
             theLayer = theDocument->getDirtyOrOriginLayer();
+        Part = g_backend.allocWay(theLayer);
+        Part->setId(Id);
+        Part->setLastUpdated(Feature::NotYetDownloaded);
         theLayer->add(Part);
 
     }
@@ -991,11 +1023,11 @@ Relation* Feature::getRelationOrCreatePlaceHolder(Document *theDocument, Layer *
     Relation* Part = CAST_RELATION(theDocument->getFeature(Id));
     if (!Part)
     {
-        Part = new Relation;
-        Part->setId(Id);
-        Part->setLastUpdated(Feature::NotYetDownloaded);
         if (!theLayer)
             theLayer = theDocument->getDirtyOrOriginLayer();
+        Part = g_backend.allocRelation(theLayer);
+        Part->setId(Id);
+        Part->setLastUpdated(Feature::NotYetDownloaded);
         theLayer->add(Part);
     }
     return Part;
@@ -1038,12 +1070,13 @@ QString Feature::toMainHtml(QString type, QString systemtype)
     "<br/>"
     "<i>"
     "<small>";
+#ifndef FRISIUS_BUILD
     S += QApplication::translate("MapFeature", "<i>V: </i><b>%1</b> ").arg(QString::number(versionNumber()));
     if (!user().isEmpty())
         S += QApplication::translate("MapFeature", "<i>last: </i><b>%1</b> by <b>%2</b>").arg(time().toString(Qt::SystemLocaleDate)).arg(user());
     else
         S += QApplication::translate("MapFeature", "<i>last: </i><b>%1</b>").arg(time().toString(Qt::SystemLocaleDate));
-
+#endif
     if (layer())
         S += QApplication::translate("MapFeature", "<br/><i>layer: </i><b>%1</b> ").arg(layer()->name());
     S += "</small>"
@@ -1069,18 +1102,8 @@ QString Feature::toMainHtml(QString type, QString systemtype)
 
     if (hasOSMId()) {
         S += "<hr/>"
-        "<a href='/" + systemtype + "/" + xmlId() + "/history'>"+QApplication::translate("MapFeature", "History")+"</a>";
-        if (systemtype == "node") {
-            S += "<br/>"
-            "<a href='/" + systemtype + "/" + xmlId() + "/ways'>"+QApplication::translate("MapFeature", "Referenced by ways")+"</a>";
-        }
-        S += "<br/>"
-        "<a href='/" + systemtype + "/" + xmlId() + "/relations'>"+QApplication::translate("MapFeature", "Referenced by relation")+"</a>";
+        "<a href='/browse/" + systemtype + "/" + xmlId() + "'>"+QApplication::translate("MapFeature", "Browse")+"</a>";
     }
-//	QString bbox("bbox=%1,%2,%3,%4");
-//	bbox = bbox.arg(intToAng(boundingBox().bottomLeft().lon()), 0, 'f').arg(intToAng(boundingBox().bottomLeft().lat()), 0, 'f').arg(intToAng(boundingBox().topRight().lon()), 0, 'f').arg(intToAng(boundingBox().topRight().lat()), 0, 'f');
-//	S += "<br/>"
-//		"<a href='" + "/changesets?" + bbox + "'>"+QApplication::translate("MapFeature", "Changesets in feature bbox")+"</a>";
     S += "</body></html>";
 
     return S;
