@@ -1,10 +1,12 @@
 #include "Features.h"
 #include "MapView.h"
+#include "MapRenderer.h"
 #include "MainWindow.h"
 #include "DocumentCommands.h"
 #include "RelationCommands.h"
 #include "Document.h"
-#include "Utils/LineF.h"
+#include "LineF.h"
+#include "Global.h"
 
 #include <QApplication>
 #include <QAbstractTableModel>
@@ -37,6 +39,8 @@ class RelationPrivate
     public:
         RelationPrivate(Relation* R)
             : theRelation(R), theModel(0), ModelReferences(0)
+            , PathUpToDate(false)
+            , ProjectionRevision(0)
             , BBoxUpToDate(false)
             , Width(0)
         {
@@ -53,13 +57,14 @@ class RelationPrivate
         int ModelReferences;
         QPainterPath thePath;
         QPainterPath theBoundingPath;
+        bool PathUpToDate;
+        int ProjectionRevision;
 
-        mutable CoordBox BBox;
         bool BBoxUpToDate;
 
         RenderPriority theRenderPriority;
 
-        double Width;
+        qreal Width;
     };
 
 #define DEFAULTWIDTH 6
@@ -95,12 +100,13 @@ void RelationPrivate::CalculateWidth()
 
 
 Relation::Relation()
+    : Feature()
 {
     p = new RelationPrivate(this);
 }
 
 Relation::Relation(const Relation& other)
-: Feature(other)
+    : Feature(other)
 {
     p = new RelationPrivate(this);
 }
@@ -134,8 +140,11 @@ void Relation::partChanged(Feature*, int ChangeId)
     if (isDeleted())
         return;
 
+    p->PathUpToDate = false;
     p->BBoxUpToDate = false;
     MetaUpToDate = false;
+    g_backend.sync(this);
+
     notifyParents(ChangeId);
 }
 
@@ -147,40 +156,41 @@ QString Relation::description() const
     return QString("%1").arg(id().numId);
 }
 
-const RenderPriority& Relation::renderPriority()
-{
-    setRenderPriority(p->theRenderPriority);
-    return p->theRenderPriority;
-}
-
 const CoordBox& Relation::boundingBox(bool update) const
 {
     if (!p->BBoxUpToDate && update)
     {
         if (p->Members.size() == 0)
-            p->BBox = CoordBox(Coord(0,0),Coord(0,0));
+            BBox = CoordBox(Coord(0,0),Coord(0,0));
         else
         {
             CoordBox Clip;
             bool haveFirst = false;
-            for (int i=0; i<p->Members.size(); ++i)
-                if (p->Members[i].second && !p->Members[i].second->notEverythingDownloaded()/* && !CAST_RELATION(p->Members[i].second)*/) {
+            for (int i=0; i<p->Members.size(); ++i) {
+                if (p->Members[i].second && !p->Members[i].second->boundingBox().isNull()/* && !CAST_RELATION(p->Members[i].second)*/) {
                     if (!haveFirst) {
                         Clip = p->Members[i].second->boundingBox();
                         haveFirst = true;
                     } else
                         Clip.merge(p->Members[i].second->boundingBox());
                 }
-            p->BBox = Clip;
+            }
+            BBox = Clip;
             p->BBoxUpToDate = true;
         }
     }
-    return p->BBox;
+    return BBox;
 }
 
-void Relation::draw(QPainter& P, MapView* theView)
+void Relation::drawSimple(QPainter &P, MapView *theView)
 {
-    if (!TEST_RFLAGS(RendererOptions::RelationsVisible))
+    Q_UNUSED(P)
+    Q_UNUSED(theView)
+}
+
+void Relation::drawTouchup(QPainter& P, MapView* theView)
+{
+    if (!theView->renderOptions().options.testFlag(RendererOptions::RelationsVisible))
         return;
 
     if (notEverythingDownloaded())
@@ -205,6 +215,7 @@ void Relation::drawSpecial(QPainter& thePainter, QPen& Pen, MapView* theView)
     } */
 
     thePainter.setPen(TP);
+    buildPath(theView->projection());
     thePainter.drawPath(theView->transform().map(p->theBoundingPath));
 }
 
@@ -219,7 +230,7 @@ void Relation::drawParentsSpecial(QPainter& thePainter, QPen& Pen, MapView* theV
     }
 }
 
-void Relation::drawChildrenSpecial(QPainter& thePainter, QPen& Pen, MapView *theView, int depth)
+void Relation::drawChildrenSpecial(QPainter& thePainter, QPen& Pen, MapView* theView, int depth)
 {
     QPen TP(Pen);
     TP.setStyle(Qt::DashLine);
@@ -234,23 +245,25 @@ void Relation::drawChildrenSpecial(QPainter& thePainter, QPen& Pen, MapView *the
 }
 
 
-double Relation::pixelDistance(const QPointF& Target, double ClearEndDistance, bool, MapView* theView) const
+qreal Relation::pixelDistance(const QPointF& Target, qreal ClearEndDistance, const QList<Feature*>& NoSnap, MapView* theView) const
 {
-    double Best = 1000000;
+    Q_UNUSED(NoSnap)
+
+    qreal Best = 1000000;
     if (!TEST_RFLAGS(RendererOptions::RelationsVisible) && !M_PREFS->getRelationsSelectableWhenHidden())
         return Best;
 
     //for (int i=0; i<p->Members.size(); ++i)
     //{
     //	if (p->Members[i].second) {
-    //		double Dist = p->Members[i].second->pixelDistance(Target, ClearEndDistance, theProjection);
+    //		qreal Dist = p->Members[i].second->pixelDistance(Target, ClearEndDistance, theProjection);
     //		if (Dist < Best)
     //			Best = Dist;
     //	}
     //}
 
 
-    double D;
+    qreal D;
     //QRectF bb = QRectF(theView->toView(boundingBox().bottomLeft()),theView->toView(boundingBox().topRight()));
     //bb.adjust(-10, -10, 10, 10);
 
@@ -275,24 +288,25 @@ void Relation::cascadedRemoveIfUsing(Document* theDocument, Feature* aFeature, C
     for (int i=0; i<p->Members.size();) {
         if (p->Members[i].second && p->Members[i].second == aFeature)
         {
-            if ( (p->Members.size() == 1) && (Alternatives.size() == 0) )
-                theList->add(new RemoveFeatureCommand(theDocument,this));
-            else
-            {
-                QString Role = p->Members[i].first;
-                theList->add(new RelationRemoveFeatureCommand(this, i, theDocument->getDirtyOrOriginLayer(layer())));
-                for (int j=0; j<Alternatives.size(); ++j)
-                    if (p->Members[i+j].second != Alternatives[j]) {
-                        if ((i+j) == 0)
-                            theList->add(new RelationAddFeatureCommand(this, Role, Alternatives[j], 0, theDocument->getDirtyOrOriginLayer(Alternatives[j]->layer())));
-                        else if (p->Members[i+j-1].second != Alternatives[j])
-                            theList->add(new RelationAddFeatureCommand(this, Role, Alternatives[j], i+j, theDocument->getDirtyOrOriginLayer(Alternatives[j]->layer())));
-                    }
-                continue;
-            }
+            QString Role = p->Members[i].first;
+            theList->add(new RelationRemoveFeatureCommand(this, i, theDocument->getDirtyOrOriginLayer(layer())));
+            for (int j=0; j<Alternatives.size(); ++j)
+                if (i+j >= p->Members.size() || p->Members[i+j].second != Alternatives[j]) {
+                    if ((i+j) == 0)
+                        theList->add(new RelationAddFeatureCommand(this, Role, Alternatives[j], 0, theDocument->getDirtyOrOriginLayer(Alternatives[j]->layer())));
+                    else if (p->Members[i+j-1].second != Alternatives[j])
+                        theList->add(new RelationAddFeatureCommand(this, Role, Alternatives[j], i+j, theDocument->getDirtyOrOriginLayer(Alternatives[j]->layer())));
+                }
+            continue;
         }
         ++i;
     }
+    if (p->Members.size() == 0) {
+        if (!isDeleted()) {
+            QList<Feature*> alt;
+            theList->add(new RemoveFeatureCommand(theDocument,this,alt));
+        }
+    }
 }
 
 bool Relation::notEverythingDownloaded()
@@ -309,52 +323,40 @@ bool Relation::notEverythingDownloaded()
 
 void Relation::add(const QString& Role, Feature* F)
 {
-    if (layer())
-        layer()->indexRemove(p->BBox, this);
     p->Members.push_back(qMakePair(Role,F));
     F->setParentFeature(this);
+    p->PathUpToDate = false;
     p->BBoxUpToDate = false;
     MetaUpToDate = false;
-    if (layer() && !isDeleted()) {
-        CoordBox bb = boundingBox();
-        layer()->indexAdd(boundingBox(), this);
-    }
+    g_backend.sync(this);
 
     notifyChanges();
 }
 
 void Relation::add(const QString& Role, Feature* F, int Idx)
 {
-    if (layer())
-        layer()->indexRemove(p->BBox, this);
     p->Members.push_back(qMakePair(Role,F));
     std::rotate(p->Members.begin()+Idx,p->Members.end()-1,p->Members.end());
     F->setParentFeature(this);
+    p->PathUpToDate = false;
     p->BBoxUpToDate = false;
     MetaUpToDate = false;
-    if (layer() && !isDeleted()) {
-        CoordBox bb = boundingBox();
-        layer()->indexAdd(bb, this);
-    }
+    g_backend.sync(this);
 
     notifyChanges();
 }
 
 void Relation::remove(int Idx)
 {
-    if (layer())
-        layer()->indexRemove(p->BBox, this);
     Feature* F = p->Members[Idx].second;
     // only remove as parent if the feature is only a member once
     p->Members.erase(p->Members.begin()+Idx);
     if (F && find(F) == p->Members.size())
         F->unsetParentFeature(this);
+    p->PathUpToDate = false;
     p->BBoxUpToDate = false;
     MetaUpToDate = false;
-    if (layer() && !isDeleted()) {
-        CoordBox bb = boundingBox();
-        layer()->indexAdd(bb, this);
-    }
+    g_backend.sync(this);
 
     notifyChanges();
 }
@@ -364,8 +366,6 @@ void Relation::remove(Feature* F)
     for (int i=p->Members.size(); i; --i)
         if (F == p->Members[i-1].second)
             remove(i-1);
-    p->BBoxUpToDate = false;
-    MetaUpToDate = false;
 }
 
 int Relation::size() const
@@ -419,11 +419,12 @@ void Relation::releaseMemberModel()
     }
 }
 
-void Relation::buildPath(Projection const &theProjection, const QTransform& /*theTransform*/, const QRectF& cr)
+void Relation::buildPath(Projection const &theProjection)
 {
-    QPainterPath clipPath;
-    clipPath.addRect(cr);
+//    QPainterPath clipPath;
+//    clipPath.addRect(cr);
 
+    QMutexLocker mutlock(&featMutex);
     p->theBoundingPath = QPainterPath();
 
     if (!p->Members.size())
@@ -440,137 +441,259 @@ void Relation::buildPath(Projection const &theProjection, const QTransform& /*th
     //p->theBoundingPath.addRect(bb);
 
     p->theBoundingPath.addPolygon(theVector);
+//    p->theBoundingPath = p->theBoundingPath.intersected(clipPath);
 
-    p->theBoundingPath = p->theBoundingPath.intersected(clipPath);
+    if (!p->PathUpToDate || p->ProjectionRevision != theProjection.projectionRevision()) {
+        p->thePath = QPainterPath();
+
+        Way* outerWay = NULL;
+        int numOuter = 0;
+        bool isMultipolygon = false;
+        if (tagValue("type", "") == "multipolygon")
+            isMultipolygon = true;
+
+
+        // Handle polygons made of scattered ways
+        QList< QPair<QString,QPainterPath> > memberPaths;
+        for (int i=0; i<size(); ++i) {
+            if (CHECK_WAY(p->Members[i].second)) {
+                Way* M = STATIC_CAST_WAY(p->Members[i].second);
+                M->buildPath(theProjection);
+                if (M->getPath().elementCount() > 1) {
+                    memberPaths << qMakePair(p->Members[i].first, M->getPath());
+                    if (isMultipolygon && (p->Members[i].first == "outer" || p->Members[i].first.isEmpty())) {
+                        if (!numOuter)
+                            outerWay = M;
+                        else
+                            outerWay = NULL;
+                        ++numOuter;
+                    }
+                }
+            }
+        }
+
+        QList<QPainterPath> innerPaths;
+        QList<QPainterPath> outerPaths;
+
+        while (memberPaths.size()) {
+            // handle the start...
+            QPointF curPoint;
+            QPainterPath curPath;
+            QString curRole;
+
+            curRole = memberPaths[0].first;
+            curPath.moveTo(memberPaths[0].second.elementAt(0));
+            for (int j=1; j<memberPaths[0].second.elementCount(); ++j) {
+                curPoint = memberPaths[0].second.elementAt(j);
+                curPath.lineTo(curPoint);
+            }
+            // ... and remove it
+            memberPaths.removeAt(0);
+            // Check if any remaining path starts or ends at the current point
+            for (int k=0; k<memberPaths.size(); ++k) {
+                if (memberPaths[k].second.elementAt(0) == curPoint && memberPaths[k].first == curRole) { // Check start
+                    for (int l=1; l<memberPaths[k].second.elementCount(); ++l) {
+                        curPoint = memberPaths[k].second.elementAt(l);
+                        curPath.lineTo(curPoint);
+                    }
+                    memberPaths.removeAt(k);
+                    k=0;
+                } else if (memberPaths[k].second.elementAt(memberPaths[k].second.elementCount()-1) == curPoint  && memberPaths[k].first == curRole) { // Check end
+                    for (int l=memberPaths[k].second.elementCount()-2; l>=0; --l) {
+                        curPoint = memberPaths[k].second.elementAt(l);
+                        curPath.lineTo(curPoint);
+                    }
+                    memberPaths.removeAt(k);
+                    k=0;
+                }
+            }
+            if (curRole == "inner" and isMultipolygon)
+                innerPaths << curPath;
+            else
+                outerPaths << curPath;
+        }
+
+        if (outerWay && tagSize() == 1) {
+            outerWay->rebuildPath(theProjection);
+            for (int i=0; i<innerPaths.size(); ++i) {
+                outerWay->addPathHole(innerPaths[i]);
+            }
+        } else {
+            for (int i=0; i<outerPaths.size(); ++i) {
+                p->thePath.addPath(outerPaths[i]);
+            }
+            for (int i=0; i<innerPaths.size(); ++i) {
+                p->thePath = p->thePath.subtracted(innerPaths[i]);
+            }
+        }
+
+        p->ProjectionRevision = theProjection.projectionRevision();
+        p->PathUpToDate = true;
+    }
 }
 
 const QPainterPath& Relation::getPath() const
 {
-    p->thePath = QPainterPath();
-    for (int i=0; i<size(); ++i)
-        if (Way* M = dynamic_cast<Way*>(p->Members[i].second)) {
-            p->thePath.addPath(M->getPath());
-        }
     return p->thePath;
 }
 
+const RenderPriority& Relation::renderPriority()
+{
+    if (!MetaUpToDate)
+        updateMeta();
+    return p->theRenderPriority;
+}
+
 void Relation::updateMeta()
 {
-    Feature::updateMeta();
-    MetaUpToDate = true;
+    QMutexLocker mutlock(&featMutex);
+    if (MetaUpToDate)
+        return;
 
+    Feature::updateMeta();
+
+    p->PathUpToDate = false;
     p->CalculateWidth();
 
     p->theRenderPriority = RenderPriority(RenderPriority::IsSingular, 0., 0);
     for (int i=0; i<p->Members.size(); ++i) {
-        if (p->Members.at(i).second->renderPriority() < p->theRenderPriority)
-            p->theRenderPriority = p->Members.at(i).second->renderPriority();
+        if (Way* W = CAST_WAY(p->Members.at(i).second)) {
+            if (W->renderPriority() < p->theRenderPriority)
+                p->theRenderPriority = W->renderPriority();
+        } else if (Relation* R = CAST_RELATION(p->Members.at(i).second)) {
+            if (R->renderPriority() < p->theRenderPriority)
+                p->theRenderPriority = R->renderPriority();
+        }
     }
+    MetaUpToDate = true;
 }
 
-bool Relation::toXML(QDomElement xParent, QProgressDialog * progress, bool strict)
+bool Relation::toXML(QXmlStreamWriter& stream, QProgressDialog * progress, bool strict, QString changetsetid)
 {
     bool OK = true;
 
-    QDomElement e = xParent.ownerDocument().createElement("relation");
-    xParent.appendChild(e);
+    stream.writeStartElement("relation");
+    Feature::toXML(stream, strict, changetsetid);
 
-    Feature::toXML(e, strict);
+    // Has to be first to be picked up when reading back
+    if (!strict)
+        boundingBox().toXML("BoundingBox", stream);
 
     for (int i=0; i<size(); ++i) {
         QString Type("node");
-        if (dynamic_cast<const Way*>(get(i)))
+        if (CHECK_WAY(get(i)))
             Type="way";
-        else if (dynamic_cast<const Relation*>(get(i)))
+        else if (CHECK_RELATION(get(i)))
             Type="relation";
 
-        QDomElement n = xParent.ownerDocument().createElement("member");
-        e.appendChild(n);
-
-        n.setAttribute("type", Type);
-        n.setAttribute("ref", get(i)->xmlId());
-        n.setAttribute("role", getRole(i));
+        stream.writeStartElement("member");
+        stream.writeAttribute("type", Type);
+        stream.writeAttribute("ref", get(i)->xmlId());
+        stream.writeAttribute("role", getRole(i));
+        stream.writeEndElement();
     }
 
-    tagsToXML(e, strict);
+    tagsToXML(stream, strict);
+
+    stream.writeEndElement();
 
     if (progress)
         progress->setValue(progress->value()+1);
     return OK;
 }
 
-Relation * Relation::fromXML(Document * d, Layer * L, const QDomElement e)
+Relation * Relation::fromXML(Document * d, Layer * L, QXmlStreamReader& stream)
 {
-    QString sid = (e.hasAttribute("id") ? e.attribute("id") : e.attribute("xml:id"));
+    bool hasBbox = false;
+
+    QString sid = (stream.attributes().hasAttribute("id") ? stream.attributes().value("id").toString() : stream.attributes().value("xml:id").toString());
     IFeature::FId id;
     id.type = IFeature::OsmRelation;
     id.numId = sid.toLongLong();
 
     Relation* R = CAST_RELATION(d->getFeature(id));
     if (!R) {
-        R = new Relation;
+        R = g_backend.allocRelation(L);
         R->setId(id);
-        Feature::fromXML(e, R);
+        L->add(R);
+        Feature::fromXML(stream, R);
     } else {
-        Feature::fromXML(e, R);
-        R->layer()->remove(R);
+        Feature::fromXML(stream, R);
+        if (R->layer() != L) {
+            R->layer()->remove(R);
+            L->add(R);
+        }
         while (R->p->Members.size())
             R->remove(0);
     }
 
-    QDomElement c = e.firstChildElement();
-    while(!c.isNull()) {
-        if (c.tagName() == "member") {
-            QString Type = c.attribute("type");
+    stream.readNext();
+    while(!stream.atEnd() && !stream.isEndElement()) {
+        if (stream.name() == "member") {
+            QString Type = stream.attributes().value("type").toString();
+            QString sId = stream.attributes().value("ref").toString();
+            QString role = stream.attributes().value("role").toString();
             Feature* F = 0;
             if (Type == "node") {
-                QString sId = c.attribute("ref");
                 IFeature::FId nId(IFeature::Point, sId.toLongLong());
                 Node* Part = CAST_NODE(d->getFeature(nId));
                 if (!Part)
                 {
-                    Part = new Node(Coord(0,0));
+                    Part = g_backend.allocNode(L, Coord(0,0));
                     Part->setId(nId);
                     Part->setLastUpdated(Feature::NotYetDownloaded);
                     L->add(Part);
                 }
                 F = Part;
             } else if (Type == "way") {
-                QString sId = c.attribute("ref");
                 IFeature::FId rId(IFeature::LineString, sId.toLongLong());
                 Way* Part = CAST_WAY(d->getFeature(rId));
                 if (!Part)
                 {
-                    Part = new Way;
+                    Part = g_backend.allocWay(L);
                     Part->setId(rId);
                     Part->setLastUpdated(Feature::NotYetDownloaded);
                     L->add(Part);
                 }
                 F = Part;
             } else if (Type == "relation") {
-                QString sId = c.attribute("ref");
                 IFeature::FId RId(IFeature::OsmRelation, sId.toLongLong());
                 Relation* Part = dynamic_cast<Relation*>(d->getFeature(RId));
                 if (!Part)
                 {
-                    Part = new Relation;
+                    Part = g_backend.allocRelation(L);
                     Part->setId(RId);
                     Part->setLastUpdated(Feature::NotYetDownloaded);
                     L->add(Part);
                 }
                 F = Part;
             }
-            if (F)
-            {
-                R->add(c.attribute("role"),F);
+            if (F) {
+                if (!hasBbox) {
+                    R->add(role, F);
+                } else {
+                    R->p->Members.push_back(qMakePair(role,F));
+                    F->setParentFeature(R);
+                }
             }
+            stream.readNext();
+        } else if (stream.name() == "tag") {
+            R->setTag(stream.attributes().value("k").toString(), stream.attributes().value("v").toString());
+            stream.readNext();
+        } else if (stream.name() == "BoundingBox") {
+            R->BBox = CoordBox::fromXML(stream);
+            R->p->BBoxUpToDate = true;
+            hasBbox = true;
+            stream.readNext();
+        } else if (!stream.isWhitespace()) {
+            qDebug() << "Relation: logic error: " << stream.name() << " : " << stream.tokenType() << " (" << stream.lineNumber() << ")";
+            stream.skipCurrentElement();
         }
-        c = c.nextSiblingElement();
+        stream.readNext();
     }
 
-    L->add(R);
-    Feature::tagsFromXML(d, R, e);
-
+    if (hasBbox && !R->isDeleted())
+        g_backend.indexAdd(L, R->BBox, R);
     return R;
 }
 
@@ -581,110 +704,14 @@ QString Relation::toHtml()
     D += "<i>"+QApplication::translate("MapFeature", "size")+": </i>" + QString::number(size()) + " " + QApplication::translate("MapFeature", "members");
     CoordBox bb = boundingBox();
     D += "<br/>";
-    D += "<i>"+QApplication::translate("MapFeature", "Topleft")+": </i>" + COORD2STRING(coordToAng(bb.topLeft().lat())) + " / " + COORD2STRING(coordToAng(bb.topLeft().lon()));
+    D += "<i>"+QApplication::translate("MapFeature", "Topleft")+": </i>" + COORD2STRING(bb.topLeft().y()) + " / " + COORD2STRING(bb.topLeft().x());
     D += "<br/>";
-    D += "<i>"+QApplication::translate("MapFeature", "Botright")+": </i>" + COORD2STRING(coordToAng(bb.bottomRight().lat())) + " / " + COORD2STRING(coordToAng(bb.bottomRight().lon()));
+    D += "<i>"+QApplication::translate("MapFeature", "Botright")+": </i>" + COORD2STRING(bb.bottomRight().y()) + " / " + COORD2STRING(bb.bottomRight().x());
 
     return Feature::toMainHtml(QApplication::translate("MapFeature", "Relation"),"relation").arg(D);
 }
 
-void Relation::toBinary(QDataStream& ds, QHash <QString, quint64>& theIndex)
-{
-    quint8 Type = '\0';
-    quint64 ref;
-
-    theIndex["L" + id().numId] = ds.device()->pos();
-    ds << (qint8)'L';
-    ds << id().numId;
-    ds << size();
-    for (int i=0; i<size(); ++i) {
-        if (dynamic_cast<const Node*>(get(i))) {
-            Type='N';
-            ref = get(i)->id().numId;
-        }
-        else if (dynamic_cast<const Way*>(get(i))) {
-            Type='R';
-            ref = get(i)->id().numId;
-        }
-        else if (dynamic_cast<const Relation*>(get(i))) {
-            Type='L';
-            ref = get(i)->id().numId;
-        }
-//		ds << (qint8) Type << ref << getRole(i);
-        ds << Type << get(i)->id().numId << getRole(i);
-    }
-}
-
-Relation* Relation::fromBinary(Document* d, OsbLayer* L, QDataStream& ds, qint8 c, qint64 id)
-{
-    Q_UNUSED(c);
-
-    qint32	fSize;
-    quint8 Type;
-    qint64 refId;
-    QString Role;
-
-    ds >> fSize;
-
-    if (!L) {
-        for (int i=0; i < fSize; ++i) {
-            ds >> Type;
-            ds >> refId;
-            ds >> Role;
-        }
-        return NULL;
-    }
-
-    IFeature::FId strId(IFeature::OsmRelation, id);
-    Relation* R = CAST_RELATION(d->getFeature(strId));
-    if (!R) {
-        R = new Relation();
-        R->setId(strId);
-        R->setLastUpdated(Feature::OSMServer);
-    } else {
-        if (R->lastUpdated() == Feature::NotYetDownloaded) {
-            R->setLastUpdated(Feature::OSMServer);
-            L->remove(R);
-        } else  {
-            for (int i=0; i < fSize; ++i) {
-                ds >> Type;
-                ds >> refId;
-                ds >> Role;
-            }
-            return R;
-        }
-    }
-
-    for (int i=0; i < fSize; ++i) {
-        ds >> Type;
-        ds >> refId;
-        ds >> Role;
-
-        //MapFeature* F = d->getFeature(QString::number(refId), false);
-        Feature* F;
-        switch (Type) {
-            case 'N':
-                F = d->getFeature(IFeature::FId(IFeature::Point, refId));
-                break;
-            case 'R':
-                F = d->getFeature(IFeature::FId(IFeature::LineString, refId));
-                break;
-            case 'L':
-                F = d->getFeature(IFeature::FId(IFeature::OsmRelation, refId));
-                break;
-        default:
-                F = NULL;
-        break;
-        }
-        if (F)
-            R->add(Role, F);
-    }
-    L->add(R);
-
-    return R;
-}
-
-double Relation::widthOf()
+qreal Relation::widthOf()
 {
     if (MetaUpToDate == false)
         updateMeta();
