@@ -12,7 +12,8 @@
 #ifndef ImportExportGDAL_H
 #define ImportExportGDAL_H
 
-#include <ImportExport/IImportExport.h>
+#include "IImportExport.h"
+#include "ogrsf_frmts.h"
 
 class Projection;
 class Layer;
@@ -38,19 +39,21 @@ public:
     virtual bool saveFile(QString filename);
     // import the  input
     virtual bool import(Layer* aLayer);
+    bool import(Layer *aLayer, const QByteArray &ba, bool confirmProjection);
 
     //export
     virtual bool export_(const QList<Feature *>& featList);
 
 protected:
     OGRCoordinateTransformation *toWGS84;
-    Projection* theProjection;
 
     Feature* parseGeometry(Layer* aLayer, OGRGeometry *poGeometry);
 
-    Node *nodeFor(OGRPoint point);
+    Node *nodeFor(Layer* aLayer, OGRPoint point);
     Way *readWay(Layer* aLayer, OGRLineString *poRing);
 
+    bool importGDALDataset(OGRDataSource *poDs, Layer *aLayer, bool confirmProjection);
+
 private:
     QHash<OGRPoint, Node*> pointHash;
 };
