@@ -5,6 +5,7 @@
 #include "Document.h"
 #include "Node.h"
 #include "TrackSegment.h"
+#include "Global.h"
 
 #include <QtCore/QBuffer>
 #include <QtCore/QDateTime>
@@ -14,12 +15,12 @@
 #include <QProgressDialog>
 
 
-static Node* importTrkPt(const QDomElement& Root, Document* /* theDocument */, Layer* theLayer)
+static TrackNode* importTrkPt(const QDomElement& Root, Document* /* theDocument */, Layer* theLayer)
 {
-    double Lat = Root.attribute("lat").toDouble();
-    double Lon = Root.attribute("lon").toDouble();
+    qreal Lat = Root.attribute("lat").toDouble();
+    qreal Lon = Root.attribute("lon").toDouble();
 
-    Node* Pt = new Node(Coord(angToCoord(Lat),angToCoord(Lon)));
+    TrackNode* Pt = g_backend.allocTrackNode(theLayer, Coord(Lon,Lat));
     Pt->setLastUpdated(Feature::Log);
     if (Root.hasAttribute("xml:id"))
         Pt->setId(IFeature::FId(IFeature::Point, Root.attribute("xml:id").toLongLong()));
@@ -90,7 +91,8 @@ static Node* importTrkPt(const QDomElement& Root, Document* /* theDocument */, L
 
 static void importTrkSeg(const QDomElement& Root, Document* theDocument, Layer* theLayer, bool MakeSegment, QProgressDialog & progress)
 {
-    TrackSegment* S = new TrackSegment;
+    TrackSegment* S = g_backend.allocSegment(theLayer);
+    theLayer->add(S);
 
     if (Root.hasAttribute("xml:id"))
         S->setId(IFeature::FId(IFeature::GpxSegment, Root.attribute("xml:id").toLongLong()));
@@ -107,23 +109,22 @@ static void importTrkSeg(const QDomElement& Root, Document* theDocument, Layer*
         if (progress.wasCanceled())
             return;
 
-        Node* Pt = importTrkPt(t,theDocument, theLayer);
+        TrackNode* Pt = importTrkPt(t,theDocument, theLayer);
 
         if (MakeSegment == false)
             continue;
 
         if (lastPoint)
         {
-            double kilometer = Pt->position().distanceFrom( lastPoint->position() );
+            qreal kilometer = Pt->position().distanceFrom( lastPoint->position() );
 
             if (M_PREFS->getMaxDistNodes() != 0.0 && kilometer > M_PREFS->getMaxDistNodes())
             {
-                if (S->size())
-                    theLayer->add(S);
-                else
-                    delete S;
+                if (!S->size())
+                    g_backend.deallocFeature(theLayer, S);
 
-                S = new TrackSegment;
+                S = g_backend.allocSegment(theLayer);
+                theLayer->add(S);
             }
         }
 
@@ -131,20 +132,19 @@ static void importTrkSeg(const QDomElement& Root, Document* theDocument, Layer*
         lastPoint = Pt;
     }
 
-    if (S->size())
-        theLayer->add(S);
-    else
-        delete S;
+    if (!S->size())
+        g_backend.deallocFeature(theLayer, S);
 }
 
 static void importRte(const QDomElement& Root, Document* theDocument, Layer* theLayer, bool MakeSegment, QProgressDialog & progress)
 {
-    TrackSegment* S = new TrackSegment;
+    TrackSegment* S = g_backend.allocSegment(theLayer);
+    theLayer->add(S);
 
     if (Root.hasAttribute("xml:id"))
         S->setId(IFeature::FId(IFeature::GpxSegment, Root.attribute("xml:id").toLongLong()));
 
-    Node* lastPoint = NULL;
+    TrackNode* lastPoint = NULL;
 
     for(QDomNode n = Root.firstChild(); !n.isNull(); n = n.nextSibling())
     {
@@ -161,23 +161,21 @@ static void importRte(const QDomElement& Root, Document* theDocument, Layer* the
             if (progress.wasCanceled())
                 return;
 
-            Node* Pt = importTrkPt(t,theDocument, theLayer);
+            TrackNode* Pt = importTrkPt(t,theDocument, theLayer);
 
             if (MakeSegment == false)
                 continue;
 
             if (lastPoint)
             {
-                double kilometer = Pt->position().distanceFrom( lastPoint->position() );
+                qreal kilometer = Pt->position().distanceFrom( lastPoint->position() );
 
                 if (M_PREFS->getMaxDistNodes() != 0.0 && kilometer > M_PREFS->getMaxDistNodes())
                 {
-                    if (S->size())
-                        theLayer->add(S);
-                    else
-                        delete S;
+                    if (!S->size())
+                        g_backend.deallocFeature(theLayer, S);
 
-                    S = new TrackSegment;
+                    S = g_backend.allocSegment(theLayer);
                 }
             }
             S->add(Pt);
@@ -185,10 +183,8 @@ static void importRte(const QDomElement& Root, Document* theDocument, Layer* the
         }
     }
 
-    if (S->size())
-        theLayer->add(S);
-    else
-        delete S;
+    if (!S->size())
+        g_backend.deallocFeature(theLayer, S);
 }
 
 static void importTrk(const QDomElement& Root, Document* theDocument, Layer* theLayer, bool MakeSegment, QProgressDialog & progress)
@@ -222,14 +218,11 @@ static void importGPX(const QDomElement& Root, Document* theDocument, QList<Trac
         {
             TrackLayer* newLayer = new TrackLayer();
             theDocument->add(newLayer);
-            newLayer->blockIndexing(true);
             importTrk(t,theDocument, newLayer, MakeSegment, progress);
             if (!newLayer->size()) {
                 theDocument->remove(newLayer);
                 delete newLayer;
             } else {
-                newLayer->blockIndexing(false);
-                newLayer->reIndex();
                 theTracklayers.append(newLayer);
             }
         }
@@ -237,14 +230,11 @@ static void importGPX(const QDomElement& Root, Document* theDocument, QList<Trac
         {
             TrackLayer* newLayer = new TrackLayer();
             theDocument->add(newLayer);
-            newLayer->blockIndexing(true);
             importRte(t,theDocument, newLayer, MakeSegment, progress);
             if (!newLayer->size()) {
                 theDocument->remove(newLayer);
                 delete newLayer;
             } else {
-                newLayer->blockIndexing(false);
-                newLayer->reIndex();
                 theTracklayers.append(newLayer);
             }
         }
