@@ -13,6 +13,7 @@
 #include <QtGui>
 
 #include "../ImportExport/ImportNMEA.h"
+#include "Global.h"
 
 
 ImportNMEA::ImportNMEA(Document* doc)
@@ -41,9 +42,9 @@ bool ImportNMEA::import(Layer* aLayer)
     QTextStream in(Device);
 
     theLayer = dynamic_cast <TrackLayer *> (aLayer);
-    theList = new CommandList(MainWindow::tr("Import NMEA"), NULL);
+    theList = new CommandList(QApplication::tr("Import NMEA"), NULL);
 
-    TrackSegment* TS = new TrackSegment;
+    TrackSegment* TS = g_backend.allocSegment(aLayer);
 
     while (!in.atEnd()) {
         QString line = in.readLine();
@@ -59,8 +60,8 @@ bool ImportNMEA::import(Layer* aLayer)
                 if (TS->size())
                     theLayer->add(TS);
                 else
-                    delete TS;
-                TS = new TrackSegment;
+                    g_backend.deallocFeature(aLayer, TS);
+                TS = g_backend.allocSegment(aLayer);
             }
         } else
         if (command == "GSV") {
@@ -74,8 +75,8 @@ bool ImportNMEA::import(Layer* aLayer)
                 if (TS->size())
                     theLayer->add(TS);
                 else
-                    delete TS;
-                TS = new TrackSegment;
+                    g_backend.deallocFeature(aLayer, TS);
+                TS = g_backend.allocSegment(aLayer);
             }
         } else
         if (command == "GLL") {
@@ -85,13 +86,13 @@ bool ImportNMEA::import(Layer* aLayer)
                 if (TS->size())
                     theLayer->add(TS);
                 else
-                    delete TS;
-                TS = new TrackSegment;
+                    g_backend.deallocFeature(aLayer, TS);
+                TS = g_backend.allocSegment(aLayer);
             }
         } else
         if (command == "RMC") {
             if (goodFix && goodFix3D) {
-                Node* p = importRMC(line);
+                TrackNode* p = importRMC(line);
                 if (p)
                     TS->add(p);
             }
@@ -102,7 +103,7 @@ bool ImportNMEA::import(Layer* aLayer)
     if (TS->size())
         theLayer->add(TS);
     else
-        delete TS;
+        g_backend.deallocFeature(aLayer, TS);
 
     delete theList;
     return true;
@@ -120,9 +121,9 @@ bool ImportNMEA::importGSA (QString line)
     QString autoSelectFix = tokens[1];
     int Fix3D = tokens[2].toInt();
 
-    // double PDOP = tokens[15].toDouble();
-    // double HDOP = tokens[16].toDouble();
-    // double VDOP = tokens[17].toDouble();
+    // qreal PDOP = tokens[15].toDouble();
+    // qreal HDOP = tokens[16].toDouble();
+    // qreal VDOP = tokens[17].toDouble();
 
     return (Fix3D == 1 ? false: true);
 }
@@ -142,14 +143,14 @@ bool ImportNMEA::importGGA (QString line)
     if (tokens.size() < 10)
         return false;
 
-    //double lat = tokens[2].left(2).toDouble();
-    //double lon = tokens[4].left(3).toDouble();
+    //qreal lat = tokens[2].left(2).toDouble();
+    //qreal lon = tokens[4].left(3).toDouble();
 
-    //double latmin = tokens[2].mid(2).toDouble();
+    //qreal latmin = tokens[2].mid(2).toDouble();
     //lat += latmin / 60.0;
     //if (tokens[3] != "N")
     //	lat = -lat;
-    //double lonmin = tokens[4].mid(3).toDouble();
+    //qreal lonmin = tokens[4].mid(3).toDouble();
     //lon += lonmin / 60.0;
     //if (tokens[5] != "E")
     //	lon = -lon;
@@ -179,7 +180,7 @@ bool ImportNMEA::importGLL (QString line)
     return true;
 }
 
-Node* ImportNMEA::importRMC (QString line)
+TrackNode* ImportNMEA::importRMC (QString line)
 {
     if (line.count('$') > 1)
         return NULL;
@@ -192,17 +193,17 @@ Node* ImportNMEA::importRMC (QString line)
     if (tokens[2] != "A")
         return NULL;
 
-    double lat = tokens[3].left(2).toDouble();
-    double latmin = tokens[3].mid(2).toDouble();
+    qreal lat = tokens[3].left(2).toDouble();
+    qreal latmin = tokens[3].mid(2).toDouble();
     lat += latmin / 60.0;
     if (tokens[4] != "N")
         lat = -lat;
-    double lon = tokens[5].left(3).toDouble();
-    double lonmin = tokens[5].mid(3).toDouble();
+    qreal lon = tokens[5].left(3).toDouble();
+    qreal lonmin = tokens[5].mid(3).toDouble();
     lon += lonmin / 60.0;
     if (tokens[6] != "E")
         lon = -lon;
-    double speed = tokens[7].toDouble() * 1.852;
+    qreal speed = tokens[7].toDouble() * 1.852;
     //int date = token[9];
 
     QString strDate = tokens[9] + tokens[1];
@@ -214,7 +215,7 @@ Node* ImportNMEA::importRMC (QString line)
         date = date.addYears(100);
     //date.setTimeSpec(Qt::UTC);
 
-    Node* Pt = new Node(Coord(angToCoord(lat),angToCoord(lon)));
+    TrackNode* Pt = g_backend.allocTrackNode(theLayer, Coord(lon,lat));
     theLayer->add(Pt);
     Pt->setLastUpdated(Feature::Log);
     Pt->setElevation(curAltitude);
