@@ -8,6 +8,8 @@
 #include "Document.h"
 #include "Layer.h"
 #include "Features.h"
+#include "Global.h"
+#include "IProgressWindow.h"
 
 #include <QApplication>
 #include <QtCore/QBuffer>
@@ -35,6 +37,7 @@ void OSMHandler::parseTag(const QXmlAttributes &atts)
 
 void parseStandardAttributes(const QXmlAttributes& atts, Feature* F)
 {
+#ifndef FRISIUS_BUILD
     QString ts = atts.value("timestamp");
     QDateTime time = QDateTime::fromString(ts.left(19), Qt::ISODate);
     if (!time.isValid())
@@ -45,18 +48,19 @@ void parseStandardAttributes(const QXmlAttributes& atts, Feature* F)
     QString version = atts.value("version");
     if (!version.isEmpty())
         F->setVersionNumber(version.toInt());
+#endif
 }
 
 void OSMHandler::parseNode(const QXmlAttributes& atts)
 {
-    double Lat = atts.value("lat").toDouble();
-    double Lon = atts.value("lon").toDouble();
+    qreal Lat = atts.value("lat").toDouble();
+    qreal Lon = atts.value("lon").toDouble();
     QString id = atts.value("id");
     Node* Pt = CAST_NODE(theDocument->getFeature(IFeature::FId(IFeature::Point, id.toLongLong())));
     if (Pt)
     {
         Node* userPt = Pt;
-        Pt = new Node(Coord(angToCoord(Lat),angToCoord(Lon)));
+        Pt = g_backend.allocNode(theLayer, Coord(Lon,Lat));
         Pt->setId(IFeature::FId(IFeature::Point | IFeature::Conflict, id.toLongLong()));
         Pt->setLastUpdated(Feature::OSMServerConflict);
         parseStandardAttributes(atts,Pt);
@@ -65,31 +69,38 @@ void OSMHandler::parseNode(const QXmlAttributes& atts)
         {
             // conflict
             userPt->setLastUpdated(Feature::UserResolved);
+#ifndef FRISIUS_BUILD
             if (Pt->time() > userPt->time() || Pt->versionNumber() != userPt->versionNumber()) {
                 if (conflictLayer)
                     conflictLayer->add(Pt);
                 userPt->setVersionNumber(Pt->versionNumber());
                 NewFeature = true;
-            } else {
-                delete Pt;
+            } else
+#endif
+            {
+                g_backend.deallocFeature(theLayer, Pt);
                 Pt = userPt;
                 NewFeature = false;
             }
         }
         else if (userPt->lastUpdated() != Feature::UserResolved)
         {
-            if (Pt->time() > userPt->time() || Pt->versionNumber() != userPt->versionNumber()) {
-                delete Pt;
+#ifndef FRISIUS_BUILD
+            if (userPt->lastUpdated() == Feature::NotYetDownloaded || (Pt->time() > userPt->time() || Pt->versionNumber() != userPt->versionNumber())) {
+#else
+            if (userPt->lastUpdated() == Feature::NotYetDownloaded) {
+#endif
+                g_backend.deallocFeature(theLayer, Pt);
                 Pt = userPt;
                 Pt->layer()->remove(Pt);
                 theLayer->add(Pt);
-                Pt->setPosition(Coord(angToCoord(Lat),angToCoord(Lon)));
+                Pt->setPosition(Coord(Lon,Lat));
                 Pt->clearTags();
                 NewFeature = true;
                 if (Pt->lastUpdated() == Feature::NotYetDownloaded)
                     Pt->setLastUpdated(Feature::OSMServer);
             } else {
-                delete Pt;
+                g_backend.deallocFeature(theLayer, Pt);
                 Pt = userPt;
                 NewFeature = false;
             }
@@ -97,7 +108,7 @@ void OSMHandler::parseNode(const QXmlAttributes& atts)
     }
     else
     {
-        Pt = new Node(Coord(angToCoord(Lat),angToCoord(Lon)));
+        Pt = g_backend.allocNode(theLayer, Coord(Lon,Lat));
         Pt->setId(IFeature::FId(IFeature::Point, id.toLongLong()));
         Pt->setLastUpdated(Feature::OSMServer);
         theLayer->add(Pt);
@@ -120,7 +131,7 @@ void OSMHandler::parseNd(const QXmlAttributes& atts)
 {
     Way* R = dynamic_cast<Way*>(Current);
     if (!R) return;
-    Node *Part = Feature::getTrackPointOrCreatePlaceHolder(theDocument, theLayer, IFeature::FId(IFeature::Point, atts.value("ref").toLongLong()));
+    Node *Part = Feature::getNodeOrCreatePlaceHolder(theDocument, theLayer, IFeature::FId(IFeature::Point, atts.value("ref").toLongLong()));
     if (NewFeature)
         R->add(Part);
 }
@@ -133,7 +144,7 @@ void OSMHandler::parseWay(const QXmlAttributes& atts)
     if (R)
     {
         Way* userRd = R;
-        R = new Way();
+        R = g_backend.allocWay(theLayer);
         R->setId(IFeature::FId(IFeature::LineString | IFeature::Conflict, id.toLongLong()));
         R->setLastUpdated(Feature::OSMServerConflict);
         parseStandardAttributes(atts,R);
@@ -143,21 +154,28 @@ void OSMHandler::parseWay(const QXmlAttributes& atts)
             userRd->setLastUpdated(Feature::UserResolved);
             NewFeature = false;
             // conflict
+#ifndef FRISIUS_BUILD
             if (R->time() > userRd->time() || R->versionNumber() != userRd->versionNumber()) {
                 if (conflictLayer)
                     conflictLayer->add(R);
                 userRd->setVersionNumber(R->versionNumber());
                 NewFeature = true;
-            } else {
-                delete R;
+            } else
+#endif
+            {
+                g_backend.deallocFeature(theLayer, R);
                 R = userRd;
                 NewFeature = false;
             }
         }
         else if (R->lastUpdated() != Feature::UserResolved)
         {
-            if (R->time() > userRd->time() || R->versionNumber() != userRd->versionNumber()) {
-                delete R;
+#ifndef FRISIUS_BUILD
+            if (userRd->lastUpdated() == Feature::NotYetDownloaded || (R->time() > userRd->time() || R->versionNumber() != userRd->versionNumber())) {
+#else
+            if (userRd->lastUpdated() == Feature::NotYetDownloaded) {
+#endif
+                g_backend.deallocFeature(theLayer, R);
                 R = userRd;
                 R->layer()->remove(R);
                 theLayer->add(R);
@@ -168,7 +186,7 @@ void OSMHandler::parseWay(const QXmlAttributes& atts)
                 if (R->lastUpdated() == Feature::NotYetDownloaded)
                     R->setLastUpdated(Feature::OSMServer);
             } else {
-                delete R;
+                g_backend.deallocFeature(theLayer, R);
                 R = userRd;
                 NewFeature = false;
             }
@@ -176,7 +194,7 @@ void OSMHandler::parseWay(const QXmlAttributes& atts)
     }
     else
     {
-        R = new Way;
+        R = g_backend.allocWay(theLayer);
         R->setId(IFeature::FId(IFeature::LineString, id.toLongLong()));
         R->setLastUpdated(Feature::OSMServer);
         theLayer->add(R);
@@ -199,7 +217,7 @@ void OSMHandler::parseMember(const QXmlAttributes& atts)
     QString Type = atts.value("type");
     Feature* F = 0;
     if (Type == "node")
-        F = Feature::getTrackPointOrCreatePlaceHolder(theDocument, theLayer, IFeature::FId(IFeature::Point, atts.value("ref").toLongLong()));
+        F = Feature::getNodeOrCreatePlaceHolder(theDocument, theLayer, IFeature::FId(IFeature::Point, atts.value("ref").toLongLong()));
     else if (Type == "way")
         F = Feature::getWayOrCreatePlaceHolder(theDocument, theLayer, IFeature::FId(IFeature::LineString, atts.value("ref").toLongLong()));
     else if (Type == "relation")
@@ -217,7 +235,7 @@ void OSMHandler::parseRelation(const QXmlAttributes& atts)
     if (R)
     {
         Relation* userR = R;
-        R = new Relation();
+        R = g_backend.allocRelation(theLayer);
         R->setId(IFeature::FId(IFeature::OsmRelation | IFeature::Conflict, id.toLongLong()));
         R->setLastUpdated(Feature::OSMServerConflict);
         parseStandardAttributes(atts,R);
@@ -227,21 +245,28 @@ void OSMHandler::parseRelation(const QXmlAttributes& atts)
             R->setLastUpdated(Feature::UserResolved);
             NewFeature = false;
             // conflict
+#ifndef FRISIUS_BUILD
             if (R->time() > userR->time() || R->versionNumber() != userR->versionNumber()) {
                 if (conflictLayer)
                     conflictLayer->add(R);
                 userR->setVersionNumber(R->versionNumber());
                 NewFeature = true;
-            } else {
-                delete R;
+            } else
+#endif
+            {
+                g_backend.deallocFeature(theLayer, R);
                 R = userR;
                 NewFeature = false;
             }
         }
         else if (R->lastUpdated() != Feature::UserResolved)
         {
-            if (R->time() > userR->time() || R->versionNumber() != userR->versionNumber()) {
-                delete R;
+#ifndef FRISIUS_BUILD
+            if (userR->lastUpdated() == Feature::NotYetDownloaded || (R->time() > userR->time() || R->versionNumber() != userR->versionNumber())) {
+#else
+            if (userR->lastUpdated() == Feature::NotYetDownloaded) {
+#endif
+                g_backend.deallocFeature(theLayer, R);
                 R = userR;
                 R->layer()->remove(R);
                 theLayer->add(R);
@@ -252,7 +277,7 @@ void OSMHandler::parseRelation(const QXmlAttributes& atts)
                 if (R->lastUpdated() == Feature::NotYetDownloaded)
                     R->setLastUpdated(Feature::OSMServer);
             } else {
-                delete R;
+                g_backend.deallocFeature(theLayer, R);
                 R = userR;
                 NewFeature = false;
             }
@@ -260,7 +285,7 @@ void OSMHandler::parseRelation(const QXmlAttributes& atts)
     }
     else
     {
-        R = new Relation;
+        R = g_backend.allocRelation(theLayer);
         R->setId(IFeature::FId(IFeature::OsmRelation, id.toLongLong()));
         R->setLastUpdated(Feature::OSMServer);
         NewFeature = true;
@@ -312,7 +337,7 @@ static bool downloadToResolve(const QList<Feature*>& Resolution, QWidget* aParen
 
     for (int i=0; i<Resolution.size(); i++ )
     {
-        QUrl URL = M_PREFS->getOsmWebsite()+theDownloader->getURLToFetchFull(Resolution[i]);
+        QUrl URL = M_PREFS->getOsmApiUrl()+theDownloader->getURLToFetchFull(Resolution[i]);
         Lbl->setText(QApplication::translate("Downloader","Downloading unresolved %1 of %2").arg(i+1).arg(Resolution.size()));
         if (theDownloader->go(URL))
         {
