@@ -3,21 +3,22 @@
 #include "RelationCommands.h"
 #include "WayCommands.h"
 #include "NodeCommands.h"
-#include "Maps/Painting.h"
+#include "Painting.h"
 #include "Relation.h"
 #include "Way.h"
 #include "Node.h"
-#include "Utils/LineF.h"
+#include "LineF.h"
 #include "MainWindow.h"
 #include "PropertiesDock.h"
-#include "Utils/MDiscardableDialog.h"
+#include "MDiscardableDialog.h"
+#include "Global.h"
 
 #include <QtGui/QDockWidget>
 #include <QtGui/QMessageBox>
 #include <QtGui/QPainter>
 
-CreateAreaInteraction::CreateAreaInteraction(MainWindow* aMain, MapView* aView)
-    : FeatureSnapInteraction(aView), Main(aMain),
+CreateAreaInteraction::CreateAreaInteraction(MainWindow* aMain)
+    : FeatureSnapInteraction(aMain),
       theRelation(0), theRoad(0), LastRoad(0), FirstPoint(0,0),
       FirstNode(0), HaveFirst(false), EndNow(false)
 {
@@ -50,6 +51,7 @@ void CreateAreaInteraction::paintEvent(QPaintEvent* anEvent, QPainter& thePainte
     if (theRoad && (!theRoad->layer() || theRoad->isDeleted())) { // The road was begon and then undoed. Restarting....
         HaveFirst = false;
         theRoad = NULL;
+        view()->setInteracting(false);
     }
 
     if (HaveFirst)
@@ -64,7 +66,7 @@ void CreateAreaInteraction::paintEvent(QPaintEvent* anEvent, QPainter& thePainte
         ::draw(thePainter,TP,Feature::UnknownDirection, PreviousPoint,LastCursor ,4 ,view()->projection());
 
         Coord NewPoint = XY_TO_COORD(LastCursor);
-        const double distance = FirstPoint.distanceFrom(NewPoint);
+        const qreal distance = FirstPoint.distanceFrom(NewPoint);
 
         QString distanceTag;
         if (distance < 1.0)
@@ -102,34 +104,35 @@ void CreateAreaInteraction::startNewRoad(QMouseEvent* anEvent, Feature* aFeature
         Coord P(XY_TO_COORD(anEvent->pos()));
         CommandList* theList  = new CommandList(MainWindow::tr("Create Area %1").arg(aRoad->description()), aRoad);
         int SnapIdx = findSnapPointIndex(aRoad, P);
-        Node* N = new Node(P);
-        theList->add(new AddFeatureCommand(main()->document()->getDirtyOrOriginLayer(),N,true));
+        Node* N = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), P);
+        theList->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),N,true));
         theList->add(new WayAddNodeCommand(aRoad,N,SnapIdx));
         document()->addHistory(theList);
-        view()->invalidate(true, false);
+        view()->invalidate(true, true, false);
         FirstNode = N;
     }
+    view()->setInteracting(true);
 }
 
 void CreateAreaInteraction::createNewRoad(CommandList* L)
 {
     Node* From = 0;
-    theRoad = new Way;
+    theRoad = g_backend.allocWay(theMain->document()->getDirtyOrOriginLayer());
     if (FirstNode)
     {
         From = FirstNode;
         FirstNode = 0;
         if (!From->isDirty() && !From->hasOSMId() && From->isUploadable())
-            L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),From,true));
+            L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),From,true));
     }
     else
     {
-        From = new Node(FirstPoint);
-        L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),From,true));
+        From = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), FirstPoint);
+        L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),From,true));
     }
-    L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),theRoad,true));
+    L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),theRoad,true));
     if (M_PREFS->getAutoSourceTag()) {
-        QStringList sl = Main->document()->getCurrentSourceTags();
+        QStringList sl = theMain->document()->getCurrentSourceTags();
         if (sl.size())
             theRoad->setTag("source", sl.join(";"));
     }
@@ -144,10 +147,10 @@ void CreateAreaInteraction::finishRoad(CommandList* L)
         L->add(new RelationAddFeatureCommand(theRelation,"inner",theRoad));
     else if (LastRoad)
     {
-        theRelation = new Relation;
-        L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),theRelation,true));
+        theRelation = g_backend.allocRelation(theMain->document()->getDirtyOrOriginLayer());
+        L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),theRelation,true));
         if (M_PREFS->getAutoSourceTag()) {
-            QStringList sl = Main->document()->getCurrentSourceTags();
+            QStringList sl = theMain->document()->getCurrentSourceTags();
             if (sl.size())
                 theRelation->setTag("source", sl.join(";"));
         }
@@ -156,11 +159,10 @@ void CreateAreaInteraction::finishRoad(CommandList* L)
         theRelation->add("inner",theRoad);
         LastRoad = 0;
     }
-    else
-        LastRoad = theRoad;
     HaveFirst = false;
     LastRoad = theRoad;
     theRoad = 0;
+    view()->setInteracting(false);
 
     MDiscardableMessage dlg(NULL,
         MainWindow::tr("Add a hole."),
@@ -182,23 +184,23 @@ void CreateAreaInteraction::addToRoad(QMouseEvent* anEvent, Feature* Snap, Comma
     {
         Coord P(XY_TO_COORD(anEvent->pos()));
         int SnapIdx = findSnapPointIndex(aRoad, P);
-        Node* N = new Node(P);
+        Node* N = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), P);
         CommandList* theList  = new CommandList(MainWindow::tr("Area: Add node %1 to Road %2").arg(N->description()).arg(theRoad->description()), N);
-        theList->add(new AddFeatureCommand(main()->document()->getDirtyOrOriginLayer(),N,true));
+        theList->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),N,true));
         theList->add(new WayAddNodeCommand(aRoad,N,SnapIdx));
         document()->addHistory(theList);
-        view()->invalidate(true, false);
+        view()->invalidate(true, true, false);
         To = N;
     }
     if (!To)
     {
-        To = new Node(XY_TO_COORD(anEvent->pos()));
-        L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),To,true));
+        To = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), XY_TO_COORD(anEvent->pos()));
+        L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),To,true));
         L->setDescription(MainWindow::tr("Area: Add node %1 to Road %2").arg(To->description()).arg(theRoad->description()));
         L->setFeature(To);
     } else {
         if (!To->isDirty() && !To->hasOSMId() && To->isUploadable())
-            L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),To,true));
+            L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),To,true));
     }
     L->add(new WayAddNodeCommand(theRoad,To));
     if (To == theRoad->get(0))
@@ -212,6 +214,7 @@ void CreateAreaInteraction::snapMouseReleaseEvent(QMouseEvent* anEvent, Feature*
         theRelation = NULL;
         HaveFirst = false;
         EndNow = false;
+        view()->setInteracting(false);
     } else
     if ( !panning() ) {
         if (!HaveFirst)
@@ -226,28 +229,29 @@ void CreateAreaInteraction::snapMouseReleaseEvent(QMouseEvent* anEvent, Feature*
                 createNewRoad(L);
             addToRoad(anEvent, aFeature, L);
             document()->addHistory(L);
-            view()->invalidate(true, false);
+            view()->invalidate(true, true, false);
             if (theRelation)
-                Main->properties()->setSelection(theRelation);
+                theMain->properties()->setSelection(theRelation);
             else
-                Main->properties()->setSelection(theRoad);
+                theMain->properties()->setSelection(theRoad);
         }
         FirstPoint = XY_TO_COORD(anEvent->pos());
 
         if (EndNow) {
             if (theRelation)
-                Main->properties()->setSelection(theRelation);
+                theMain->properties()->setSelection(theRelation);
             else
-                Main->properties()->setSelection(LastRoad);
+                theMain->properties()->setSelection(LastRoad);
             LastRoad = NULL;
             theRelation = NULL;
             HaveFirst = false;
             EndNow = false;
+            view()->setInteracting(false);
         }
     }
 }
 
-#ifndef Q_OS_SYMBIAN
+#ifndef _MOBILE
 QCursor CreateAreaInteraction::cursor() const
 {
     return QCursor(Qt::CrossCursor);
@@ -256,6 +260,8 @@ QCursor CreateAreaInteraction::cursor() const
 
 void CreateAreaInteraction::closeAndFinish()
 {
+    view()->setInteracting(false);
+
     if (!theRoad || theRoad->size() < 3)
         return;
 
@@ -263,9 +269,9 @@ void CreateAreaInteraction::closeAndFinish()
     CommandList* theList  = new CommandList(MainWindow::tr("Close Area %1").arg(theRoad->description()), theRoad);
     addToRoad(NULL, N, theList);
     document()->addHistory(theList);
-    view()->invalidate(true, false);
+    view()->invalidate(true, true, false);
     if (theRelation)
-        Main->properties()->setSelection(theRelation);
+        theMain->properties()->setSelection(theRelation);
     else
-        Main->properties()->setSelection(theRoad);
+        theMain->properties()->setSelection(theRoad);
 }
