@@ -2,30 +2,33 @@
 #include "DocumentCommands.h"
 #include "WayCommands.h"
 #include "NodeCommands.h"
-#include "Maps/Painting.h"
+#include "Painting.h"
 #include "Way.h"
 #include "Node.h"
-#include "Utils/LineF.h"
+#include "LineF.h"
 #include "MainWindow.h"
-#include "Preferences/MerkaartorPreferences.h"
+#include "MerkaartorPreferences.h"
+#include "Global.h"
 
 #include <QtGui/QDockWidget>
 #include <QtGui/QPainter>
 
-CreateDoubleWayInteraction::CreateDoubleWayInteraction(MainWindow* aMain, MapView* aView)
-    : Interaction(aView), Main(aMain), R1(0), R2(0), FirstPoint(0,0), HaveFirst(false)
+CreateDoubleWayInteraction::CreateDoubleWayInteraction(MainWindow* aMain)
+    : Interaction(aMain), R1(0), R2(0), FirstPoint(0,0), HaveFirst(false), theDock(0)
 {
-    theDock = new QDockWidget(Main);
+#ifndef _MOBILE
+    theDock = new QDockWidget(theMain);
     QWidget* DockContent = new QWidget(theDock);
     DockData.setupUi(DockContent);
     theDock->setWidget(DockContent);
     theDock->setAllowedAreas(Qt::LeftDockWidgetArea);
-    Main->addDockWidget(Qt::LeftDockWidgetArea, theDock);
+    theMain->addDockWidget(Qt::LeftDockWidgetArea, theDock);
     theDock->show();
     DockData.DriveRight->setChecked(M_PREFS->getrightsidedriving());
     DockData.RoadDistance->setText(QString().setNum(M_PREFS->getdoubleroaddistance()));
 
-    aView->setCursor(cursor());
+    theMain->view()->setCursor(cursor());
+#endif
 }
 
 CreateDoubleWayInteraction::~CreateDoubleWayInteraction()
@@ -59,10 +62,11 @@ void CreateDoubleWayInteraction::paintEvent(QPaintEvent* /* anEvent */, QPainter
 {
     if (R1 && (!R1->layer() || R1->isDeleted())) { // The roads were begon and then undoed. Restarting....
         HaveFirst = false;
+        view()->setInteracting(false);
         R1 = R2 = NULL;
     }
 
-    double rB = view()->pixelPerM()*DockData.RoadDistance->text().toDouble()/2;
+    qreal rB = view()->pixelPerM()*DockData.RoadDistance->text().toDouble()/2;
     if (!HaveFirst)
     {
         thePainter.setPen(QColor(0,0,0));
@@ -78,7 +82,7 @@ void CreateDoubleWayInteraction::paintEvent(QPaintEvent* /* anEvent */, QPainter
 
         if (distance(COORD_TO_XY(PreviousPoint), LastCursor) > 1)
         {
-            double rA = FirstDistance * view()->pixelPerM()/2;
+            qreal rA = FirstDistance * view()->pixelPerM()/2;
             LineF FA1(COORD_TO_XY(PreviousPoint),LastCursor);
             LineF FA2(FA1);
             LineF FB1(FA1);
@@ -132,6 +136,7 @@ void CreateDoubleWayInteraction::mousePressEvent(QMouseEvent* anEvent)
         if (!HaveFirst)
         {
             HaveFirst = true;
+            view()->setInteracting(true);
             FirstPoint = XY_TO_COORD(anEvent->pos());
             FirstDistance = DockData.RoadDistance->text().toDouble();
         }
@@ -149,13 +154,13 @@ void CreateDoubleWayInteraction::mousePressEvent(QMouseEvent* anEvent)
             Coord PreviousPoint = PreviousPoints[R1->size()-1];
             if (distance(COORD_TO_XY(PreviousPoint), LastCursor) > 1)
             {
-                double rB = view()->pixelPerM()*DockData.RoadDistance->text().toDouble()/2;
-                double rA = FirstDistance * view()->pixelPerM()/2;
+                qreal rB = view()->pixelPerM()*DockData.RoadDistance->text().toDouble()/2;
+                qreal rA = FirstDistance * view()->pixelPerM()/2;
                 LineF FA1(COORD_TO_XY(PreviousPoint),LastCursor);
                 LineF FA2(FA1);
                 LineF FB1(FA1);
                 LineF FB2(FA1);
-                double Modifier = DockData.DriveRight->isChecked()?1:-1;
+                qreal Modifier = DockData.DriveRight->isChecked()?1:-1;
                 FA1.slide(rA*Modifier);
                 FA2.slide(-rA*Modifier);
                 FB1.slide(rB*Modifier);
@@ -172,17 +177,17 @@ void CreateDoubleWayInteraction::mousePressEvent(QMouseEvent* anEvent)
                     P1.intersectionWith(N1).toPoint())));
                 L->add(new MoveNodeCommand(A2,XY_TO_COORD(
                     P2.intersectionWith(N2).toPoint())));
-                Node* B1 = new Node(XY_TO_COORD(
+                Node* B1 = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), XY_TO_COORD(
                     FB1.project(LastCursor).toPoint()));
-                Node* B2 = new Node(XY_TO_COORD(
+                Node* B2 = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), XY_TO_COORD(
                     FB2.project(LastCursor).toPoint()));
 
-                L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),B1,true));
-                L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),B2,true));
+                L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),B1,true));
+                L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),B2,true));
                 L->add(new WayAddNodeCommand(R1,B1));
                 L->add(new WayAddNodeCommand(R2,B2,(int)0));
                 document()->addHistory(L);
-                view()->invalidate(true, false);
+                view()->invalidate(true, true, false);
                 //FirstPoint = view()->projection().inverse(anEvent->pos());
                 PreviousPoints[R1->size()-1] = XY_TO_COORD(anEvent->pos());
                 FirstDistance = DockData.RoadDistance->text().toDouble();
@@ -193,39 +198,39 @@ void CreateDoubleWayInteraction::mousePressEvent(QMouseEvent* anEvent)
             Coord PreviousPoint = FirstPoint;
             if (distance(COORD_TO_XY(PreviousPoint), LastCursor) > 1)
             {
-                double rB = view()->pixelPerM()*DockData.RoadDistance->text().toDouble()/2;
-                double rA = FirstDistance * view()->pixelPerM()/2;
+                qreal rB = view()->pixelPerM()*DockData.RoadDistance->text().toDouble()/2;
+                qreal rA = FirstDistance * view()->pixelPerM()/2;
                 LineF FA1(COORD_TO_XY(PreviousPoint),LastCursor);
                 LineF FA2(FA1);
                 LineF FB1(FA1);
                 LineF FB2(FA1);
-                double Modifier = DockData.DriveRight->isChecked()?1:-1;
+                qreal Modifier = DockData.DriveRight->isChecked()?1:-1;
                 FA1.slide(rA*Modifier);
                 FA2.slide(-rA*Modifier);
                 FB1.slide(rB*Modifier);
                 FB2.slide(-rB*Modifier);
 
-                Node* A1 = new Node(XY_TO_COORD(
+                Node* A1 = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), XY_TO_COORD(
                     FA1.project(COORD_TO_XY(PreviousPoint)).toPoint()));
-                Node* A2 = new Node(XY_TO_COORD(
+                Node* A2 = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), XY_TO_COORD(
                     FA2.project(COORD_TO_XY(PreviousPoint)).toPoint()));
-                Node* B1 = new Node(XY_TO_COORD(
+                Node* B1 = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), XY_TO_COORD(
                     FB1.project(LastCursor).toPoint()));
-                Node* B2 = new Node(XY_TO_COORD(
+                Node* B2 = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), XY_TO_COORD(
                     FB2.project(LastCursor).toPoint()));
-                R1 = new Way;
-                R2 = new Way;
+                R1 = g_backend.allocWay(theMain->document()->getDirtyOrOriginLayer());
+                R2 = g_backend.allocWay(theMain->document()->getDirtyOrOriginLayer());
 
                 CommandList* L  = new CommandList(MainWindow::tr("Create double-way Road %1").arg(R1->id().numId), R1);
-                L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),A1,true));
-                L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),A2,true));
-                L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),B1,true));
-                L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),B2,true));
+                L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),A1,true));
+                L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),A2,true));
+                L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),B1,true));
+                L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),B2,true));
 
-                L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),R1,true));
-                L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),R2,true));
+                L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),R1,true));
+                L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),R2,true));
                 if (M_PREFS->getAutoSourceTag()) {
-                    QStringList sl = Main->document()->getCurrentSourceTags();
+                    QStringList sl = theMain->document()->getCurrentSourceTags();
                     if (sl.size()) {
                         R1->setTag("source", sl.join(";"));
                         R2->setTag("source", sl.join(";"));
@@ -238,7 +243,7 @@ void CreateDoubleWayInteraction::mousePressEvent(QMouseEvent* anEvent)
                 L->add(new WayAddNodeCommand(R2,B2));
                 L->add(new WayAddNodeCommand(R2,A2));
                 document()->addHistory(L);
-                view()->invalidate(true, false);
+                view()->invalidate(true, true, false);
                 //FirstPoint = view()->projection().inverse(anEvent->pos());
                 PreviousPoints[R1->size()-1] = XY_TO_COORD(anEvent->pos());
                 FirstDistance = DockData.RoadDistance->text().toDouble();
@@ -249,7 +254,7 @@ void CreateDoubleWayInteraction::mousePressEvent(QMouseEvent* anEvent)
         Interaction::mousePressEvent(anEvent);
 }
 
-#ifndef Q_OS_SYMBIAN
+#ifndef _MOBILE
 QCursor CreateDoubleWayInteraction::cursor() const
 {
     return QCursor(Qt::CrossCursor);
