@@ -1,23 +1,28 @@
 #include "CreatePolygonInteraction.h"
+
+#include "MainWindow.h"
 #include "DocumentCommands.h"
 #include "WayCommands.h"
 #include "NodeCommands.h"
-#include "Maps/Painting.h"
+#include "Painting.h"
 #include "Way.h"
 #include "Node.h"
-#include "Utils/LineF.h"
+#include "LineF.h"
 #include "PropertiesDock.h"
-#include "Preferences/MerkaartorPreferences.h"
+#include "MerkaartorPreferences.h"
+#include "Global.h"
 
 #include <QtGui/QPainter>
 #include <QInputDialog>
 
 #include <math.h>
 
-CreatePolygonInteraction::CreatePolygonInteraction(MainWindow* aMain, MapView* aView, int sides)
-    : Interaction(aView), Main(aMain), Origin(0,0), Sides(sides), HaveOrigin(false), bAngle(0.0), bScale(QPointF(1., 1.))
+CreatePolygonInteraction::CreatePolygonInteraction(MainWindow* aMain, int sides, const QList< QPair <QString, QString> >& tags)
+    : Interaction(aMain), Origin(0,0), Sides(sides), HaveOrigin(false), bAngle(0.0), bScale(QPointF(1., 1.)), theTags(tags)
 {
-    aView->setCursor(cursor());
+#ifndef _MOBILE
+    theMain->view()->setCursor(cursor());
+#endif
 }
 
 CreatePolygonInteraction::~CreatePolygonInteraction()
@@ -58,6 +63,7 @@ void CreatePolygonInteraction::mousePressEvent(QMouseEvent * event)
         if (!HaveOrigin)
         {
             HaveOrigin = true;
+            view()->setInteracting(true);
             Origin = XY_TO_COORD(event->pos());
             OriginF = QPointF(event->pos());
             bAngle = 0.;
@@ -66,10 +72,10 @@ void CreatePolygonInteraction::mousePressEvent(QMouseEvent * event)
         else
         {
             QPointF CenterF(0.5, 0.5);
-            double Radius = 0.5;
+            qreal Radius = 0.5;
             if (Sides == 4)
                 Radius = sqrt(2.)/2.;
-            double Angle = 2*M_PI/Sides;
+            qreal Angle = 2*M_PI/Sides;
             QBrush SomeBrush(QColor(0xff,0x77,0x11,128));
             QPen TP(SomeBrush,view()->pixelPerM()*4);
 
@@ -79,35 +85,40 @@ void CreatePolygonInteraction::mousePressEvent(QMouseEvent * event)
             m.scale(bScale.x(), bScale.y());
 
             QPointF Prev(CenterF.x()+cos(-Angle/2)*Radius,CenterF.y()+sin(-Angle/2)*Radius);
-            Node* First = new Node(XY_TO_COORD(m.map(Prev).toPoint()));
-            Way* R = new Way;
-            R->add(First);
+            Node* First = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), XY_TO_COORD(m.map(Prev).toPoint()));
+            Way* R = g_backend.allocWay(theMain->document()->getDirtyOrOriginLayer());
             CommandList* L  = new CommandList(MainWindow::tr("Create Polygon %1").arg(R->id().numId), R);
-            L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),First,true));
-            for (double a = 2*M_PI - Angle*3/2; a>0; a-=Angle)
+            L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),R,true));
+            R->add(First);
+            L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),First,true));
+            for (qreal a = 2*M_PI - Angle*3/2; a>0; a-=Angle)
             {
                 QPointF Next(CenterF.x()+cos(a)*Radius,CenterF.y()+sin(a)*Radius);
-                Node* New = new Node(XY_TO_COORD(m.map(Next).toPoint()));
-                L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),New,true));
+                Node* New = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), XY_TO_COORD(m.map(Next).toPoint()));
+                L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),New,true));
                 R->add(New);
             }
             R->add(First);
-            L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),R,true));
             if (M_PREFS->getAutoSourceTag()) {
-                QStringList sl = Main->document()->getCurrentSourceTags();
+                QStringList sl = theMain->document()->getCurrentSourceTags();
                 if (sl.size())
                     R->setTag("source", sl.join(";"));
             }
+            QPair <QString, QString> tag;
+            foreach (tag, theTags) {
+                R->setTag(tag.first, tag.second);
+            }
             for (FeatureIterator it(document()); !it.isEnd(); ++it)
             {
                 Way* W1 = dynamic_cast<Way*>(it.get());
                 if (W1 && (W1 != R))
-                    Way::createJunction(Main->document(), L, R, W1, true);
+                    Way::createJunction(theMain->document(), L, R, W1, true);
             }
-            Main->properties()->setSelection(R);
+            theMain->properties()->setSelection(R);
             document()->addHistory(L);
-            view()->invalidate(true, false);
-            view()->launch(0);
+            view()->setInteracting(false);
+            view()->invalidate(true, true, false);
+            theMain->launchInteraction(0);
         }
     }
     else
@@ -119,7 +130,7 @@ void CreatePolygonInteraction::paintEvent(QPaintEvent* , QPainter& thePainter)
     if (HaveOrigin)
     {
         QPointF CenterF(0.5, 0.5);
-        double Radius = 0.5;
+        qreal Radius = 0.5;
         if (Sides == 4)
             Radius = sqrt(2.)/2.;
 
@@ -132,11 +143,11 @@ void CreatePolygonInteraction::paintEvent(QPaintEvent* , QPainter& thePainter)
         thePainter.setPen(QPen(QColor(0,0,255),1,Qt::DotLine));
         thePainter.drawPolygon(thePoly);
 
-        double Angle = 2*M_PI/Sides;
+        qreal Angle = 2*M_PI/Sides;
         QBrush SomeBrush(QColor(0xff,0x77,0x11,128));
         QPen TP(SomeBrush,view()->pixelPerM()*4+1);
         QPointF Prev(CenterF.x()+cos(-Angle/2)*Radius,CenterF.y()+sin(-Angle/2)*Radius);
-        for (double a = 2*M_PI - Angle*3/2; a>0; a-=Angle)
+        for (qreal a = 2*M_PI - Angle*3/2; a>0; a-=Angle)
         {
             QPointF Next(CenterF.x()+cos(a)*Radius,CenterF.y()+sin(a)*Radius);
             ::draw(thePainter,TP,Feature::UnknownDirection, m.map(Prev),m.map(Next),4,view()->projection());
@@ -175,13 +186,14 @@ void CreatePolygonInteraction::mouseReleaseEvent(QMouseEvent* event)
 {
     if (M_PREFS->getMouseSingleButton() && event->button() == Qt::RightButton) {
         HaveOrigin = false;
+        view()->setInteracting(false);
         view()->update();
     }
     Interaction::mouseReleaseEvent(event);
 }
 
 
-#ifndef Q_OS_SYMBIAN
+#ifndef _MOBILE
 QCursor CreatePolygonInteraction::cursor() const
 {
     return QCursor(Qt::CrossCursor);
