@@ -1,32 +1,37 @@
 #include "CreateRoundaboutInteraction.h"
+
+#include "MainWindow.h"
 #include "DocumentCommands.h"
 #include "WayCommands.h"
 #include "NodeCommands.h"
-#include "Maps/Painting.h"
+#include "Painting.h"
 #include "Way.h"
 #include "Node.h"
-#include "Utils/LineF.h"
+#include "LineF.h"
 #include "PropertiesDock.h"
-#include "Preferences/MerkaartorPreferences.h"
+#include "MerkaartorPreferences.h"
+#include "Global.h"
 
 #include <QtGui/QDockWidget>
 #include <QtGui/QPainter>
 
 #include <math.h>
 
-CreateRoundaboutInteraction::CreateRoundaboutInteraction(MainWindow* aMain, MapView* aView)
-    : Interaction(aView), Main(aMain), Center(0,0), HaveCenter(false)
+CreateRoundaboutInteraction::CreateRoundaboutInteraction(MainWindow* aMain)
+    : Interaction(aMain), Center(0,0), HaveCenter(false), theDock(0)
 {
-    theDock = new QDockWidget(Main);
+#ifndef _MOBILE
+    theDock = new QDockWidget(theMain);
     QWidget* DockContent = new QWidget(theDock);
     DockData.setupUi(DockContent);
     theDock->setWidget(DockContent);
     theDock->setAllowedAreas(Qt::LeftDockWidgetArea);
-    Main->addDockWidget(Qt::LeftDockWidgetArea, theDock);
+    theMain->addDockWidget(Qt::LeftDockWidgetArea, theDock);
     theDock->show();
     DockData.DriveRight->setChecked(M_PREFS->getrightsidedriving());
 
-    aView->setCursor(cursor());
+    theMain->view()->setCursor(cursor());
+#endif
 }
 
 CreateRoundaboutInteraction::~CreateRoundaboutInteraction()
@@ -61,42 +66,43 @@ void CreateRoundaboutInteraction::mousePressEvent(QMouseEvent * event)
         if (!HaveCenter)
         {
             HaveCenter = true;
+            view()->setInteracting(true);
             Center = XY_TO_COORD(event->pos());
         }
         else
         {
             QPointF CenterF(COORD_TO_XY(Center));
-            double Radius = distance(CenterF,LastCursor)/view()->pixelPerM();
-            double Precision = 2.49;
+            qreal Radius = distance(CenterF,LastCursor)/view()->pixelPerM();
+            qreal Precision = 2.49;
             if (Radius<2.5)
                 Radius = 2.5;
-            double Angle = 2*acos(1-Precision/Radius);
-            double Steps = ceil(2*M_PI/Angle);
+            qreal Angle = 2*acos(1-Precision/Radius);
+            qreal Steps = ceil(2*M_PI/Angle);
             Angle = 2*M_PI/Steps;
             Radius *= view()->pixelPerM();
-            double Modifier = DockData.DriveRight->isChecked()?-1:1;
+            qreal Modifier = DockData.DriveRight->isChecked()?-1:1;
             QBrush SomeBrush(QColor(0xff,0x77,0x11,128));
             QPen TP(SomeBrush,view()->pixelPerM()*4+1);
             QPointF Prev(CenterF.x()+cos(Modifier*Angle/2)*Radius,CenterF.y()+sin(Modifier*Angle/2)*Radius);
-            Node* First = new Node(XY_TO_COORD(Prev.toPoint()));
-            Way* R = new Way;
+            Node* First = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), XY_TO_COORD(Prev.toPoint()));
+            Way* R = g_backend.allocWay(theMain->document()->getDirtyOrOriginLayer());
             CommandList* L  = new CommandList(MainWindow::tr("Create Roundabout %1").arg(R->id().numId), R);
-            L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),R,true));
+            L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),R,true));
             R->add(First);
-            L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),First,true));
+            L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),First,true));
             if (M_PREFS->getAutoSourceTag()) {
-                QStringList sl = Main->document()->getCurrentSourceTags();
+                QStringList sl = theMain->document()->getCurrentSourceTags();
                 if (sl.size())
                     R->setTag("source", sl.join(";"));
             }
             // "oneway" is implied on roundabouts
             //R->setTag("oneway","yes");
             R->setTag("junction","roundabout");
-            for (double a = Angle*3/2; a<2*M_PI; a+=Angle)
+            for (qreal a = Angle*3/2; a<2*M_PI; a+=Angle)
             {
                 QPointF Next(CenterF.x()+cos(Modifier*a)*Radius,CenterF.y()+sin(Modifier*a)*Radius);
-                Node* New = new Node(XY_TO_COORD(Next.toPoint()));
-                L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),New,true));
+                Node* New = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), XY_TO_COORD(Next.toPoint()));
+                L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),New,true));
                 R->add(New);
             }
             R->add(First);
@@ -104,12 +110,13 @@ void CreateRoundaboutInteraction::mousePressEvent(QMouseEvent * event)
             {
                 Way* W1 = CAST_WAY(it.get());
                 if (W1 && (W1 != R))
-                    Way::createJunction(Main->document(), L, R, W1, true);
+                    Way::createJunction(theMain->document(), L, R, W1, true);
             }
-            Main->properties()->setSelection(R);
+            theMain->properties()->setSelection(R);
             document()->addHistory(L);
-            view()->invalidate(true, false);
-            view()->launch(0);
+            view()->setInteracting(false);
+            view()->invalidate(true, true, false);
+            theMain->launchInteraction(0);
         }
     }
     else
@@ -128,6 +135,7 @@ void CreateRoundaboutInteraction::mouseReleaseEvent(QMouseEvent* anEvent)
 {
     if (M_PREFS->getMouseSingleButton() && anEvent->button() == Qt::RightButton) {
         HaveCenter = false;
+        view()->setInteracting(false);
         view()->update();
     }
     Interaction::mouseReleaseEvent(anEvent);
@@ -138,19 +146,19 @@ void CreateRoundaboutInteraction::paintEvent(QPaintEvent* , QPainter& thePainter
     if (HaveCenter)
     {
         QPointF CenterF(COORD_TO_XY(Center));
-        double Radius = distance(CenterF,LastCursor)/view()->pixelPerM();
-        double Precision = 1.99;
+        qreal Radius = distance(CenterF,LastCursor)/view()->pixelPerM();
+        qreal Precision = 1.99;
         if (Radius<2)
             Radius = 2;
-        double Angle = 2*acos(1-Precision/Radius);
-        double Steps = ceil(2*M_PI/Angle);
+        qreal Angle = 2*acos(1-Precision/Radius);
+        qreal Steps = ceil(2*M_PI/Angle);
         Angle = 2*M_PI/Steps;
         Radius *= view()->pixelPerM();
-        double Modifier = DockData.DriveRight->isChecked()?-1:1;
+        qreal Modifier = DockData.DriveRight->isChecked()?-1:1;
         QBrush SomeBrush(QColor(0xff,0x77,0x11,128));
         QPen TP(SomeBrush,view()->pixelPerM()*4);
         QPointF Prev(CenterF.x()+cos(Modifier*Angle/2)*Radius,CenterF.y()+sin(Modifier*Angle/2)*Radius);
-        for (double a = Angle*3/2; a<2*M_PI; a+=Angle)
+        for (qreal a = Angle*3/2; a<2*M_PI; a+=Angle)
         {
             QPointF Next(CenterF.x()+cos(Modifier*a)*Radius,CenterF.y()+sin(Modifier*a)*Radius);
             ::draw(thePainter,TP,Feature::OneWay, Prev,Next,4,view()->projection());
@@ -161,7 +169,7 @@ void CreateRoundaboutInteraction::paintEvent(QPaintEvent* , QPainter& thePainter
     }
 }
 
-#ifndef Q_OS_SYMBIAN
+#ifndef _MOBILE
 QCursor CreateRoundaboutInteraction::cursor() const
 {
     return QCursor(Qt::CrossCursor);
