@@ -3,18 +3,19 @@
 #include "DocumentCommands.h"
 #include "WayCommands.h"
 #include "NodeCommands.h"
-#include "Maps/Painting.h"
+#include "Painting.h"
 #include "Way.h"
 #include "Node.h"
-#include "Utils/LineF.h"
+#include "LineF.h"
 #include "MainWindow.h"
 #include "PropertiesDock.h"
+#include "Global.h"
 
 #include <QtGui/QDockWidget>
 #include <QtGui/QPainter>
 
-CreateSingleWayInteraction::CreateSingleWayInteraction(MainWindow* aMain, MapView* aView, Node *firstNode, bool aCurved)
-    : FeatureSnapInteraction(aView), Main(aMain), theRoad(0), FirstPoint(0,0),
+CreateSingleWayInteraction::CreateSingleWayInteraction(MainWindow* aMain, Node *firstNode, bool aCurved)
+    : FeatureSnapInteraction(aMain), theRoad(0), FirstPoint(0,0),
      FirstNode(firstNode), HaveFirst(false), Prepend(false), IsCurved(aCurved), Creating(false)
      , SnapAngle(0)
      , ParallelMode(false)
@@ -24,6 +25,7 @@ CreateSingleWayInteraction::CreateSingleWayInteraction(MainWindow* aMain, MapVie
         FirstPoint = firstNode->position();
         LastCursor = COORD_TO_XY(FirstPoint);
         HaveFirst = true;
+        view()->setInteracting(true);
         if ((theRoad = Way::GetSingleParentRoad(firstNode))) {
             if (theRoad->isExtrimity(firstNode)) {
                 Prepend = (theRoad->get(0) == firstNode) ? true : false;
@@ -38,7 +40,7 @@ CreateSingleWayInteraction::~CreateSingleWayInteraction()
 {
 }
 
-void CreateSingleWayInteraction::setSnapAngle(double angle)
+void CreateSingleWayInteraction::setSnapAngle(qreal angle)
 {
     SnapAngle = angle;
 }
@@ -72,6 +74,7 @@ void CreateSingleWayInteraction::paintEvent(QPaintEvent* anEvent, QPainter& theP
         HaveFirst = false;
         theRoad = NULL;
         Creating = false;
+        view()->setInteracting(false);
     }
 
     if (HaveFirst)
@@ -86,7 +89,7 @@ void CreateSingleWayInteraction::paintEvent(QPaintEvent* anEvent, QPainter& theP
         ::draw(thePainter,TP,Feature::UnknownDirection, PreviousPoint,LastCursor ,4 ,view()->projection());
 
         Coord NewPoint = XY_TO_COORD(LastCursor);
-        const double distance = FirstPoint.distanceFrom(NewPoint);
+        const qreal distance = FirstPoint.distanceFrom(NewPoint);
 
         QString distanceTag;
         if (distance < 1.0)
@@ -113,7 +116,7 @@ void CreateSingleWayInteraction::snapMouseMoveEvent(QMouseEvent* ev, Feature* la
     {
         QLineF l1(COORD_TO_XY(theRoad->getNode(theRoad->size()-1)), COORD_TO_XY(theRoad->getNode(theRoad->size()-2)));
         QLineF l2(COORD_TO_XY(theRoad->getNode(theRoad->size()-1)), ev->pos());
-        double a = l1.angleTo(l2);
+        qreal a = l1.angleTo(l2);
         a = qRound(a/SnapAngle) * SnapAngle;
         l2.setAngle(l1.angle() + a);
         LastCursor = l2.p2().toPoint();
@@ -128,16 +131,16 @@ void CreateSingleWayInteraction::snapMouseMoveEvent(QMouseEvent* ev, Feature* la
             PreviousPoint = COORD_TO_XY(FirstPoint);
 
         CoordBox HotZone(XY_TO_COORD(ev->pos()-QPoint(CLEAR_DISTANCE, CLEAR_DISTANCE)),XY_TO_COORD(ev->pos()+QPoint(CLEAR_DISTANCE, CLEAR_DISTANCE)));
-        double BestDistanceNW = 9999, AngleNW = 0;
-        double BestDistanceNE = 9999, AngleNE = 0;
-        double* BestDistance;
-        double* BestAngle;
-        double curAngle;
+        qreal BestDistanceNW = 9999, AngleNW = 0;
+        qreal BestDistanceNE = 9999, AngleNE = 0;
+        qreal* BestDistance;
+        qreal* BestAngle;
+        qreal curAngle;
 
         Way* R;
         for (int j=0; j<document()->layerSize(); ++j) {
-            QList < MapFeaturePtr > ret = document()->getLayer(j)->indexFind(HotZone);
-            foreach(MapFeaturePtr F, ret) {
+            QList < Feature* > ret = g_backend.indexFind(document()->getLayer(j), HotZone);
+            foreach(Feature* F, ret) {
                 if (!(R = CAST_WAY(F)))
                     continue;
 
@@ -149,10 +152,10 @@ void CreateSingleWayInteraction::snapMouseMoveEvent(QMouseEvent* ev, Feature* la
                 for (int i=0; i<R->size()-1; ++i)
                 {
                     LineF F(COORD_TO_XY(R->getNode(i)),COORD_TO_XY(R->getNode(i+1)));
-                    double D = F.capDistance(ev->pos());
+                    qreal D = F.capDistance(ev->pos());
                     if (D < CLEAR_DISTANCE) {
                         QLineF l(COORD_TO_XY(R->getNode(i)), COORD_TO_XY(R->getNode(i+1)));
-                        double a = l.angle();
+                        qreal a = l.angle();
                         if ((a >= 0 && a < 90) || (a < -270 && a >= -360)) {
                             BestDistance = &BestDistanceNE;
                             BestAngle = &AngleNE;
@@ -183,7 +186,7 @@ void CreateSingleWayInteraction::snapMouseMoveEvent(QMouseEvent* ev, Feature* la
         }
 
         QLineF l(PreviousPoint, ev->pos());
-        double a = l.angle();
+        qreal a = l.angle();
         if ((a >= 0 && a < 90) || (a < -270 && a >= -360)) {
             if (BestDistanceNE < 9999)
                 a = AngleNE;
@@ -208,14 +211,16 @@ void CreateSingleWayInteraction::snapMousePressEvent(QMouseEvent* /* anEvent */,
 {
     Q_UNUSED(lastSnap)
     Creating = true;
+    view()->setInteracting(true);
 }
 
 void CreateSingleWayInteraction::snapMouseReleaseEvent(QMouseEvent* anEvent, Feature* lastSnap)
 {
-    if (M_PREFS->getMouseSingleButton() && anEvent->button() == Qt::RightButton) {
+    if (M_PREFS->getMouseSingleButton() && anEvent->button() == Qt::RightButton) {  // Abort
         HaveFirst = false;
         theRoad = NULL;
         Creating = false;
+        view()->setInteracting(false);
     } else
     if ( Creating && !panning() )
     {
@@ -229,12 +234,12 @@ void CreateSingleWayInteraction::snapMouseReleaseEvent(QMouseEvent* anEvent, Fea
             {
                 Coord P(XY_TO_COORD(LastCursor));
                 int SnapIdx = findSnapPointIndex(aRoad, P);
-                Node* N = new Node(P);
+                Node* N = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), P);
                 CommandList* theList  = new CommandList(MainWindow::tr("Create Node %1 in Road %2").arg(N->description()).arg(aRoad->description()), N);
-                theList->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),N,true));
-                theList->add(new WayAddNodeCommand(aRoad,N,SnapIdx,Main->document()->getDirtyOrOriginLayer(aRoad)));
+                theList->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),N,true));
+                theList->add(new WayAddNodeCommand(aRoad,N,SnapIdx,theMain->document()->getDirtyOrOriginLayer(aRoad)));
                 document()->addHistory(theList);
-                view()->invalidate(true, false);
+                view()->invalidate(true, true, false);
                 FirstNode = N;
             }
         }
@@ -244,30 +249,30 @@ void CreateSingleWayInteraction::snapMouseReleaseEvent(QMouseEvent* anEvent, Fea
             if (!theRoad)
             {
                 Node* From = 0;
-                theRoad = new Way;
+                theRoad = g_backend.allocWay(theMain->document()->getDirtyOrOriginLayer());
+                L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),theRoad,true));
                 if (FirstNode) {
                     if (FirstNode->isVirtual()) {
                         Way* aRoad = CAST_WAY(FirstNode->getParent(0));
                         int SnapIdx = aRoad->findVirtual(FirstNode)+1;
-                        From = new Node(*FirstNode);
+                        From = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(aRoad->layer()), *FirstNode);
                         From->setVirtual(false);
                         From->setPosition(FirstNode->position());
-                        L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(aRoad->layer()),From,true));
+                        L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(aRoad->layer()),From,true));
                         L->add(new WayAddNodeCommand(aRoad,From,SnapIdx,main()->document()->getDirtyOrOriginLayer(aRoad->layer())));
                     } else {
                         From = FirstNode;
                         if (!From->isDirty() && !From->hasOSMId() && From->isUploadable())
-                            L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),From,true));
+                            L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),From,true));
                     }
                 }
                 else
                 {
-                    From = new Node(FirstPoint);
-                    L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),From,true));
+                    From = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), FirstPoint);
+                    L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),From,true));
                 }
-                L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),theRoad,true));
                 if (M_PREFS->getAutoSourceTag()) {
-                    QStringList sl = Main->document()->getCurrentSourceTags();
+                    QStringList sl = theMain->document()->getCurrentSourceTags();
                     if (sl.size())
                         theRoad->setTag("source", sl.join(";"));
                 }
@@ -281,7 +286,7 @@ void CreateSingleWayInteraction::snapMouseReleaseEvent(QMouseEvent* anEvent, Fea
             if (Pt) {
                 To = Pt;
                 if (!To->isDirty() && !To->hasOSMId() && To->isUploadable()) {
-                    L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),To,true));
+                    L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),To,true));
                     L->setDescription(MainWindow::tr("Create Node: %1").arg(To->description()));
                 }
             }
@@ -289,29 +294,29 @@ void CreateSingleWayInteraction::snapMouseReleaseEvent(QMouseEvent* anEvent, Fea
             {
                 Coord P(XY_TO_COORD(LastCursor));
                 int SnapIdx = findSnapPointIndex(aRoad, P);
-                Node* N = new Node(P);
+                Node* N = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), P);
                 CommandList* theList  = new CommandList(MainWindow::tr("Create Node %1 in Road %2").arg(N->description()).arg(aRoad->description()), N);
-                theList->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),N,true));
-                theList->add(new WayAddNodeCommand(aRoad,N,SnapIdx,Main->document()->getDirtyOrOriginLayer(aRoad)));
+                theList->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),N,true));
+                theList->add(new WayAddNodeCommand(aRoad,N,SnapIdx,theMain->document()->getDirtyOrOriginLayer(aRoad)));
                 document()->addHistory(theList);
-                view()->invalidate(true, false);
+                view()->invalidate(true, true, false);
                 To = N;
             }
             if (!To)
             {
-                To = new Node(XY_TO_COORD(LastCursor));
-                L->add(new AddFeatureCommand(Main->document()->getDirtyOrOriginLayer(),To,true));
+                To = g_backend.allocNode(theMain->document()->getDirtyOrOriginLayer(), XY_TO_COORD(LastCursor));
+                L->add(new AddFeatureCommand(theMain->document()->getDirtyOrOriginLayer(),To,true));
                 L->setDescription(MainWindow::tr("Create Node %1 in Road %2").arg(To->description()).arg(theRoad->description()));
                 L->setFeature(To);
             }
             L->setDescription(MainWindow::tr("Add Node %1 to Road %2").arg(To->description()).arg(theRoad->description()));
             if (Prepend)
-                L->add(new WayAddNodeCommand(theRoad,To,(int)0,Main->document()->getDirtyOrOriginLayer(theRoad)));
+                L->add(new WayAddNodeCommand(theRoad,To,(int)0,theMain->document()->getDirtyOrOriginLayer(theRoad)));
             else
-                L->add(new WayAddNodeCommand(theRoad,To,Main->document()->getDirtyOrOriginLayer(theRoad)));
+                L->add(new WayAddNodeCommand(theRoad,To,theMain->document()->getDirtyOrOriginLayer(theRoad)));
             document()->addHistory(L);
-            view()->invalidate(true, false);
-            Main->properties()->setSelection(theRoad);
+            view()->invalidate(true, true, false);
+            theMain->properties()->setSelection(theRoad);
         }
         FirstPoint = XY_TO_COORD(LastCursor);
     }
@@ -324,12 +329,13 @@ void CreateSingleWayInteraction::snapMouseDoubleClickEvent(QMouseEvent* anEvent,
     HaveFirst = false;
     theRoad = NULL;
     Creating = false;
+    view()->setInteracting(false);
 
     if ((lastSnap() && lastSnap()->getType() & IFeature::LineString) || !lastSnap())
         CreateNodeInteraction::createNode(XY_TO_COORD(anEvent->pos()), lastSnap());
 }
 
-#ifndef Q_OS_SYMBIAN
+#ifndef _MOBILE
 QCursor CreateSingleWayInteraction::cursor() const
 {
     return QCursor(Qt::CrossCursor);
@@ -343,12 +349,13 @@ void CreateSingleWayInteraction::closeAndFinish()
 
     Node* N = theRoad->getNode(0);
     CommandList* theList  = new CommandList(MainWindow::tr("Close Road %1").arg(theRoad->description()), theRoad);
-    theList->add(new WayAddNodeCommand(theRoad,N,Main->document()->getDirtyOrOriginLayer(theRoad)));
+    theList->add(new WayAddNodeCommand(theRoad,N,theMain->document()->getDirtyOrOriginLayer(theRoad)));
     document()->addHistory(theList);
-    view()->invalidate(true, false);
-    Main->properties()->setSelection(theRoad);
+    view()->invalidate(true, true, false);
+    theMain->properties()->setSelection(theRoad);
 
     HaveFirst = false;
     theRoad = NULL;
     Creating = false;
+    view()->setInteracting(false);
 }
