@@ -2,7 +2,9 @@
 #include "EditInteraction.h"
 
 #include "MainWindow.h"
+#ifndef _MOBILE
 #include "ui_MainWindow.h"
+#endif
 #include "MapView.h"
 #include "PropertiesDock.h"
 #include "InfoDock.h"
@@ -12,10 +14,10 @@
 #include "MoveNodeInteraction.h"
 #include "Document.h"
 #include "Features.h"
-#include "Maps/FeatureManipulations.h"
-#include "Maps/Projection.h"
-#include "Utils/LineF.h"
-#include "Utils/MDiscardableDialog.h"
+#include "FeatureManipulations.h"
+#include "Projection.h"
+#include "LineF.h"
+#include "MDiscardableDialog.h"
 
 #include <QtGui/QMouseEvent>
 #include <QtGui/QPainter>
@@ -23,14 +25,16 @@
 
 #include <QList>
 
-EditInteraction::EditInteraction(MapView* theView)
-: FeatureSnapInteraction(theView), Dragging(false), StartDrag(0,0), EndDrag(0,0)
+#define PROPERTIES(x) {if (theMain->properties()) theMain->properties()->x;}
+
+EditInteraction::EditInteraction(MainWindow* aMain)
+: FeatureSnapInteraction(aMain), Dragging(false), StartDrag(0,0), EndDrag(0,0)
 {
     defaultCursor = QCursor(Qt::ArrowCursor);
 
     connect(main(),SIGNAL(remove_triggered()),this,SLOT(on_remove_triggered()));
     connect(main(),SIGNAL(reverse_triggered()), this,SLOT(on_reverse_triggered()));
-    view()->properties()->checkMenuStatus();
+    PROPERTIES(checkMenuStatus());
 
     if (!M_PREFS->getSeparateMoveMode()) {
         setDontSelectVirtual(false);
@@ -39,11 +43,6 @@ EditInteraction::EditInteraction(MapView* theView)
 
 EditInteraction::~EditInteraction(void)
 {
-    if(main())
-    {
-        main()->ui->editRemoveAction->setEnabled(false);
-        main()->ui->editReverseAction->setEnabled(false);
-    }
 }
 
 QString EditInteraction::toHtml()
@@ -102,10 +101,12 @@ static bool modifiersForDrag(Qt::KeyboardModifiers modifiers)
         return true;
 }
 
+#if 0
 static bool modifiersForSegmentSelect(Qt::KeyboardModifiers modifiers)
 {
     return modifiers.testFlag(Qt::AltModifier);
 }
+#endif
 
 static bool modifiersForGreedyAdd(Qt::KeyboardModifiers modifiers)
 {
@@ -124,25 +125,25 @@ void EditInteraction::snapMousePressEvent(QMouseEvent * ev, Feature* aLast)
     if (!view()->isSelectionLocked()) {
         if (modifiers) {
             if (modifiersForToggle(modifiers) && aLast)
-                view()->properties()->toggleSelection(aLast);
+                PROPERTIES(toggleSelection(aLast));
 
             if (modifiersForAdd(modifiers) && aLast)
-                view()->properties()->addSelection(aLast);
+                PROPERTIES(addSelection(aLast));
 
             if (g_Merk_Segment_Mode && aLast) {
-                view()->properties()->setSelection(aLast);
+                PROPERTIES(setSelection(aLast));
             }
         } else {
             StackSnap = SnapList;
 //				if (aLast)
-//					view()->properties()->setSelection(aLast);
+//					PROPERTIES(setSelection(aLast));
         }
         if (!aLast && modifiersForDrag(modifiers))
         {
             EndDrag = StartDrag = XY_TO_COORD(ev->pos());
             Dragging = true;
         }
-        view()->properties()->checkMenuStatus();
+        PROPERTIES(checkMenuStatus());
         view()->update();
     }
 }
@@ -212,14 +213,14 @@ void EditInteraction::snapMouseReleaseEvent(QMouseEvent * ev , Feature* aLast)
             }
         }
         if (!List.isEmpty() || (!modifiersForAdd(modifiers) && !modifiersForToggle(modifiers)))
-            view()->properties()->setSelection(List);
-        view()->properties()->checkMenuStatus();
+            PROPERTIES(setSelection(List));
+        PROPERTIES(checkMenuStatus());
         Dragging = false;
         view()->update();
     } else {
         if (!panning() && !modifiers) {
-            view()->properties()->setSelection(aLast);
-            view()->properties()->checkMenuStatus();
+            PROPERTIES(setSelection(aLast));
+            PROPERTIES(checkMenuStatus());
             view()->update();
         }
     }
@@ -249,35 +250,35 @@ void EditInteraction::snapMouseDoubleClickEvent(QMouseEvent* anEvent, Feature* a
 //            theFeatures << aLast;
 //            for (int i=0; i<aLast->size(); ++i)
 //                theFeatures << aLast->get(i);
-//            view()->properties()->setSelection(theFeatures);
-//            view()->properties()->checkMenuStatus();
+//            PROPERTIES(setSelection(theFeatures));
+//            PROPERTIES(checkMenuStatus());
 //            view()->update();
 //        } else {
-//            Node* N = new Node(XY_TO_COORD(anEvent->pos()));
+//            Node* N = g_backend.allocNode(XY_TO_COORD(anEvent->pos()));
 //            CommandList* theList  = new CommandList(MainWindow::tr("Create point %1").arg(N->id()), N);
 //            theList->add(new AddFeatureCommand(main()->document()->getDirtyOrOriginLayer(),N,true));
 //            document()->addHistory(theList);
 //            main()->properties()->setSelection(N);
-//            view()->invalidate(true, false);
+//            view()->invalidate(true, true, false);
 //       }
 //    }
 }
 
 void EditInteraction::on_remove_triggered()
 {
-    if (view()->properties()->size() == 0) return;
+    if (theMain->properties()->selectionSize() == 0) return;
 
     QList<Feature*> Sel;
-    for (int i=0; i<view()->properties()->size(); ++i) {
-        if (!document()->isDownloadedSafe(view()->properties()->selection(i)->boundingBox()) && view()->properties()->selection(i)->hasOSMId())
+    for (int i=0; i<theMain->properties()->selectionSize(); ++i) {
+        if (!document()->isDownloadedSafe(theMain->properties()->selection(i)->boundingBox()) && theMain->properties()->selection(i)->hasOSMId())
             continue;
         else
-            Sel.push_back(view()->properties()->selection(i));
+            Sel.push_back(theMain->properties()->selection(i));
     }
     if (Sel.size() == 0) {
         QMessageBox::critical(NULL, tr("Cannot delete"), tr("Cannot delete the selection because it is outside the downloaded area."));
         return;
-    } else if (Sel.size() != view()->properties()->size()) {
+    } else if (Sel.size() != theMain->properties()->selectionSize()) {
         if (!QMessageBox::warning(NULL, tr("Cannot delete everything"),
                              tr("The complete selection cannot be deleted because part of it is outside the downloaded area.\n"
                                 "Delete what can be?"),
@@ -313,17 +314,17 @@ void EditInteraction::on_remove_triggered()
 
     if (theList->size()) {
         document()->addHistory(theList);
-        view()->properties()->setSelection(0);
-        view()->properties()->checkMenuStatus();
+        theMain->properties()->setSelection(0);
+        theMain->properties()->checkMenuStatus();
     }
     else
         delete theList;
-    view()->invalidate(true, false);
+    view()->invalidate(true, true, false);
 }
 
 void EditInteraction::on_reverse_triggered()
 {
-    QList<Feature*> selection = view()->properties()->selection();
+    QList<Feature*> selection = theMain->properties()->selection();
     QString desc = selection.size() == 1 ? tr("Reverse way %1").arg(selection[0]->id().numId) : tr("Reverse %1 ways");
     CommandList* theList  = new CommandList(MainWindow::tr("Reverse %1 ways").arg(selection.size()), NULL);
     foreach (Feature* f, selection)
@@ -333,11 +334,11 @@ void EditInteraction::on_reverse_triggered()
         delete theList;
     } else {
         document()->addHistory(theList);
-        view()->invalidate(true, false);
+        view()->invalidate(true, true, false);
     }
 }
 
-#ifndef Q_OS_SYMBIAN
+#ifndef _MOBILE
 QCursor EditInteraction::cursor() const
 {
     if (LastSnap)
