@@ -11,7 +11,6 @@ class QMouseEvent;
 class QPaintEvent;
 class QPainter;
 
-#include "MainWindow.h"
 #include "MapView.h"
 #include "InfoDock.h"
 #include "FeaturesDock.h"
@@ -27,14 +26,14 @@ class QPainter;
 
 #include <algorithm>
 
-#define XY_TO_COORD(x)  theView->fromView(x)
-#define COORD_TO_XY(x)  theView->toView(x)
+#define XY_TO_COORD(x)  theMain->view()->fromView(x)
+#define COORD_TO_XY(x)  theMain->view()->toView(x)
 
 class Interaction : public QObject
 {
     Q_OBJECT
 public:
-    Interaction(MapView* theView);
+    Interaction(MainWindow* aMain);
     virtual ~Interaction();
 
     virtual void mousePressEvent(QMouseEvent * event);
@@ -51,11 +50,11 @@ public:
     MainWindow* main();
     bool panning() const;
 
-    void updateSnap(QMouseEvent* event);
+    virtual void updateSnap(QMouseEvent* event);
     Feature* lastSnap();
 
 protected:
-    MapView* theView;
+    MainWindow* theMain;
     bool Panning;
     QPoint FirstPan;
     QPoint LastPan;
@@ -74,6 +73,7 @@ protected:
 
 signals:
     void requestCustomContextMenu(const QPoint & pos);
+    void featureSnap(Feature*);
 
 protected:
     bool Dragging;
@@ -84,7 +84,7 @@ protected:
 class FeatureSnapInteraction : public Interaction
 {
 public:
-    FeatureSnapInteraction(MapView* theView);
+    FeatureSnapInteraction(MainWindow* aMain);
 
     virtual void paintEvent(QPaintEvent* anEvent, QPainter& thePainter);
 
@@ -100,6 +100,7 @@ public:
 
     void activateSnap(bool b);
     void addToNoSnap(Feature* F);
+    void addToNoSnap(QList<Feature *> Fl);
     void clearNoSnap();
     void clearSnap();
     void clearLastSnap();
@@ -113,10 +114,9 @@ public:
     void setDontSelectRoads(bool b);
     void setDontSelectVirtual(bool b);
 
-#ifndef Q_OS_SYMBIAN
+#ifndef _MOBILE
     virtual QCursor cursor() const;
 #endif
-
 private:
     QCursor handCursor;
     QCursor grabCursor;
