@@ -1,17 +1,20 @@
 #include "ZoomInteraction.h"
 
+#include "MainWindow.h"
 #include "MapView.h"
 #include "Document.h"
-#include "Maps/Projection.h"
+#include "Projection.h"
 #include "Node.h"
 
 #include <QtGui/QMouseEvent>
 #include <QtGui/QPainter>
 
-ZoomInteraction::ZoomInteraction(MapView* aView)
-: Interaction(aView), HaveFirstPoint(false)
+ZoomInteraction::ZoomInteraction(MainWindow* aMain)
+: Interaction(aMain), HaveFirstPoint(false)
 {
-    aView->setCursor(cursor());
+#ifndef _MOBILE
+    theMain->view()->setCursor(cursor());
+#endif
 }
 
 ZoomInteraction::~ZoomInteraction(void)
@@ -59,8 +62,8 @@ void ZoomInteraction::mouseReleaseEvent(QMouseEvent * event)
     {
         P2 = event->pos();
         view()->setViewport(CoordBox(XY_TO_COORD(P1),XY_TO_COORD(P2)),view()->rect());
-        view()->invalidate(true, true);
-        view()->launch(0);
+        view()->invalidate(true, true, true);
+        theMain->launchInteraction(0);
     }
 }
 
@@ -73,7 +76,7 @@ void ZoomInteraction::mouseMoveEvent(QMouseEvent* event)
     }
 }
 
-#ifndef Q_OS_SYMBIAN
+#ifndef _MOBILE
 QCursor ZoomInteraction::cursor() const
 {
     QPixmap pm(":/Icons/zoomico.xpm");
