@@ -1,21 +1,22 @@
 #include "Global.h"
 
+#include "MainWindow.h"
 #include "MapView.h"
 #include "ImageMapLayer.h"
 
 #include "Document.h"
-#include "Preferences/MerkaartorPreferences.h"
-#include "Maps/Projection.h"
+#include "MerkaartorPreferences.h"
+#include "Projection.h"
 
 #include "IMapAdapterFactory.h"
 #include "IMapAdapter.h"
-#include "QMapControl/imagemanager.h"
+#include "imagemanager.h"
 #ifdef USE_WEBKIT
-#include "QMapControl/browserimagemanager.h"
+#include "browserimagemanager.h"
 #endif
-#include "QMapControl/tilemapadapter.h"
-#include "QMapControl/wmsmapadapter.h"
-#include "QMapControl/WmscMapAdapter.h"
+#include "tilemapadapter.h"
+#include "wmsmapadapter.h"
+#include "WmscMapAdapter.h"
 
 #include <QLocale>
 #include <QPainter>
@@ -53,6 +54,7 @@ public:
     QRectF Viewport;
     QTransform AlignementTransform;
     QVector<QTransform> AlignementTransformList;
+    QPointF cumulatedDelta;
 
 public:
     ImageMapLayerPrivate()
@@ -240,24 +242,7 @@ void ImageMapLayer::setMapAdapter(const QUuid& theAdapterUid, const QString& ser
         }
     } else
     if (p->bgType == SHAPE_ADAPTER_UUID) {
-        if (!M_PREFS->getUseShapefileForBackground()) {
-            setNoneAdapter();
-        } else {
-#if defined(Q_OS_MAC)
-            QDir resources = QDir(QCoreApplication::applicationDirPath());
-            resources.cdUp();
-            resources.cd("Resources");
-            QString world_shp = resources.absolutePath() + "/" + STRINGIFY(WORLD_SHP);
-//            setFilename(world_shp);
-#else
-//            if (QDir::isAbsolutePath(STRINGIFY(WORLD_SHP)))
-//                setFilename(STRINGIFY(WORLD_SHP));
-//            else
-//                setFilename(QCoreApplication::applicationDirPath() + "/" + STRINGIFY(WORLD_SHP));
-#endif
-        }
-            setName(tr("Map - OSB Background"));
-            setVisible(true);
+        setNoneAdapter();
     } else
     {
         IMapAdapterFactory* fac = M_PREFS->getBackgroundPlugin(p->bgType);
@@ -270,7 +255,10 @@ void ImageMapLayer::setMapAdapter(const QUuid& theAdapterUid, const QString& ser
             Q_ASSERT(theImageWidget);
             connect(p->theMapAdapter, SIGNAL(forceZoom()), theImageWidget, SLOT(zoomLayer()));
             connect(p->theMapAdapter, SIGNAL(forceProjection()), theImageWidget, SLOT(setProjection()));
-            connect(p->theMapAdapter, SIGNAL(forceRefresh()), g_Merk_MainWindow, SLOT(invalidateView()));
+#ifndef _MOBILE
+            if (g_Merk_MainWindow)
+                connect(p->theMapAdapter, SIGNAL(forceRefresh()), g_Merk_MainWindow, SLOT(invalidateView()));
+#endif
         } else
             setNoneAdapter();
     }
@@ -284,7 +272,7 @@ void ImageMapLayer::setMapAdapter(const QUuid& theAdapterUid, const QString& ser
                 QUrl u(s);
                 if (u.isValid()) {
                     Ui::LicenseDisplayDialog ui;
-                    QDialog dlg(g_Merk_MainWindow);
+                    QDialog dlg;
                     ui.setupUi(&dlg);
                     dlg.setWindowTitle(tr("Licensing Terms: %1").arg(name()));
                     ui.webView->load(u);
@@ -386,130 +374,139 @@ bool ImageMapLayer::isTiled()
     return (p->theMapAdapter->isTiled());
 }
 
-void QTransformToXml(QDomElement& parent, const QTransform& theTransform)
+void QTransformToXml(QXmlStreamWriter& stream, const QTransform& theTransform)
 {
-    parent.setAttribute("m11", theTransform.m11());
-    parent.setAttribute("m12", theTransform.m12());
-    parent.setAttribute("m13", theTransform.m13());
-    parent.setAttribute("m21", theTransform.m21());
-    parent.setAttribute("m22", theTransform.m22());
-    parent.setAttribute("m23", theTransform.m23());
-    parent.setAttribute("m31", theTransform.m31());
-    parent.setAttribute("m32", theTransform.m32());
-    parent.setAttribute("m33", theTransform.m33());
+    stream.writeAttribute("m11", QString::number(theTransform.m11()));
+    stream.writeAttribute("m12", QString::number(theTransform.m12()));
+    stream.writeAttribute("m13", QString::number(theTransform.m13()));
+    stream.writeAttribute("m21", QString::number(theTransform.m21()));
+    stream.writeAttribute("m22", QString::number(theTransform.m22()));
+    stream.writeAttribute("m23", QString::number(theTransform.m23()));
+    stream.writeAttribute("m31", QString::number(theTransform.m31()));
+    stream.writeAttribute("m32", QString::number(theTransform.m32()));
+    stream.writeAttribute("m33", QString::number(theTransform.m33()));
 }
 
-QTransform QTransformFomXml(const QDomElement& parent)
+QTransform QTransformFomXml(QXmlStreamReader& stream)
 {
-    qreal m11 = parent.attribute("m11").toDouble();
-    qreal m12 = parent.attribute("m12").toDouble();
-    qreal m13 = parent.attribute("m13").toDouble();
-    qreal m21 = parent.attribute("m21").toDouble();
-    qreal m22 = parent.attribute("m11").toDouble();
-    qreal m23 = parent.attribute("m23").toDouble();
-    qreal m31 = parent.attribute("m31").toDouble();
-    qreal m32 = parent.attribute("m32").toDouble();
-    qreal m33 = parent.attribute("m33").toDouble();
+    qreal m11 = stream.attributes().value("m11").toString().toDouble();
+    qreal m12 = stream.attributes().value("m12").toString().toDouble();
+    qreal m13 = stream.attributes().value("m13").toString().toDouble();
+    qreal m21 = stream.attributes().value("m21").toString().toDouble();
+    qreal m22 = stream.attributes().value("m11").toString().toDouble();
+    qreal m23 = stream.attributes().value("m23").toString().toDouble();
+    qreal m31 = stream.attributes().value("m31").toString().toDouble();
+    qreal m32 = stream.attributes().value("m32").toString().toDouble();
+    qreal m33 = stream.attributes().value("m33").toString().toDouble();
 
     return QTransform(m11, m12, m13, m21, m22, m23, m31, m32, m33);
 }
 
-bool ImageMapLayer::toXML(QDomElement& xParent, bool asTemplate, QProgressDialog * /* progress */)
+bool ImageMapLayer::toXML(QXmlStreamWriter& stream, bool asTemplate, QProgressDialog * /* progress */)
 {
     bool OK = true;
 
-    QDomElement e = xParent.ownerDocument().createElement(metaObject()->className());
-    xParent.appendChild(e);
+    stream.writeStartElement(metaObject()->className());
 
-    e.setAttribute("xml:id", id());
-    e.setAttribute("name", name());
-    e.setAttribute("alpha", QString::number(getAlpha(),'f',2));
-    e.setAttribute("visible", QString((isVisible() ? "true" : "false")));
-    e.setAttribute("selected", QString((isSelected() ? "true" : "false")));
-    e.setAttribute("enabled", QString((isEnabled() ? "true" : "false")));
+    stream.writeAttribute("xml:id", id());
+    stream.writeAttribute("name", name());
+    stream.writeAttribute("alpha", QString::number(getAlpha(),'f',2));
+    stream.writeAttribute("visible", QString((isVisible() ? "true" : "false")));
+    stream.writeAttribute("selected", QString((isSelected() ? "true" : "false")));
+    stream.writeAttribute("enabled", QString((isEnabled() ? "true" : "false")));
 
-    e.setAttribute("bgtype", p->bgType.toString());
+    stream.writeAttribute("bgtype", p->bgType.toString());
 
-    QDomElement c;
     WmsServer ws;
     TmsServer ts;
 
     if (p->bgType == WMS_ADAPTER_UUID) {
-        c = e.ownerDocument().createElement("WmsServer");
-        e.appendChild(c);
-
-        c.setAttribute("name", p->selServer);
+        stream.writeStartElement("WmsServer");
+        stream.writeAttribute("name", p->selServer);
+        stream.writeEndElement();
     } else if (p->bgType == TMS_ADAPTER_UUID) {
-        c = e.ownerDocument().createElement("TmsServer");
-        e.appendChild(c);
-
-        c.setAttribute("name", p->selServer);
+        stream.writeStartElement("TmsServer");
+        stream.writeAttribute("name", p->selServer);
+        stream.writeEndElement();
     } else if (p->bgType != NONE_ADAPTER_UUID) {
-        c = e.ownerDocument().createElement("Data");
-        e.appendChild(c);
-
+        stream.writeStartElement("Data");
         if (!asTemplate)
-            p->theMapAdapter->toXML(c);
+            p->theMapAdapter->toXML(stream);
+        stream.writeEndElement();
     }
     if (!asTemplate) {
-        QDomElement atListEl = e.ownerDocument().createElement("AdjustmentList");
-        e.appendChild(atListEl);
+        stream.writeStartElement("AdjustmentList");
         for (int i=0; i<p->AlignementTransformList.size(); ++i) {
             if (!p->AlignementTransformList.at(i).isIdentity()) {
-                QDomElement atEl = e.ownerDocument().createElement("Adjustment");
-                atListEl.appendChild(atEl);
-                atEl.setAttribute("zoom", i);
-                QTransformToXml(atEl, p->AlignementTransformList.at(i));
+                stream.writeStartElement("Adjustment");
+                stream.writeAttribute("zoom", QString::number(i));
+                QTransformToXml(stream, p->AlignementTransformList.at(i));
+                stream.writeEndElement();
             }
         }
+        stream.writeEndElement();
     }
+    stream.writeEndElement();
 
     return OK;
 }
 
-ImageMapLayer * ImageMapLayer::fromXML(Document* d, const QDomElement& e, QProgressDialog * /*progress*/)
+ImageMapLayer * ImageMapLayer::fromXML(Document* d, QXmlStreamReader& stream, QProgressDialog * /*progress*/)
 {
-    ImageMapLayer* l = new ImageMapLayer(e.attribute("name"));
-    l->blockIndexing(true);
-    l->setId(e.attribute("xml:id"));
-
-    l->blockIndexing(false);
+    ImageMapLayer* l = new ImageMapLayer(stream.attributes().value("name").toString());
 
+    l->setId(stream.attributes().value("xml:id").toString());
     d->addImageLayer(l);
-    l->reIndex();
 
     QString server;
-    QUuid bgtype = QUuid(e.attribute("bgtype"));
+    QUuid bgtype = QUuid(stream.attributes().value("bgtype").toString());
 
-    QDomElement c = e.firstChildElement();
-    while(!c.isNull()) {
-        if (c.tagName() == "AdjustmentList") {
-            QDomNodeList atListEl = c.elementsByTagName("Adjustment");
-            for (int i=0; i<atListEl.size(); ++i) {
-                QDomElement el = atListEl.at(i).toElement();
-                int z = el.attribute("zoom").toInt();
-                if (l->p->AlignementTransformList.size() < z+1)
-                    l->p->AlignementTransformList.resize(z+1);
-                l->p->AlignementTransformList[z] = QTransformFomXml(el);
+    qreal alpha = stream.attributes().value("alpha").toString().toDouble();
+    bool visible = (stream.attributes().value("visible") == "true" ? true : false);
+    bool selected = (stream.attributes().value("selected") == "true" ? true : false);
+    bool enabled = (stream.attributes().value("enabled") == "false" ? false : true);
+
+    stream.readNext();
+    while(!stream.atEnd() && !stream.isEndElement()) {
+        if (stream.name() == "AdjustmentList") {
+            stream.readNext();
+            while(!stream.atEnd() && !stream.isEndElement()) {
+                if (stream.name() == "Adjustment") {
+                    int z = stream.attributes().value("zoom").toString().toInt();
+                    if (l->p->AlignementTransformList.size() < z+1)
+                        l->p->AlignementTransformList.resize(z+1);
+                    l->p->AlignementTransformList[z] = QTransformFomXml(stream);
+                    stream.readNext();
+                } else if (!stream.isWhitespace()) {
+                    qDebug() << "ImgLay: logic error: " << stream.name() << " : " << stream.tokenType() << " (" << stream.lineNumber() << ")";
+                    stream.skipCurrentElement();
+                }
+                stream.readNext();
             }
-        } else if (c.tagName() == "WmsServer") {
-            server = c.attribute("name");
+        } else if (stream.name() == "WmsServer") {
+            server = stream.attributes().value("name").toString();
             l->setMapAdapter(bgtype, server);
-        } else if (c.tagName() == "TmsServer") {
-            server = c.attribute("name");
+            stream.readNext();
+        } else if (stream.name() == "TmsServer") {
+            server = stream.attributes().value("name").toString();
             l->setMapAdapter(bgtype, server);
-        } else if (c.tagName() == "Data") {
+            stream.readNext();
+        } else if (stream.name() == "Data") {
             l->setMapAdapter(bgtype, server);
+            stream.readNext();
             if (l->getMapAdapter())
-                l->getMapAdapter()->fromXML(c);
+                l->getMapAdapter()->fromXML(stream);
+        } else if (!stream.isWhitespace()) {
+            qDebug() << "ImgLay: logic error: " << stream.name() << " : " << stream.tokenType() << " (" << stream.lineNumber() << ")";
+            stream.skipCurrentElement();
         }
-        c = c.nextSiblingElement();
+        stream.readNext();
     }
 
-    l->setAlpha(e.attribute("alpha").toDouble());
-    l->setVisible((e.attribute("visible") == "true" ? true : false));
-    l->setSelected((e.attribute("selected") == "true" ? true : false));
-    l->setEnabled((e.attribute("enabled") == "false" ? false : true));
+    l->setAlpha(alpha);
+    l->setVisible(visible);
+    l->setSelected(selected);
+    l->setEnabled(enabled);
 
     return l;
 }
@@ -523,9 +520,7 @@ void ImageMapLayer::drawImage(QPainter* P)
     P->drawPixmap(0, 0, p->curPix);
 }
 
-using namespace ggl;
-
-void ImageMapLayer::zoom(double zoom, const QPoint& pos, const QRect& rect)
+void ImageMapLayer::zoom(qreal zoom, const QPoint& pos, const QRect& rect)
 {
     if (!p->theMapAdapter)
         return;
@@ -538,7 +533,8 @@ void ImageMapLayer::zoom(double zoom, const QPoint& pos, const QRect& rect)
     p->curPix.fill(Qt::transparent);
     QPainter P(&p->curPix);
     QPointF fPos(pos);
-    P.drawPixmap(fPos - (fPos * zoom), tpm);
+    fPos -= fPos * zoom;
+    P.drawPixmap(fPos, tpm);
 }
 
 void ImageMapLayer::pan(QPoint delta)
@@ -550,21 +546,13 @@ void ImageMapLayer::pan(QPoint delta)
     if (p->curPix.isNull())
         return;
 
-#if QT_VERSION < 0x040600
-        QPixmap savPix;
-        savPix = p->curPix.copy();
-        p->curPix.fill(Qt::transparent);
-        QPainter P(&p->curPix);
-        P.drawPixmap(delta, savPix);
-#else
-        QRegion exposed;
-        p->curPix.scroll(delta.x(), delta.y(), p->curPix.rect(), &exposed);
-        QPainter P(&p->curPix);
-        P.setClipping(true);
-        P.setClipRegion(exposed);
-        P.eraseRect(p->curPix.rect());
-#endif
-//        on_imageReceived();
+    QRegion exposed;
+    p->curPix.scroll(delta.x(), delta.y(), p->curPix.rect(), &exposed);
+    QPainter P(&p->curPix);
+    P.setClipping(true);
+    P.setClipRegion(exposed);
+    P.eraseRect(p->curPix.rect());
+    //        on_imageReceived();
 }
 
 void ImageMapLayer::zoom_in()
@@ -600,10 +588,10 @@ void ImageMapLayer::setCurrentZoom(MapView& theView, const CoordBox& viewport, c
         projVp.setTopLeft(theView.invertedTransform().map(fRect.topLeft()));
         projVp.setBottomRight(theView.invertedTransform().map(fRect.bottomRight()));
     } else
-        projVp = p->theProjection.getProjectedViewport(viewport, rect);
+        projVp = p->theProjection.toProjectedRectF(viewport, rect);
 
     qreal tileWidth, tileHeight;
-    int maxZoom = p->theMapAdapter->getAdaptedMaxZoom();
+    int maxZoom = p->theMapAdapter->getAdaptedMaxZoom(viewport);
     int tilesizeW = p->theMapAdapter->getTileSizeW();
     int tilesizeH = p->theMapAdapter->getTileSizeH();
 
@@ -647,12 +635,12 @@ void ImageMapLayer::forceRedraw(MapView& theView, QTransform& aTransform, QRect
     if (!p->theMapAdapter)
         return;
 
-    if (!p->Viewport.intersects(theView.viewport().toQRectF())) {
+    if (!p->Viewport.intersects(theView.viewport())) {
         p->curPix = QPixmap(Screen.size());
         p->curPix.fill(Qt::transparent);
     }
     p->AlignementTransform = aTransform;
-    p->Viewport = theView.viewport().toQRectF();
+    p->Viewport = theView.viewport();
 
     draw(theView, Screen);
 }
@@ -663,6 +651,7 @@ void ImageMapLayer::draw(MapView& theView, QRect& rect)
         return;
 
     p->theProjection.setProjectionType(p->theMapAdapter->projection());
+    p->cumulatedDelta = QPointF();
 
     if (p->theMapAdapter->isTiled())
         p->pr = drawTiled(theView, rect);
@@ -680,10 +669,10 @@ void ImageMapLayer::draw(MapView& theView, QRect& rect)
     const QSize ps = p->pr.size();
     const QSize pmSize = p->newPix.size();
     const qreal ratio = qMax<const qreal>((qreal)pmSize.width()/ps.width()*1.0, (qreal)pmSize.height()/ps.height()*1.0);
-    qDebug() << "Bg image ratio " << ratio;
+//    qDebug() << "Bg image ratio " << ratio;
     QPixmap pms;
     if (ratio >= 1.0) {
-        qDebug() << "Bg image scale 1 " << ps << " : " << p->newPix.size();
+//        qDebug() << "Bg image scale 1 " << ps << " : " << p->newPix.size();
         pms = p->newPix.scaled(ps);
     } else {
         const QSizeF drawingSize = pmSize * ratio;
@@ -691,14 +680,13 @@ void ImageMapLayer::draw(MapView& theView, QRect& rect)
         const QPointF drawingOrigin = QPointF(originSize.width(), originSize.height());
         const QRect drawingRect = QRect(drawingOrigin.toPoint(), drawingSize.toSize());
 
-        qDebug() << "Bg image scale 2 " << ps << " : " << p->newPix.size();
+//        qDebug() << "Bg image scale 2 " << ps << " : " << p->newPix.size();
         if (ps*ratio != drawingRect.size())
             pms = p->newPix.copy(drawingRect).scaled(ps*ratio);
         else
             pms = p->newPix.copy(drawingRect);
     }
 
-    p->newPix.fill(Qt::transparent);
     QPainter P(&p->curPix);
     P.drawPixmap((pmSize.width()-pms.width())/2, (pmSize.height()-pms.height())/2, pms);
     //    if (p->theMapAdapter->isTiled())
@@ -715,31 +703,31 @@ QRect ImageMapLayer::drawFull(MapView& theView, QRect& rect)
     p->AlignementTransform = QTransform();
     QRectF alignedViewport = p->AlignementTransformList.at(0).mapRect(p->Viewport);
 
-    MapView::transformCalc(p->theTransform, p->theProjection, 0.0, CoordBox::fromQRectF(alignedViewport), rect);
+    MapView::transformCalc(p->theTransform, p->theProjection, 0.0, CoordBox(alignedViewport), rect);
 
     CoordBox cViewport(p->theProjection.inverse2Coord(p->theTransform.inverted().map(fRect.bottomLeft())),
                      p->theProjection.inverse2Coord(p->theTransform.inverted().map(fRect.topRight())));
-    CoordBox Viewport = CoordBox::fromQRectF(p->AlignementTransformList.at(0).mapRect(cViewport.toQRectF()));
+    CoordBox Viewport = CoordBox(p->AlignementTransformList.at(0).mapRect(cViewport));
     QPointF bl = theView.toView(Viewport.bottomLeft());
     QPointF tr = theView.toView(Viewport.topRight());
 
     if (
-            Viewport.bottomLeft().lat() >= -90. && Viewport.bottomLeft().lat() <= 90.
-            && Viewport.bottomLeft().lon() >= -180. && Viewport.bottomLeft().lon() <= 180.
-            && Viewport.topRight().lat() >= -90. && Viewport.topRight().lat() <= 90.
-            && Viewport.topRight().lon() >= -180. && Viewport.topRight().lon() <= 180.
+            Viewport.bottomLeft().y() >= -90. && Viewport.bottomLeft().y() <= 90.
+            && Viewport.bottomLeft().x() >= -180. && Viewport.bottomLeft().x() <= 180.
+            && Viewport.topRight().y() >= -90. && Viewport.topRight().y() <= 90.
+            && Viewport.topRight().x() >= -180. && Viewport.topRight().x() <= 180.
             ) {
         QRectF vp;
-        if (p->theProjection.getProjectionProj4() == theView.projection().getProjectionProj4()  && alignedViewport == theView.viewport().toQRectF()) {
+        if (p->theProjection.getProjectionProj4() == theView.projection().getProjectionProj4()  && alignedViewport == theView.viewport()) {
             bl = QPointF(rect.bottomLeft());
             tr = QPointF(rect.topRight());
             vp.setTopLeft(theView.invertedTransform().map(fRect.topLeft()));
             vp.setBottomRight(theView.invertedTransform().map(fRect.bottomRight()));
         } else
-            vp = p->theProjection.getProjectedViewport(CoordBox::fromQRectF(alignedViewport), rect);
+            vp = p->theProjection.toProjectedRectF(CoordBox(alignedViewport), rect);
 
-        QRectF wgs84vp = QRectF(QPointF(coordToAng(Viewport.bottomLeft().lon()), coordToAng(Viewport.bottomLeft().lat()))
-                                , QPointF(coordToAng(Viewport.topRight().lon()), coordToAng(Viewport.topRight().lat())));
+        QRectF wgs84vp = QRectF(QPointF(Viewport.bottomLeft().x(), Viewport.bottomLeft().y())
+                                , QPointF(Viewport.topRight().x(), Viewport.topRight().y()));
 
         // Act depending on adapter type
         if (p->theMapAdapter->getType() == IMapAdapter::DirectBackground) {
@@ -761,7 +749,7 @@ QRect ImageMapLayer::drawFull(MapView& theView, QRect& rect)
                         IFeature::FId id(IFeature::Point, -(f->id().numId));
                         if (get(id))
                             continue;
-                        Node* N = new Node(Coord::fromQPointF((QPointF)thePath.elementAt(0)));
+                        Node* N = g_backend.allocNode(this, Coord((QPointF)thePath.elementAt(0)));
                         N->setId(id);
                         add(N);
                         for (int i=0; i<f->tagSize(); ++i)
@@ -770,10 +758,10 @@ QRect ImageMapLayer::drawFull(MapView& theView, QRect& rect)
                         IFeature::FId id(IFeature::LineString, -(f->id().numId));
                         if (get(id))
                             continue;
-                        Way* W = new Way();
+                        Way* W = g_backend.allocWay(this);
                         W->setId(id);
                         for (int i=0; i<thePath.elementCount(); ++i) {
-                            Node* N = new Node(Coord::fromQPointF((QPointF)thePath.elementAt(i)));
+                            Node* N = g_backend.allocNode(this, Coord((QPointF)thePath.elementAt(i)));
                             add(N);
                             W->add(N);
                         }
@@ -849,7 +837,7 @@ QRect ImageMapLayer::drawFull(MapView& theView, QRect& rect)
             QString url (p->theMapAdapter->getQuery(wgs84vp, vp, rect));
             if (!url.isEmpty()) {
                 qDebug() << "ImageMapLayer::drawFull: getting: " << url;
-                QPixmap pm = p->theMapAdapter->getImageManager()->getPixmap(p->theMapAdapter,url);
+                QPixmap pm = QPixmap::fromImage(p->theMapAdapter->getImageManager()->getImage(p->theMapAdapter,url));
                 if (!pm.isNull()) {
                     p->curPix = QPixmap();
                     p->newPix = pm.scaled(rect.size(), Qt::IgnoreAspectRatio);
@@ -867,16 +855,17 @@ QRect ImageMapLayer::drawFull(MapView& theView, QRect& rect)
 QRect ImageMapLayer::drawTiled(MapView& theView, QRect& rect)
 {
     QRectF projVp;
+//    QRectF fRect(-rect.width(), -rect.height(), rect.width()*3.0, rect.height()*3.0);
     QRectF fRect(rect);
 
     if (p->theProjection.getProjectionProj4() == theView.projection().getProjectionProj4()) {
         projVp.setTopLeft(theView.invertedTransform().map(fRect.topLeft()));
         projVp.setBottomRight(theView.invertedTransform().map(fRect.bottomRight()));
     } else
-        projVp = p->theProjection.getProjectedViewport(CoordBox::fromQRectF(p->Viewport), rect);
+        projVp = p->theProjection.toProjectedRectF(CoordBox(p->Viewport), fRect.toRect());
 
     qreal tileWidth, tileHeight;
-    int maxZoom = p->theMapAdapter->getAdaptedMaxZoom();
+    int maxZoom = p->theMapAdapter->getAdaptedMaxZoom(p->Viewport);
     int tilesizeW = p->theMapAdapter->getTileSizeW();
     int tilesizeH = p->theMapAdapter->getTileSizeH();
 
@@ -916,11 +905,11 @@ QRect ImageMapLayer::drawTiled(MapView& theView, QRect& rect)
     QRectF alignedViewport = p->AlignementTransformList.at(p->theMapAdapter->getAdaptedZoom()).mapRect(p->Viewport);
 
     if (alignedViewport != p->Viewport) {
-        if (p->theProjection.getProjectionProj4() == theView.projection().getProjectionProj4() && alignedViewport == theView.viewport().toQRectF()) {
+        if (p->theProjection.getProjectionProj4() == theView.projection().getProjectionProj4() && alignedViewport == theView.viewport()) {
             projVp.setTopLeft(theView.invertedTransform().map(fRect.topLeft()));
             projVp.setBottomRight(theView.invertedTransform().map(fRect.bottomRight()));
         } else
-            projVp = p->theProjection.getProjectedViewport(CoordBox::fromQRectF(alignedViewport), rect);
+            projVp = p->theProjection.toProjectedRectF(CoordBox(alignedViewport), rect);
     }
 
     QPointF vpCenter = projVp.center();
@@ -973,7 +962,7 @@ QRect ImageMapLayer::drawTiled(MapView& theView, QRect& rect)
     int cross_scr_x = cross_x * tilesizeW / tileWidth;
     int cross_scr_y = cross_y * tilesizeH / tileHeight;
 
-    QSize pmSize = rect.size();
+    QSize pmSize = fRect.size().toSize();
 //    QSize pmSize((tiles_right+tiles_left+1)*tilesizeW, (tiles_bottom+tiles_above+1)*tilesizeH);
 //    QPixmap tmpPm = p->pm.scaled(retRect.size(), Qt::IgnoreAspectRatio);
     p->newPix = QPixmap(pmSize);
@@ -981,17 +970,17 @@ QRect ImageMapLayer::drawTiled(MapView& theView, QRect& rect)
     QPainter painter(&p->newPix);
 //    painter.drawPixmap(0, 0, tmpPm);
 
-    qDebug() << "Tiles: " << tiles_right+tiles_left+1 << "x" << tiles_bottom+tiles_above+1;
-    for (i=-tiles_left+mapmiddle_tile_x; i<=tiles_right+mapmiddle_tile_x; i++)
+//    qDebug() << "Tiles: " << tiles_right+tiles_left+1 << "x" << tiles_bottom+tiles_above+1;
+    for (i=-tiles_left; i<=tiles_right; i++)
     {
-        for (j=-tiles_above+mapmiddle_tile_y; j<=tiles_bottom+mapmiddle_tile_y; j++)
+        for (j=-tiles_above; j<=tiles_bottom; j++)
         {
-            if (p->theMapAdapter->isValid(i, j, p->theMapAdapter->getZoom()))
+            if (p->theMapAdapter->isValid(mapmiddle_tile_x+i, mapmiddle_tile_y+j, p->theMapAdapter->getZoom()))
             {
 #ifdef Q_CC_MSVC
-                double priority = _hypot(i - mapmiddle_tile_x, j - mapmiddle_tile_y);
+                qreal priority = _hypot(i, j);
 #else
-                double priority = hypot(i - mapmiddle_tile_x, j - mapmiddle_tile_y);
+                qreal priority = hypot(i, j);
 #endif
                 tiles.append(Tile(i, j, priority));
             }
@@ -1003,24 +992,28 @@ QRect ImageMapLayer::drawTiled(MapView& theView, QRect& rect)
     int n=0; // Arbitrarily limit the number of tiles to 100
     for (QList<Tile>::const_iterator tile = tiles.begin(); tile != tiles.end() && n<100; ++tile)
     {
-        QPixmap pm = p->theMapAdapter->getImageManager()->getPixmap(p->theMapAdapter, tile->i, tile->j, p->theMapAdapter->getZoom());
+        QImage pm = p->theMapAdapter->getImageManager()->getImage(p->theMapAdapter, p->theMapAdapter->getQuery(mapmiddle_tile_x+tile->i, mapmiddle_tile_y+tile->j, p->theMapAdapter->getZoom()));
         if (!pm.isNull())
-            painter.drawPixmap(((tile->i-mapmiddle_tile_x)*tilesizeW)+pmSize.width()/2 -cross_scr_x,
-                               ((tile->j-mapmiddle_tile_y)*tilesizeH)+pmSize.height()/2-cross_scr_y,
+            painter.drawImage((tile->i*tilesizeW)+pmSize.width()/2 -cross_scr_x,
+                               (tile->j*tilesizeH)+pmSize.height()/2-cross_scr_y,
                                pm);
 
         if (M_PREFS->getDrawTileBoundary()) {
-            painter.drawRect(((tile->i-mapmiddle_tile_x)*tilesizeW)+pmSize.width()/2 -cross_scr_x,
-                             ((tile->j-mapmiddle_tile_y)*tilesizeH)+pmSize.height()/2-cross_scr_y,
+            painter.drawRect((tile->i*tilesizeW)+pmSize.width()/2 -cross_scr_x,
+                             (tile->j*tilesizeH)+pmSize.height()/2-cross_scr_y,
                              tilesizeW, tilesizeH);
         }
         ++n;
     }
     painter.end();
 
-    qDebug() << "tl: " << tl << "; br: " << br;
-    qDebug() << "vp: " << projVp;
-    qDebug() << "vlm: " << vlm;
+//    qDebug() << "tl: " << tl << "; br: " << br;
+//    qDebug() << "vp: " << projVp;
+    //    qDebug() << "vlm: " << vlm;
+    qDebug() << "retRect: " << retRect;
+//    QRect expR = QRect(-retRect.left(), -retRect.top(), retRect.width()+retRect.left(), retRect.height()+retRect.top());
+//    p->newPix.save("c:/tmp.png");
+//    p->newPix.copy(expR).save("c:/tmp2.png");
     return retRect;
 }
 
@@ -1052,7 +1045,7 @@ QString ImageMapLayer::toPropertiesHtml()
             if (p->theMapAdapter->isTiled()) {
                 h += "<i>" + tr("Tile size") + ": </i>" + QString("%1x%2").arg(p->theMapAdapter->getTileSizeW()).arg(p->theMapAdapter->getTileSizeH());
                 h += "<br/>";
-                h += "<i>" + tr("Min/Max zoom") + ": </i>" + QString("%1/%2").arg(p->theMapAdapter->getMinZoom()).arg(p->theMapAdapter->getMaxZoom());
+                h += "<i>" + tr("Min/Max zoom") + ": </i>" + QString("%1/%2").arg(p->theMapAdapter->getMinZoom(alignedViewport)).arg(p->theMapAdapter->getMaxZoom(alignedViewport));
                 h += "<br/>";
             }
         }
