@@ -1,12 +1,14 @@
 #ifndef LAYER_H_
 #define LAYER_H_
 
-#include "Maps/MapTypedef.h"
-#include "Maps/Coord.h"
+#include "MapTypedef.h"
+#include "Coord.h"
 #include "Feature.h"
 
 #include <QProgressDialog>
 
+#include "ILayer.h"
+
 class QString;
 class QprogressDialog;
 
@@ -19,13 +21,12 @@ class LayerWidget;
 class WMSMapAdapter;
 class TileMapAdapter;
 class TrackSegment;
-class OsbLayerPrivate;
 class IMapAdapter;
 class Document;
 
 struct IndexFindContext;
 
-class Layer : public QObject
+class Layer : public QObject, public ILayer
 {
     Q_OBJECT
 
@@ -44,10 +45,11 @@ public:
         DrawingLayerType,
         ExtractedLayerType,
         ImageLayerType,
-        OsbLayerType,
         TrackLayerType,
         UploadedLayerType,
-        FilterLayerType
+        FilterLayerType,
+        OsmBugsLayer,
+        MapDustLayer
     } LayerType;
 
     enum LayerGroup {
@@ -56,6 +58,7 @@ public:
         Draw				= 0x00000002,
         Tracks				= 0x00000004,
         Filters				= 0x00000008,
+        Special				= 0x00000010,
         All					= 0x0000ffff
     };
 
@@ -73,7 +76,6 @@ public:
     bool isEnabled() const;
 
     virtual void add(Feature* aFeature);
-    virtual void add(Feature* aFeature, int Idx);
     virtual void remove(Feature* aFeature);
     virtual void deleteFeature(Feature* aFeature);
     virtual void clear();
@@ -84,14 +86,10 @@ public:
     QList<Feature *> get();
     Feature* get(int i);
     const Feature* get(int i) const;
-    Feature* get(const IFeature::FId& id);
+    virtual Feature* get(const IFeature::FId& id);
     void notifyIdUpdate(const IFeature::FId& id, Feature* aFeature);
 
-    virtual void get(const CoordBox& hz, QList<Feature*>& theFeatures);
-    virtual void getFeatureSet(QMap<RenderPriority, QSet <Feature*> >& theFeatures, QSet<Way*>& theCoastlines, Document* theDocument,
-                       QList<CoordBox>& invalidRects, QRectF& clipRect, Projection& theProjection, QTransform& theTransform);
-
-    void setDocument(Document* aDocument);
+    virtual void setDocument(Document* aDocument);
     Document* getDocument();
 
     LayerWidget* getWidget(void);
@@ -115,8 +113,8 @@ public:
     virtual QString toHtml();
     virtual QString toPropertiesHtml();
 
-    virtual bool toXML(QDomElement& e, bool asTemplate, QProgressDialog * progress);
-    static Layer* fromXML(Layer* l, Document* d, const QDomElement e, QProgressDialog * progress);
+    virtual bool toXML(QXmlStreamWriter& stream, bool asTemplate, QProgressDialog * progress);
+    static Layer* fromXML(Layer* l, Document* d, QXmlStreamReader& stream, QProgressDialog * progress);
 
     virtual CoordBox boundingBox();
 
@@ -133,24 +131,11 @@ public:
     virtual bool isUploadable() const;
     virtual bool isReadonly() const;
     virtual bool isTrack() const {return false;}
-    virtual bool arePointsDrawable() {return true;}
-
-    virtual void blockIndexing(bool val);
-    virtual bool isIndexingBlocked();
-
-    virtual void indexAdd(const CoordBox& bb, const MapFeaturePtr aFeat);
-    virtual void indexRemove(const CoordBox& bb, const MapFeaturePtr aFeat);
-//    bool __cdecl indexFindCallback(MapFeaturePtr data, void* ctxt);
-    virtual const QList<MapFeaturePtr>& indexFind(const CoordBox& vp);
-    virtual void indexFind(const CoordBox& bb, const IndexFindContext& findResult);
-    virtual void reIndex();
-    virtual void reIndex(QProgressDialog * progress);
 
 protected:
     LayerPrivate* p;
     LayerWidget* theWidget;
     mutable QString Id;
-    QList<MapFeaturePtr> findResult;
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(Layer::LayerGroups)
@@ -160,14 +145,15 @@ class DrawingLayer : public Layer
     Q_OBJECT
 
 public:
+    DrawingLayer();
     DrawingLayer(const QString& aName);
     virtual ~DrawingLayer();
 
     virtual LayerWidget* newWidget(void);
 
-    virtual bool toXML(QDomElement& xParent, bool asTemplate, QProgressDialog * progress);
-    static DrawingLayer* fromXML(Document* d, const QDomElement& e, QProgressDialog * progress);
-    static DrawingLayer* doFromXML(DrawingLayer* l, Document* d, const QDomElement e, QProgressDialog * progress);
+    virtual bool toXML(QXmlStreamWriter& stream, bool asTemplate, QProgressDialog * progress);
+    static DrawingLayer* fromXML(Document* d, QXmlStreamReader& stream, QProgressDialog * progress);
+    static DrawingLayer* doFromXML(DrawingLayer* l, Document* d, QXmlStreamReader& stream, QProgressDialog * progress);
 
     virtual /* const */ LayerType classType() const {return Layer::DrawingLayerType;}
     virtual const LayerGroups classGroups() const {return (Layer::Draw);}
@@ -186,19 +172,39 @@ public:
     virtual const QString getFilename();
 
     virtual QString toHtml();
-    virtual bool toXML(QDomElement& xParent, bool asTemplate, QProgressDialog * progress);
-    static TrackLayer* fromXML(Document* d, const QDomElement& e, QProgressDialog * progress);
+    virtual bool toXML(QXmlStreamWriter& stream, bool asTemplate, QProgressDialog * progress);
+    static TrackLayer* fromXML(Document* d, QXmlStreamReader& stream, QProgressDialog * progress);
 
     virtual /* const */ LayerType classType() const {return Layer::TrackLayerType;}
     virtual const LayerGroups classGroups() const {return(Layer::Tracks);}
 
-    virtual bool isUploadable() {return true;}
+    virtual bool isUploadable() const {return true;}
     virtual bool isTrack() const {return true;}
 
 protected:
     QString Filename;
 };
 
+class SpecialLayer : public TrackLayer
+{
+    Q_OBJECT
+public:
+    SpecialLayer(const QString& aName="", LayerType type=Layer::UndefinedType, const QString& filename="");
+
+    virtual LayerWidget* newWidget(void);
+
+    virtual void refreshLayer();
+
+    virtual /* const */ LayerType classType() const {return m_type;}
+    virtual const LayerGroups classGroups() const {return(Layer::Special);}
+
+    virtual bool isUploadable() const {return false;}
+    virtual bool isTrack() const {return true;}
+
+protected:
+    LayerType m_type;
+};
+
 class DirtyLayer : public DrawingLayer
 {
     Q_OBJECT
@@ -206,7 +212,7 @@ public:
     DirtyLayer(const QString& aName);
     virtual ~DirtyLayer();
 
-    static DirtyLayer* fromXML(Document* d, const QDomElement e, QProgressDialog * progress);
+    static DirtyLayer* fromXML(Document* d, QXmlStreamReader& stream, QProgressDialog * progress);
 
     virtual /* const */ LayerType classType() const {return Layer::DirtyLayerType;}
     virtual const LayerGroups classGroups() const {return(Layer::Map|Layer::Draw);}
@@ -224,7 +230,7 @@ public:
     UploadedLayer(const QString& aName);
     virtual ~UploadedLayer();
 
-    static UploadedLayer* fromXML(Document* d, const QDomElement e, QProgressDialog * progress);
+    static UploadedLayer* fromXML(Document* d, QXmlStreamReader& stream, QProgressDialog * progress);
 
     virtual /* const */ LayerType classType() const {return Layer::UploadedLayerType;}
     virtual const LayerGroups classGroups() const {return(Layer::Map|Layer::Draw);}
@@ -241,14 +247,14 @@ public:
     DeletedLayer(const QString& aName);
     virtual ~DeletedLayer();
 
-    virtual bool toXML(QDomElement& xParent, bool asTemplate, QProgressDialog * progress);
-    static DeletedLayer* fromXML(Document* d, const QDomElement& e, QProgressDialog * progress);
+    virtual bool toXML(QXmlStreamWriter& stream, bool asTemplate, QProgressDialog * progress);
+    static DeletedLayer* fromXML(Document* d, QXmlStreamReader& stream, QProgressDialog * progress);
 
     virtual /* const */ LayerType classType() const {return Layer::DeletedLayerType;}
     virtual const LayerGroups classGroups() const {return(Layer::None);}
     virtual LayerWidget* newWidget(void);
 
-    virtual bool isUploadable() {return false;}
+    virtual bool isUploadable() const {return false;}
     virtual bool canDelete() const { return false; }
 };
 
@@ -259,8 +265,8 @@ public:
     FilterLayer(const QString& aId, const QString& aName, const QString& aFilter);
     virtual ~FilterLayer();
 
-    bool toXML(QDomElement& e, bool asTemplate, QProgressDialog * progress);
-    static FilterLayer* fromXML(Document* d, const QDomElement e, QProgressDialog * progress);
+    bool toXML(QXmlStreamWriter& stream, bool asTemplate, QProgressDialog * progress);
+    static FilterLayer* fromXML(Document* d, QXmlStreamReader& stream, QProgressDialog * progress);
 
     virtual /* const */ LayerType classType() const {return Layer::FilterLayerType;}
     virtual const LayerGroups classGroups() const {return(Layer::Filters);}
