@@ -6,7 +6,7 @@
 #include "MainWindow.h"
 #include "Document.h"
 #include "Layer.h"
-#include "Preferences/MerkaartorPreferences.h"
+#include "MerkaartorPreferences.h"
 
 #include "IMapAdapterFactory.h"
 #include "IMapAdapter.h"
@@ -17,12 +17,10 @@
 #include <QInputDialog>
 #include <QMessageBox>
 
-#include "Utils/SelectionDialog.h"
+#include "SelectionDialog.h"
 
 #define LINEHEIGHT 25
 
-class OsbLayer;
-
 LayerWidget::LayerWidget(Layer* aLayer, QWidget* aParent)
 : QPushButton(aParent), theLayer(aLayer), ctxMenu(0), closeAction(0), actZoom(0), associatedMenu(0)
 {
@@ -591,6 +589,41 @@ void TrackLayerWidget::extractLayer(bool)
     emit (layerChanged(this, false));
 }
 
+// SpecialLayerWidget
+
+SpecialLayerWidget::SpecialLayerWidget(SpecialLayer* aLayer, QWidget* aParent)
+    : LayerWidget(aLayer, aParent)
+{
+    initActions();
+}
+
+void SpecialLayerWidget::initActions()
+{
+    LayerWidget::initActions();
+    ctxMenu->addSeparator();
+    associatedMenu->addSeparator();
+
+    QAction* actExtract = new QAction(tr("Refresh layer"), ctxMenu);
+    ctxMenu->addAction(actExtract);
+    associatedMenu->addAction(actExtract);
+    connect(actExtract, SIGNAL(triggered(bool)), this, SLOT(refreshLayer(bool)));
+
+    closeAction = new QAction(tr("Close"), this);
+    connect(closeAction, SIGNAL(triggered()), this, SLOT(close()));
+    ctxMenu->addAction(closeAction);
+    associatedMenu->addAction(closeAction);
+    closeAction->setEnabled(theLayer->canDelete());
+}
+
+SpecialLayerWidget::~SpecialLayerWidget()
+{
+}
+
+void SpecialLayerWidget::refreshLayer(bool)
+{
+    dynamic_cast<SpecialLayer*>(theLayer.data())->refreshLayer();
+}
+
 // DirtyLayerWidget
 
 DirtyLayerWidget::DirtyLayerWidget(DirtyLayer* aLayer, QWidget* aParent)
@@ -637,28 +670,6 @@ void UploadedLayerWidget::initActions()
     closeAction->setEnabled(theLayer->canDelete());
 }
 
-// OsbLayerWidget
-
-OsbLayerWidget::OsbLayerWidget(OsbLayer* aLayer, QWidget* aParent)
-    : LayerWidget(aLayer, aParent)
-{
-    initActions();
-}
-
-void OsbLayerWidget::initActions()
-{
-    LayerWidget::initActions();
-
-    ctxMenu->addSeparator();
-    associatedMenu->addSeparator();
-
-    closeAction = new QAction(tr("Close"), this);
-    connect(closeAction, SIGNAL(triggered()), this, SLOT(close()));
-    ctxMenu->addAction(closeAction);
-    associatedMenu->addAction(closeAction);
-    closeAction->setEnabled(theLayer->canDelete());
-}
-
 // FilterLayerWidget
 
 FilterLayerWidget::FilterLayerWidget(FilterLayer* aLayer, QWidget* aParent)
