@@ -10,12 +10,8 @@
 #include "MainWindow.h"
 #include "Preferences/MerkaartorPreferences.h"
 #include "revision.h"
-#ifdef USE_PROJ
 #include "proj_api.h"
-#endif
-#ifdef USE_GDAL
 #include "gdal_version.h"
-#endif
 
 #include "IMapAdapterFactory.h"
 
@@ -80,12 +76,12 @@ void myMessageOutput(QtMsgType msgType, const char *buf)
 void showVersion()
 {
     QString o;
-    o = QString("Merkaartor %1%2(%3)\n").arg(STRINGIFY(VERSION)).arg(STRINGIFY(REVISION)).arg(STRINGIFY(SVNREV));
+    o = QString("%1 %2%3(%4)\n").arg(STRINGIFY(PRODUCT)).arg(STRINGIFY(VERSION)).arg(STRINGIFY(REVISION)).arg(STRINGIFY(SVNREV));
     fprintf(stdout, "%s", o.toLatin1().data());
     o = QString("using QT version %1 (built with %2)\n").arg(qVersion()).arg(QT_VERSION_STR);
     fprintf(stdout, "%s", o.toLatin1().data());
     fprintf(stdout, "Copyright Bart Vanhauwaert, Chris Browet and others, 2006-2010\n");
-    fprintf(stdout, "This program is licensed under the GNU Public License v2\n");
+    fprintf(stdout, "This program is licensed under the Version 2 of the GNU Public License or any later version\n");
 }
 
 void showHelp()
@@ -97,8 +93,8 @@ void showHelp()
     fprintf(stdout, "  -h, --help\t\tShow help information\n");
     fprintf(stdout, "  -l, --log logfilename\t\tSave debugging information to file \"logfilename\"\n");
     fprintf(stdout, "  -v, --version\t\tShow version information\n");
-    fprintf(stdout, "  -n, --noreuse\t\tDo not reuse an existing Merkaartor instance\n");
-    fprintf(stdout, "  -p, --portable\t\tExecute Merkaartor as a portable application (all files saved in the application directory)\n");
+    fprintf(stdout, "  -n, --noreuse\t\tDo not reuse an existing instance\n");
+    fprintf(stdout, "  -p, --portable\t\tExecute as a portable application (all files saved in the application directory)\n");
     fprintf(stdout, "  --enable_special_layers\t\tEnable old style \"Dirty\" and \"Uploaded\" layers\n");
     fprintf(stdout, "  --importag-tags-as-is\t\tDo not add underscores to imported tags (allow for immediate upload)\n");
     fprintf(stdout, "  --ignore-preferences\t\tIgnore saved preferences\n");
@@ -147,7 +143,7 @@ int main(int argc, char** argv)
 #ifdef FRISIUS_BUILD
     QCoreApplication::setApplicationName("Frisius");
 #else
-        QCoreApplication::setApplicationName("Merkaartor");
+    QCoreApplication::setApplicationName("Merkaartor");
 #endif
     QString message = argsOut.join("$");
     if (reuse)
@@ -177,13 +173,9 @@ int main(int argc, char** argv)
 
     qDebug() << "**** " << QDateTime::currentDateTime().toString(Qt::ISODate) << " -- Starting " << USER_AGENT;
     qDebug() <<	"-------" << QString("using QT version %1 (built with %2)").arg(qVersion()).arg(QT_VERSION_STR);
-#ifdef USE_PROJ
     QString projVer = QString(STRINGIFY(PJ_VERSION));
     qDebug() <<	"-------" << QString("using PROJ4 version %1.%2.%3").arg(projVer.left(1)).arg(projVer.mid(1, 1)).arg(projVer.right(1));
-#endif
-#ifdef USE_GDAL
     qDebug() <<	"-------" << QString("using GDAL version %1").arg(GDAL_RELEASE_NAME);
-#endif
     qDebug() << "-------" << "with arguments: " << QCoreApplication::arguments();
 
 #ifdef _MOBILE
@@ -193,7 +185,8 @@ int main(int argc, char** argv)
 #endif
 
     qApp->setStyleSheet(
-            " LayerWidget { color: black; border: 1px solid black; min-height: 20px}"
+            " LayerWidget { border: 1px solid black; min-height: 20px}"
+            " LayerWidget QLineEdit { color: black; }"
 
             " LayerWidget QCheckBox::indicator:checked { image: url(:Icons/eye.xpm); }"
             " LayerWidget QCheckBox::indicator:unchecked { image: url(:Icons/empty.xpm); }"
@@ -217,9 +210,15 @@ int main(int argc, char** argv)
     splash.showMessage(QString(instance.translate("Main", "%1 v%2%3(%4)\nLoading plugins...")).arg(qApp->applicationName()).arg(STRINGIFY(VERSION)).arg(STRINGIFY(REVISION)).arg(STRINGIFY(SVNREV)), Qt::AlignBottom | Qt::AlignHCenter, Qt::black);
     instance.processEvents();
 
-    if (!g_Merk_Portable)
+    if (!g_Merk_Portable) {
+#ifdef Q_OS_MAC
+        if (!QDir::home().exists(HOMEDIR))
+            QDir::home().mkpath(HOMEDIR);
+#else
         if (!QDir::home().exists("." + qApp->applicationName().toLower()))
             QDir::home().mkdir("." + qApp->applicationName().toLower());
+#endif
+    }
 #if defined(Q_OS_WIN32)
     QDir pluginsDir = QDir(qApp->applicationDirPath() + "/" + STRINGIFY(PLUGINS_DIR));
 
@@ -228,6 +227,8 @@ int main(int argc, char** argv)
     QDir pluginsDir = QDir(qApp->applicationDirPath());
     pluginsDir.cdUp();
     pluginsDir.cd("plugins");
+#elif defined(Q_OS_SYMBIAN)
+    QDir pluginsDir(QLibraryInfo::location(QLibraryInfo::PluginsPath));
 #else
     QDir pluginsDir = (g_Merk_Portable ? QDir(qApp->applicationDirPath() + "/plugins") : QDir(STRINGIFY(PLUGINS_DIR)));
 #endif
@@ -267,7 +268,16 @@ int main(int argc, char** argv)
 
     splash.finish(&Main);
 
-    int x = instance.exec();
+    int x;
+    try {
+        x = instance.exec();
+    } catch (const std::bad_alloc &) {
+        qDebug() << "Out of memory";
+        x = 254;
+    } catch (...) {
+        qDebug() << "Exception";
+        x = 255;
+    }
 
     qDebug() << "**** " << QDateTime::currentDateTime().toString(Qt::ISODate) << " -- Ending " << QString("%1 %2%3(%4)").arg(qApp->applicationName()).arg(STRINGIFY(VERSION)).arg(STRINGIFY(REVISION)).arg(STRINGIFY(SVNREV));
     if(pLogFile) {
