@@ -1,6 +1,7 @@
 #ifndef MERKATOR_MAINWINDOW_H_
 #define MERKATOR_MAINWINDOW_H_
 
+#include "IMerkMainWindow.h"
 #include "IProgressWindow.h"
 
 #include <QtGui/QMainWindow>
@@ -16,6 +17,7 @@ class MainWindowPrivate;
 class LayerDock;
 class Document;
 class Layer;
+class ImageMapLayer;
 class MapView;
 class Feature;
 class PropertiesDock;
@@ -23,6 +25,7 @@ class InfoDock;
 class DirtyDock;
 class FeaturesDock;
 class QGPS;
+class GlobalPainter;
 class Painter;
 class TrackLayer;
 class TrackSegment;
@@ -34,7 +37,7 @@ class PreferencesDialog;
 class GeoImageDock;
 #endif
 
-class MainWindow : public QMainWindow, public IProgressWindow
+class MainWindow : public QMainWindow, public IMerkMainWindow, public IProgressWindow
 {
     Q_OBJECT
 
@@ -45,8 +48,11 @@ public:
     MainWindow(QWidget *parent = 0);
     virtual ~MainWindow();
 
+    bool eventFilter ( QObject * watched, QEvent * event );
+
 public slots:
     void invalidateView(bool UpdateDock = true);
+    void toggleButtonStyle();
 
     virtual void on_createRoundaboutAction_triggered();
     virtual void on_createPolygonAction_triggered();
@@ -70,11 +76,12 @@ public slots:
     virtual void on_layersNewImageAction_triggered();
     virtual void on_layersNewDrawingAction_triggered();
     virtual void on_layersNewFilterAction_triggered();
+    virtual void on_layersOpenstreetbugsAction_triggered();
+    virtual void on_layersMapdustAction_triggered();
 
     virtual void on_fileNewAction_triggered();
     virtual void on_fileDownloadAction_triggered();
     virtual void on_fileDownloadMoreAction_triggered();
-    virtual void on_fileDownloadOpenstreetbugsAction_triggered();
     virtual void on_fileUploadAction_triggered();
     virtual void on_fileImportAction_triggered();
     virtual void on_fileOpenAction_triggered();
@@ -124,8 +131,11 @@ public slots:
     virtual void on_roadSubdivideAction_triggered();
     virtual void on_roadAxisAlignAction_triggered();
     virtual void on_roadExtrudeAction_triggered();
+    virtual void on_roadBingExtractAction_triggered();
 
     virtual void on_featureSelectChildrenAction_triggered();
+    virtual void on_featureSelectParentsAction_triggered();
+    virtual void on_featureDownloadMissingChildrenAction_triggered();
     virtual void on_featureDeleteAction_triggered();
     virtual void on_featureCommitAction_triggered();
     virtual void on_featureOsbClose_triggered();
@@ -136,23 +146,24 @@ public slots:
     virtual void on_nodeDetachAction_triggered();
     virtual void on_relationAddMemberAction_triggered();
     virtual void on_relationRemoveMemberAction_triggered();
+    virtual void on_relationAddToMultipolygonAction_triggered();
     virtual void on_areaJoinAction_triggered();
     virtual void on_areaSplitAction_triggered();
     virtual void on_areaTerraceAction_triggered();
 
     virtual void on_mapStyleSaveAction_triggered();
+    virtual void on_mapStyleSaveAsAction_triggered();
     virtual void on_mapStyleLoadAction_triggered();
     virtual void on_exportOSMAction_triggered();
-    virtual void on_exportOSMBinAction_triggered();
     virtual void on_exportOSCAction_triggered();
     virtual void on_exportGPXAction_triggered();
     virtual void on_exportKMLAction_triggered();
+    virtual void on_exportGDALAction_triggered();
     virtual void on_editSelectAction_triggered();
     virtual void on_bookmarkAddAction_triggered();
     virtual void on_bookmarkRemoveAction_triggered();
 
     virtual void on_toolsPreferencesAction_triggered() {toolsPreferencesAction_triggered();}
-    virtual void on_toolsWorldOsbAction_triggered();
     virtual void on_toolsShortcutsAction_triggered();
     virtual void on_toolsToolbarsAction_triggered();
     virtual void on_toolsWMSServersAction_triggered();
@@ -161,6 +172,7 @@ public slots:
     virtual void on_toolsFiltersAction_triggered();
     virtual void toolsPreferencesAction_triggered(bool focusData=false);
     virtual void on_toolsResetDiscardableAction_triggered();
+    virtual void on_toolsRebuildHistoryAction_triggered();
 
     virtual void on_windowPropertiesAction_triggered();
     virtual void on_windowLayersAction_triggered();
@@ -189,6 +201,13 @@ public slots:
     virtual void on_toolTemplatesMergeAction_triggered();
     virtual void on_toolTemplatesLoadAction_triggered();
 
+
+protected slots:
+    void onCustomcontextmenurequested(const QPoint &pos);
+    void onImagerequested(ImageMapLayer*);
+    void onImagereceived(ImageMapLayer* aLayer);
+    void onLoadingfinished(ImageMapLayer*);
+
 signals:
     void remove_triggered();
     void move_triggered();
@@ -233,9 +252,9 @@ public slots:
     void projectionTriggered(QAction* anAction);
 #endif
     void styleTriggered(QAction* anAction);
-    void updateGpsPosition(float latitude, float longitude, QDateTime time, float altitude, float speed, float heading);
+    void updateGpsPosition(qreal latitude, qreal longitude, QDateTime time, qreal altitude, qreal speed, qreal heading);
     void applyStyles(QString NewStyle);
-    void applyPainters(QList<Painter>* thePainters);
+    void applyPainters(GlobalPainter* theGlobalPainter, QList<Painter>* thePainters);
 
 public:
     bool importFiles(Document * mapDocument, const QStringList & filesNames, QStringList * importedFileNames = NULL);
@@ -257,6 +276,8 @@ public:
     bool hasUnsavedChanges();
     void syncOSM(const QString &aWeb, const QString &aUser, const QString &aPwd);
 
+    void launchInteraction(Interaction *anInteraction);
+
 protected:
     MapView* theView;
     Document* theDocument;
@@ -282,10 +303,11 @@ private slots:
     void setAreaOpacity(QAction*);
     void updateBookmarksMenu();
     void updateWindowMenu(bool b=false);
-    void mapView_interactionChanged(Interaction* anInteraction);
     void incomingLocalConnection();
     void readLocalConnection();
 
+    void on_viewWireframeAction_toggled(bool arg1);
+
 private:
     void updateMenu();
     void updateRecentOpenMenu();
@@ -299,6 +321,7 @@ private:
 
 protected:
     void closeEvent(QCloseEvent * event);
+    virtual QMenu * createPopupMenu ();
 };
 
 #endif
