@@ -1,11 +1,11 @@
 #include "Global.h"
 
-#include "PaintStyle/PaintStyleEditor.h"
-#include "PaintStyle/Painter.h"
+#include "PaintStyleEditor.h"
+#include "Painter.h"
 #include "MainWindow.h"
 #include "Document.h"
 
-#include "Utils/SelectionDialog.h"
+#include "SelectionDialog.h"
 
 #include <QtGui/QCheckBox>
 #include <QtGui/QColorDialog>
@@ -38,10 +38,12 @@ PaintStyleEditor::PaintStyleEditor(QWidget *aParent, const GlobalPainter& aGloba
     BackgroundColor->setIconSize(QSize(36, 18));
     ForegroundColor->setIconSize(QSize(36, 18));
     TouchupColor->setIconSize(QSize(36, 18));
+    TrafficDirectionMarksColor->setIconSize(QSize(36, 18));;
     FillColor->setIconSize(QSize(36, 18));
     LabelColor->setIconSize(QSize(36, 18));
     LabelBackgroundlColor->setIconSize(QSize(36, 18));
     GlobalBackgroundColor->setIconSize(QSize(36, 18));
+    GlobalNodesColor->setIconSize(QSize(36, 18));
 
     updatePaintList();
 
@@ -52,6 +54,13 @@ PaintStyleEditor::PaintStyleEditor(QWidget *aParent, const GlobalPainter& aGloba
     makeBoundaryIcon(GlobalBackgroundColor, theGlobalPainter.getBackgroundColor());
     on_PaintList_itemSelectionChanged();
 
+    DrawGlobalNodes->setChecked(theGlobalPainter.getDrawNodes());
+    makeBoundaryIcon(GlobalNodesColor, theGlobalPainter.getNodesColor());
+    GlobalNodesProportional->setEnabled(theGlobalPainter.getDrawNodes());
+    GlobalNodesFixed->setEnabled(theGlobalPainter.getDrawNodes());
+    GlobalNodesProportional->setValue(theGlobalPainter.NodesProportional);
+    GlobalNodesFixed->setValue(theGlobalPainter.NodesFixed);
+
     FreezeUpdate = false;
 
     resize(1, 1);
@@ -119,9 +128,11 @@ void PaintStyleEditor::on_RemoveButton_clicked()
         return;
     thePainters.erase(thePainters.begin() + idx);
     delete PaintList->takeItem(idx);
+
     if (idx && (idx >= thePainters.size()))
         --idx;
     PaintList->setCurrentRow(idx);
+    updatePaintList();
     on_PaintList_itemSelectionChanged();
 }
 
@@ -195,6 +206,8 @@ void PaintStyleEditor::on_PaintList_itemSelectionChanged()
     ProportionalBackground->setValue(FP.backgroundBoundary().Proportional);
     FixedBackground->setValue(FP.backgroundBoundary().Fixed);
     makeBoundaryIcon(BackgroundColor, FP.backgroundBoundary().Color);
+    BackgroundInterior->setChecked(FP.getBackgroundInterior());
+    BackgroundExterior->setChecked(FP.getBackgroundExterior());
     DrawForeground->setChecked(FP.foregroundBoundary().Draw);
     ProportionalForeground->setValue(FP.foregroundBoundary().Proportional);
     FixedForeground->setValue(FP.foregroundBoundary().Fixed);
@@ -209,6 +222,8 @@ void PaintStyleEditor::on_PaintList_itemSelectionChanged()
     TouchupDashed->setChecked(FP.touchupBoundary().Dashed);
     TouchupDashOn->setValue(FP.touchupBoundary().DashOn);
     TouchupDashOff->setValue(FP.touchupBoundary().DashOff);
+    DrawTrafficDirectionMarks->setChecked(FP.DrawTrafficDirectionMarks);
+    makeBoundaryIcon(TrafficDirectionMarksColor, FP.TrafficDirectionMarksColor);
     DrawFill->setChecked(FP.fillColor().isValid());
     makeBoundaryIcon(FillColor, FP.fillColor());
     DrawFillIcon->setChecked(FP.ForegroundFillUseIcon);;
@@ -271,7 +286,7 @@ void PaintStyleEditor::on_LowerZoomBoundary_valueChanged()
     if (idx >= thePainters.size())
         return;
     Painter& FP(thePainters[idx]);
-    QPair<double, double> Result(0, 0);
+    QPair<qreal, qreal> Result(0, 0);
     if (LowerZoomBoundary->value() > 10e-6)
         Result.first = 1 / LowerZoomBoundary->value();
     if (UpperZoomBoundary->value() > 10e-6)
@@ -304,6 +319,34 @@ void PaintStyleEditor::on_GlobalBackgroundColor_clicked()
     }
 }
 
+void PaintStyleEditor::on_DrawGlobalNodes_clicked(bool b)
+{
+    theGlobalPainter.nodesActive(b);
+}
+
+void PaintStyleEditor::on_GlobalNodesColor_clicked()
+{
+    QColor rgb = QColorDialog::getColor(theGlobalPainter.getNodesColor(), this
+#if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
+                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
+#endif
+                                        );
+    if (rgb.isValid()) {
+        makeBoundaryIcon(GlobalNodesColor, rgb);
+        theGlobalPainter.nodes(rgb);
+    }
+}
+
+void PaintStyleEditor::on_GlobalNodesProportional_valueChanged()
+{
+    theGlobalPainter.NodesProportional = GlobalNodesProportional->value();
+}
+
+void PaintStyleEditor::on_GlobalNodesFixed_valueChanged()
+{
+    theGlobalPainter.NodesFixed = GlobalNodesFixed->value();
+}
+
 void PaintStyleEditor::on_DrawBackground_clicked(bool b)
 {
     QListWidgetItem* it = PaintList->currentItem();
@@ -365,6 +408,39 @@ void PaintStyleEditor::on_DrawForeground_clicked(bool b)
     updatePagesIcons();
 }
 
+void PaintStyleEditor::on_BackgroundInterior_clicked(bool b)
+{
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
+    if (idx >= thePainters.size())
+        return;
+    thePainters[idx].BackgroundInterior = b;
+
+    if (b) {
+        BackgroundExterior->blockSignals(true);
+        BackgroundExterior->setChecked(false);
+        thePainters[idx].BackgroundExterior = false;
+        BackgroundExterior->blockSignals(false);
+    }
+}
+
+void PaintStyleEditor::on_BackgroundExterior_clicked(bool b)
+{
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
+    if (idx >= thePainters.size())
+        return;
+    thePainters[idx].BackgroundExterior = b;
+
+    if (b) {
+        BackgroundInterior->blockSignals(true);
+        BackgroundInterior->setChecked(false);
+        thePainters[idx].BackgroundInterior = false;
+        BackgroundInterior->blockSignals(false);
+    }
+}
 
 void PaintStyleEditor::on_ForegroundColor_clicked()
 {
@@ -473,6 +549,37 @@ void PaintStyleEditor::on_TouchupColor_clicked()
     }
 }
 
+void PaintStyleEditor::on_DrawTrafficDirectionMarks_clicked(bool b)
+{
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
+    if (idx >= thePainters.size())
+        return;
+    thePainters[idx].drawTrafficDirectionMarks(b);
+
+    updatePagesIcons();
+}
+
+void PaintStyleEditor::on_TrafficDirectionMarksColor_clicked()
+{
+    QListWidgetItem* it = PaintList->currentItem();
+    int idx = it->data(Qt::UserRole).toInt();
+
+    if (idx >= thePainters.size())
+        return;
+    Painter& FP(thePainters[idx]);
+    QColor rgb = QColorDialog::getColor(FP.TrafficDirectionMarksColor, this
+#if (QT_VERSION >= QT_VERSION_CHECK(4, 5, 0))
+                                        , tr("Select Color"), QColorDialog::ShowAlphaChannel
+#endif
+                                        );
+    if (rgb.isValid()) {
+        makeBoundaryIcon(TrafficDirectionMarksColor, rgb);
+        FP.TrafficDirectionMarksColor = rgb;
+    }
+}
+
 void PaintStyleEditor::on_ProportionalTouchup_valueChanged()
 {
     if (FreezeUpdate)
@@ -726,7 +833,7 @@ void PaintStyleEditor::refreshPainter()
 void PaintStyleEditor::on_buttonBox_clicked(QAbstractButton * button)
 {
     if (buttonBox->buttonRole(button) == QDialogButtonBox::ApplyRole) {
-        emit(stylesApplied(&thePainters));
+        emit(stylesApplied(&theGlobalPainter, &thePainters));
     }
 }
 
