@@ -20,12 +20,14 @@
 
 
 #include "MainWindow.h"
+#ifndef _MOBILE
 #include <ui_MainWindow.h>
+#endif
 #include "MapView.h"
 
 #include "IMapAdapterFactory.h"
-#include "PaintStyle/IPaintStyle.h"
-#include "PaintStyle/MasPaintStyle.h"
+#include "IPaintStyle.h"
+#include "MasPaintStyle.h"
 
 
 #define M_PARAM_IMPLEMENT_BOOL(Param, Category, Default) \
@@ -137,14 +139,14 @@
 
 #define M_PARAM_IMPLEMENT_DOUBLE(Param, Category, Default) \
     bool mb_##Param = false; \
-    void MerkaartorPreferences::set##Param(double theValue) \
+    void MerkaartorPreferences::set##Param(qreal theValue) \
     { \
         m_##Param = theValue; \
         if (!g_Merk_Ignore_Preferences) { \
             Sets->setValue(#Category"/"#Param, theValue); \
         } \
     } \
-    double MerkaartorPreferences::get##Param() \
+    qreal MerkaartorPreferences::get##Param() \
     { \
         if (!::mb_##Param) { \
             ::mb_##Param = true; \
@@ -348,7 +350,7 @@ void MerkaartorPreferences::fromOsmPref()
 
     if (getOsmUser().isEmpty() || getOsmPassword().isEmpty()) return;
 
-    QUrl osmWeb(getOsmWebsite());
+    QUrl osmWeb(getOsmApiUrl());
     if (osmWeb.port() == -1)
         osmWeb.setPort(80);
 
@@ -457,7 +459,7 @@ void MerkaartorPreferences::on_requestFinished ( int id, bool error )
 
 void MerkaartorPreferences::putOsmPref(const QString& k, const QString& v)
 {
-    QUrl osmWeb(getOsmWebsite());
+    QUrl osmWeb(getOsmApiUrl());
     if (osmWeb.port() == -1)
         osmWeb.setPort(80);
 
@@ -483,7 +485,7 @@ void MerkaartorPreferences::putOsmPref(const QString& k, const QString& v)
 
 void MerkaartorPreferences::deleteOsmPref(const QString& k)
 {
-    QUrl osmWeb(getOsmWebsite());
+    QUrl osmWeb(getOsmApiUrl());
     if (osmWeb.port() == -1)
         osmWeb.setPort(80);
 
@@ -555,8 +557,8 @@ void MerkaartorPreferences::initialize()
         sl = Sets->value("downloadosm/bookmarks").toStringList();
         if (sl.size()) {
             for (int i=0; i<sl.size(); i+=5) {
-                Bookmark B(sl[i], CoordBox(Coord(angToCoord(sl[i+1].toDouble()),angToCoord(sl[i+2].toDouble())),
-                                           Coord(angToCoord(sl[i+3].toDouble()),angToCoord(sl[i+4].toDouble()))));
+                Bookmark B(sl[i], CoordBox(Coord(sl[i+2].toDouble(),sl[i+1].toDouble()),
+                                           Coord(sl[i+4].toDouble(),sl[i+3].toDouble())));
                 theBookmarkList.addBookmark(B);
             }
             save();
@@ -623,7 +625,7 @@ const QVector<qreal> MerkaartorPreferences::getParentDashes() const
     return parentDashes;
 }
 
-double MerkaartorPreferences::apiVersionNum() const
+qreal MerkaartorPreferences::apiVersionNum() const
 {
     if (Use06Api)
         return 0.6;
@@ -779,6 +781,8 @@ M_PARAM_IMPLEMENT_STRING(LastSearchTagSelector, search, "");
 
 void MerkaartorPreferences::saveMainWindowState(const MainWindow * mainWindow)
 {
+#ifndef _MOBILE
+
     if (!g_Merk_Ignore_Preferences) {
         //    Sets->setValue("MainWindow/Position", mainWindow->pos());
         //    Sets->setValue("MainWindow/Size", mainWindow->size());
@@ -787,10 +791,12 @@ void MerkaartorPreferences::saveMainWindowState(const MainWindow * mainWindow)
         Sets->setValue("MainWindow/Fullscreen", mainWindow->ui->windowShowAllAction->isEnabled());
         Sets->setValue("MainWindow/FullscreenState", mainWindow->fullscreenState);
     }
+#endif
 }
 
 void MerkaartorPreferences::restoreMainWindowState(MainWindow * mainWindow) const
 {
+#ifndef _MOBILE
     if (!g_Merk_Ignore_Preferences && !g_Merk_Reset_Preferences) {
         //    if (Sets->contains("MainWindow/Position"))
         //        mainWindow->move( Sets->value("MainWindow/Position").toPoint());
@@ -819,6 +825,7 @@ void MerkaartorPreferences::restoreMainWindowState(MainWindow * mainWindow) cons
             mainWindow->ui->windowShowAllAction->setVisible(false);
         }
     }
+#endif
 }
 
 void MerkaartorPreferences::setInitialPosition(MapView* vw)
@@ -826,10 +833,10 @@ void MerkaartorPreferences::setInitialPosition(MapView* vw)
     if (!g_Merk_Ignore_Preferences) {
         QStringList ip;
         CoordBox cb = vw->viewport();
-        ip.append(QString::number(cb.bottomLeft().lat(), 'f', 8));
-        ip.append(QString::number(cb.bottomLeft().lon(), 'f', 8));
-        ip.append(QString::number(cb.topRight().lat(), 'f', 8));
-        ip.append(QString::number(cb.topRight().lon(), 'f', 8));
+        ip.append(QString::number(cb.bottomLeft().y(), 'f', 8));
+        ip.append(QString::number(cb.bottomLeft().x(), 'f', 8));
+        ip.append(QString::number(cb.topRight().y(), 'f', 8));
+        ip.append(QString::number(cb.topRight().x(), 'f', 8));
 
         Sets->setValue("MainWindow/InitialPosition", ip);
         //    Sets->setValue("MainWindow/ViewRect", vw->rect());
@@ -963,6 +970,8 @@ bool MerkaartorPreferences::getDrawTileBoundary()
     return false;
 }
 
+M_PARAM_IMPLEMENT_BOOL(HideToolbarLabels, interface, false)
+
 /* DATA */
 
 QString MerkaartorPreferences::getOsmWebsite() const
@@ -973,7 +982,7 @@ QString MerkaartorPreferences::getOsmWebsite() const
     else
         s = "www.openstreetmap.org";
 
-#if QT_VERSION >= 0x040600
+#if QT_VERSION >= 0x040600 && !defined(FORCE_46)
     QUrl u = QUrl::fromUserInput(s);
 #else
     // convenience for creating a valid URL
@@ -993,6 +1002,17 @@ QString MerkaartorPreferences::getOsmWebsite() const
     u.setScheme("http");
     u.setPath(p);
 #endif
+
+    if (!u.path().isEmpty())
+        u.setPath("");
+
+    return u.toString();
+}
+
+
+QString MerkaartorPreferences::getOsmApiUrl() const
+{
+    QUrl u(getOsmWebsite());
     if (u.path().isEmpty())
         u.setPath("/api/" + apiVersion());
 
@@ -1005,7 +1025,7 @@ void MerkaartorPreferences::setOsmWebsite(const QString & theValue)
         Sets->setValue("osm/Website", theValue);
 }
 
-M_PARAM_IMPLEMENT_STRING(XapiUrl, osm, "http://www.informationfreeway.org/api/0.6/")
+M_PARAM_IMPLEMENT_STRING(XapiUrl, osm, "http://www.informationfreeway.org/api/0.6")
 M_PARAM_IMPLEMENT_STRING(NominatimUrl, osm, "http://nominatim.openstreetmap.org/search")
 M_PARAM_IMPLEMENT_BOOL(AutoHistoryCleanup, data, true);
 
@@ -1189,11 +1209,17 @@ void MerkaartorPreferences::setShortcuts(const QStringList & theValue)
         Sets->setValue("Tools/shortcuts", theValue);
 }
 
-/* Styles */
+M_PARAM_IMPLEMENT_INT(PolygonSides, Tools, 3)
+
+/* Rendering */
+M_PARAM_IMPLEMENT_BOOL(UseAntiAlias, style, true)
+M_PARAM_IMPLEMENT_BOOL(AntiAliasWhilePanning, style, false)
+M_PARAM_IMPLEMENT_BOOL(UseStyledWireframe, style, false)
 M_PARAM_IMPLEMENT_STRING(DefaultStyle, style, ":/Styles/Mapnik.mas")
 M_PARAM_IMPLEMENT_STRING(CustomStyle, style, "")
 M_PARAM_IMPLEMENT_BOOL(DisableStyleForTracks, style, true)
 M_PARAM_IMPLEMENT_STRINGLIST(TechnicalTags, style, TECHNICAL_TAGS)
+M_PARAM_IMPLEMENT_INT(EditRendering, style, 0)
 
 /* Zoom */
 M_PARAM_IMPLEMENT_INT(ZoomIn, zoom, 133)
@@ -1225,6 +1251,7 @@ M_PARAM_IMPLEMENT_BOOL(RelationsVisible, visual, false)
 M_PARAM_IMPLEMENT_BOOL(PhotosVisible, visual, true)
 M_PARAM_IMPLEMENT_BOOL(VirtualNodesVisible, visual, true)
 M_PARAM_IMPLEMENT_BOOL(DirtyVisible, visual, true)
+M_PARAM_IMPLEMENT_BOOL(WireframeView, visual, false)
 
 /* Templates */
 M_PARAM_IMPLEMENT_STRING(DefaultTemplate, templates, ":/Templates/default.mat")
@@ -1252,22 +1279,16 @@ M_PARAM_IMPLEMENT_BOOL(MapTooltip, visual, false)
 M_PARAM_IMPLEMENT_BOOL(InfoOnHover, visual, true)
 M_PARAM_IMPLEMENT_BOOL(ShowParents, visual, true)
 
-M_PARAM_IMPLEMENT_INT_DELAYED(TagListFirstColumnWidth, visual, 0)
+M_PARAM_IMPLEMENT_INT_DELAYED(TagListFirstColumnWidth, visual, 20)
 M_PARAM_IMPLEMENT_BOOL(TranslateTags, locale, true)
 
-/* World OSB manager */
-M_PARAM_IMPLEMENT_DOUBLE(TileToRegionThreshold, WOSB, 0.03)
-M_PARAM_IMPLEMENT_DOUBLE(RegionTo0Threshold, WOSB, 1.)
-
-M_PARAM_IMPLEMENT_STRING(WorldOsbUri, WOSB, "")
-M_PARAM_IMPLEMENT_BOOL(WorldOsbAutoload, WOSB, false)
-M_PARAM_IMPLEMENT_BOOL(WorldOsbAutoshow, WOSB, false)
-
 /* Background */
 M_PARAM_IMPLEMENT_BOOL(AutoSourceTag, backgroundImage, true)
 
 /* Data */
 M_PARAM_IMPLEMENT_STRING(OpenStreetBugsUrl, data, "http://openstreetbugs.schokokeks.org/api/0.1/")
+M_PARAM_IMPLEMENT_STRING(MapdustUrl, data, "http://www.mapdust.com/feed?lang=en&ft=wrong_turn,bad_routing,oneway_road,blocked_street,missing_street,wrong_roundabout,missing_speedlimit,other&fd=1&minR=&maxR=")
+M_PARAM_IMPLEMENT_BOOL(GdalConfirmProjection, data, false)
 M_PARAM_IMPLEMENT_BOOL(HasAutoLoadDocument, data, false)
 M_PARAM_IMPLEMENT_STRING(AutoLoadDocumentFilename, data, "")
 
@@ -1721,6 +1742,7 @@ QString getDefaultLanguage(bool returnDefault)
             Sets = new QSettings(qApp->applicationDirPath() + "/merkaartor.ini", QSettings::IniFormat);
         }
         QString lang = Sets->value("locale/language").toString();
+        delete Sets;
         if (lang == "")
             if (returnDefault)
                 lang = QLocale::system().name().split("_")[0];
