@@ -21,15 +21,15 @@
 #include <QNetworkProxy>
 
 #ifdef USE_LIBPROXY
-#include <libproxy/proxy.h>
+#include <proxy.h>
 #endif
 
-#include "Maps/Coord.h"
-#include "Preferences/WmsServersList.h"
-#include "Preferences/TmsServersList.h"
-#include "Preferences/ProjectionsList.h"
-#include "Preferences/BookmarksList.h"
-#include "Preferences/FilterList.h"
+#include "Coord.h"
+#include "WmsServersList.h"
+#include "TmsServersList.h"
+#include "ProjectionsList.h"
+#include "BookmarksList.h"
+#include "FilterList.h"
 
 #include "IRenderer.h"
 
@@ -53,20 +53,19 @@ class IPaintStyle;
 #endif
 #define USER_AGENT (QString("%1/%2%3 (%4;%5)").arg(qApp->applicationName()).arg(STRINGIFY(VERSION)).arg(STRINGIFY(REVISION)).arg(PLATFORM).arg(STRINGIFY(SVNREV)))
 
-//#define WORLD_COORDBOX CoordBox(Coord(1.3, -1.3), Coord(-1.3, 1.3))
-#define WORLD_COORDBOX CoordBox(Coord(COORD_MAX*.80/2, -COORD_MAX*.80), Coord(-COORD_MAX*.80/2, COORD_MAX*.80))
+#define WORLD_COORDBOX CoordBox(Coord(-COORD_MAX*.80, COORD_MAX*.80/2), Coord(COORD_MAX*.80, -COORD_MAX*.80/2))
 #define BUILTIN_STYLES_DIR ":/Styles"
 #define BUILTIN_TEMPLATES_DIR ":/Templates"
 
 #define M_PREFS MerkaartorPreferences::instance()
 #define M_STYLE MerkaartorPreferences::styleinstance()
 
-#define NONE_ADAPTER_UUID QUuid("{8F5D3625-F987-45c5-A50B-17D88384F97D}")
-#define SHAPE_ADAPTER_UUID QUuid("{AFB0324E-34D0-4267-BB8A-CF56CD2D7012}")
-#define WMS_ADAPTER_UUID QUuid("{E238750A-AC27-429e-995C-A60C17B9A1E0}")
-#define TMS_ADAPTER_UUID QUuid("{CA8A07EC-A466-462b-929F-3805BC9DEC95}")
-
+#ifdef Q_WS_MACX
+// Follow conventions on Mac OS X
+#define HOMEDIR (g_Merk_Portable ? qApp->applicationDirPath() : QDir::homePath() + "/Library/Merkaartor")
+#else
 #define HOMEDIR (g_Merk_Portable ? qApp->applicationDirPath() : QDir::homePath() + "/.merkaartor")
+#endif
 #define SHAREDIR (g_Merk_Portable ? qApp->applicationDirPath() : STRINGIFY(SHARE_DIR))
 #define TEMPLATE_DOCUMENT (HOMEDIR + "/Startup.mdc")
 
@@ -103,10 +102,10 @@ class IPaintStyle;
         int get##Param();
 #define M_PARAM_DECLARE_DOUBLE(Param) \
     private: \
-        double m_##Param; \
+        qreal m_##Param; \
     public: \
-        void set##Param(double theValue); \
-        double get##Param();
+        void set##Param(qreal theValue); \
+        qreal get##Param();
 #define M_PARAM_DECLARE_COLOR(Param) \
     private: \
         QColor m_##Param; \
@@ -182,7 +181,7 @@ public:
     //bool use06Api() const;
     void setUse06Api(bool b);
     const QString apiVersion() const;
-    double apiVersionNum() const;
+    qreal apiVersionNum() const;
 
     M_PARAM_DECLARE_BOOL(rightsidedriving);
     M_PARAM_DECLARE_DOUBLE(doubleroaddistance);
@@ -238,9 +237,12 @@ public:
 
     bool getDrawTileBoundary();
 
+    M_PARAM_DECLARE_BOOL(HideToolbarLabels)
+
     /* Data */
     void setOsmWebsite(const QString & theValue);
     QString getOsmWebsite() const;
+    QString getOsmApiUrl() const;
 
     M_PARAM_DECLARE_STRING(XapiUrl)
     M_PARAM_DECLARE_STRING(NominatimUrl)
@@ -252,10 +254,10 @@ public:
     void setOsmPassword(const QString & theValue);
     QString getOsmPassword() const;
 
-    M_PARAM_DECLARE_DOUBLE(MaxDistNodes);
+    M_PARAM_DECLARE_DOUBLE(MaxDistNodes)
 
-    M_PARAM_DECLARE_BOOL(AutoSaveDoc);
-    M_PARAM_DECLARE_BOOL(AutoExtractTracks);
+    M_PARAM_DECLARE_BOOL(AutoSaveDoc)
+    M_PARAM_DECLARE_BOOL(AutoExtractTracks)
 
     /* Export Type */
     void setExportType(ExportType theValue);
@@ -268,6 +270,8 @@ public:
     QStringList getShortcuts() const;
     void setShortcuts(const QStringList & theValue);
 
+    M_PARAM_DECLARE_INT(PolygonSides)
+
     /* Recent */
     void setRecentOpen(const QStringList & theValue);
     QStringList getRecentOpen() const;
@@ -277,11 +281,15 @@ public:
     QStringList getRecentImport() const;
     void addRecentImport(const QString & theValue);
 
-    /* Styles */
+    /* Rendering */
+    M_PARAM_DECLARE_BOOL(UseAntiAlias)
+    M_PARAM_DECLARE_BOOL(AntiAliasWhilePanning)
+    M_PARAM_DECLARE_BOOL(UseStyledWireframe)
     M_PARAM_DECLARE_STRING(DefaultStyle)
     M_PARAM_DECLARE_STRING(CustomStyle)
     M_PARAM_DECLARE_BOOL(DisableStyleForTracks)
     M_PARAM_DECLARE_STRINGList(TechnicalTags)
+    M_PARAM_DECLARE_INT(EditRendering)
 
     /* Visual */
     M_PARAM_DECLARE_INT(ZoomIn)
@@ -311,6 +319,8 @@ public:
     M_PARAM_DECLARE_BOOL(RelationsVisible)
     M_PARAM_DECLARE_BOOL(PhotosVisible)
     M_PARAM_DECLARE_BOOL(DirtyVisible)
+    M_PARAM_DECLARE_BOOL(WireframeView)
+
 
     /* Templates */
     M_PARAM_DECLARE_STRING(DefaultTemplate)
@@ -335,14 +345,6 @@ public:
     M_PARAM_DECLARE_BOOL(InfoOnHover)
     M_PARAM_DECLARE_BOOL(ShowParents)
 
-    /* World OSB manager */
-    M_PARAM_DECLARE_DOUBLE(TileToRegionThreshold)
-    M_PARAM_DECLARE_DOUBLE(RegionTo0Threshold)
-
-    M_PARAM_DECLARE_STRING(WorldOsbUri)
-    M_PARAM_DECLARE_BOOL(WorldOsbAutoload)
-    M_PARAM_DECLARE_BOOL(WorldOsbAutoshow)
-
     /* Background */
     M_PARAM_DECLARE_BOOL(AutoSourceTag)
 
@@ -377,6 +379,8 @@ public:
 
     /* Data */
     M_PARAM_DECLARE_STRING(OpenStreetBugsUrl)
+    M_PARAM_DECLARE_STRING(MapdustUrl)
+    M_PARAM_DECLARE_BOOL(GdalConfirmProjection)
     M_PARAM_DECLARE_BOOL(HasAutoLoadDocument)
     M_PARAM_DECLARE_STRING(AutoLoadDocumentFilename)
 
@@ -442,7 +446,7 @@ protected:
     bool Use06Api;
     QString version;
     bool RightSideDriving;
-    double DoubleRoadDistance;
+    qreal DoubleRoadDistance;
     QString WorkingDir;
     QString OsmWebsite;
     QString OsmUser;
