@@ -9,8 +9,8 @@
 // Copyright: See COPYING file that comes with this distribution
 //
 //
-#include "Preferences/PreferencesDialog.h"
-#include "PaintStyle/MasPaintStyle.h"
+#include "PreferencesDialog.h"
+#include "MasPaintStyle.h"
 
 #include "MainWindow.h"
 #include "Document.h"
@@ -146,11 +146,11 @@ PreferencesDialog::~PreferencesDialog()
 
 void PreferencesDialog::on_buttonBox_clicked(QAbstractButton * button)
 {
-    if ((button == buttonBox->button(QDialogButtonBox::Apply))) {
+    if (button == buttonBox->button(QDialogButtonBox::Apply)) {
         savePrefs();
         emit(preferencesChanged(this));
     } else
-        if ((button == buttonBox->button(QDialogButtonBox::Ok))) {
+        if (button == buttonBox->button(QDialogButtonBox::Ok)) {
             savePrefs();
             emit(preferencesChanged(this));
             this->accept();
@@ -197,7 +197,7 @@ void PreferencesDialog::loadPrefs()
     if (!theOsmServers->size()) {
         OsmServerWidget* wOSmServer = new OsmServerWidget(grpOSM);
 
-        wOSmServer->edOsmServerUrl->setText(M_PREFS->getOsmWebsite());
+        wOSmServer->edOsmServerUrl->setText(M_PREFS->getOsmApiUrl());
         wOSmServer->edOsmServerUser->setText(M_PREFS->getOsmUser());
         wOSmServer->edOsmServerPwd->setText(M_PREFS->getOsmPassword());
         wOSmServer->rbOsmServerSelected->setChecked(true);
@@ -250,6 +250,10 @@ void PreferencesDialog::loadPrefs()
     edCacheDir->setText(M_PREFS->getCacheDir());
     sbCacheSize->setValue(M_PREFS->getCacheSize());
 
+    cbAntiAlias->setChecked(M_PREFS->getUseAntiAlias());
+    cbDisableAntialiasInPanning->setChecked(!M_PREFS->getAntiAliasWhilePanning());
+    cbDisableAntialiasInPanning->setEnabled(M_PREFS->getUseAntiAlias());
+    cbStyledWireframe->setChecked(M_PREFS->getUseStyledWireframe());
     QString s = M_PREFS->getDefaultStyle();
     QString cs = M_PREFS->getCustomStyle();
     if (QFileInfo(cs).isFile())
@@ -257,6 +261,10 @@ void PreferencesDialog::loadPrefs()
     CustomStylesDir->setText(cs);
     updateStyles();
 
+    rbQuickEdit->setChecked(M_PREFS->getEditRendering() == 0);
+    rbWireframeEdit->setChecked(M_PREFS->getEditRendering() == 1);
+    rbFullEdit->setChecked(M_PREFS->getEditRendering() == 2);
+
     cbDisableStyleForTracks->setChecked(M_PREFS->getDisableStyleForTracks());
 
     QString t = M_PREFS->getDefaultTemplate();
@@ -307,6 +315,7 @@ void PreferencesDialog::loadPrefs()
     cbAutoSaveDoc->setChecked(M_PREFS->getAutoSaveDoc());
     cbAutoExtractTracks->setChecked(M_PREFS->getAutoExtractTracks());
     cbReadonlyTracksDefault->setChecked(M_PREFS->getReadonlyTracksDefault());
+    cbGdalConfirmProjection->setChecked(M_PREFS->getGdalConfirmProjection());
 
     ToolList* tl = M_PREFS->getTools();
     ToolListIterator i(*tl);
@@ -327,7 +336,6 @@ void PreferencesDialog::loadPrefs()
     comboCustomStyle->addItems(QStyleFactory::keys());
     comboCustomStyle->setCurrentIndex(comboCustomStyle->findText(M_PREFS->getMerkaartorStyleString()));
 
-    cbUseShapefileForBackground->setChecked(M_PREFS->getUseShapefileForBackground());
     cbAutoSourceTag->setChecked(M_PREFS->getAutoSourceTag());
 }
 
@@ -359,7 +367,7 @@ void PreferencesDialog::savePrefs()
         srv.Password = wOsmServer->edOsmServerPwd->text();
         srv.Selected = wOsmServer->rbOsmServerSelected->isChecked();
 
-        if (srv.Selected && (srv.Url != M_PREFS->getOsmWebsite() || srv.User != M_PREFS->getOsmUser() || srv.Password != M_PREFS->getOsmPassword())) {
+        if (srv.Selected && (srv.Url != M_PREFS->getOsmApiUrl() || srv.User != M_PREFS->getOsmUser() || srv.Password != M_PREFS->getOsmPassword())) {
             M_PREFS->setOsmWebsite(srv.Url);
             M_PREFS->setOsmUser(srv.User);
             M_PREFS->setOsmPassword(srv.Password);
@@ -398,8 +406,18 @@ void PreferencesDialog::savePrefs()
     M_PREFS->setCacheDir(edCacheDir->text());
     M_PREFS->setCacheSize(sbCacheSize->value());
 
+    M_PREFS->setUseAntiAlias(cbAntiAlias->isChecked());
+    M_PREFS->setAntiAliasWhilePanning(!cbDisableAntialiasInPanning->isChecked());
+    M_PREFS->setUseStyledWireframe(cbStyledWireframe->isChecked());
     M_PREFS->setCustomStyle(CustomStylesDir->text());
 
+    if (rbQuickEdit->isChecked())
+        M_PREFS->setEditRendering(0);
+    else if (rbWireframeEdit->isChecked())
+        M_PREFS->setEditRendering(1);
+    else
+        M_PREFS->setEditRendering(2);
+
     bool PainterToInvalidate = false;
     if (cbDisableStyleForTracks->isChecked() != M_PREFS->getDisableStyleForTracks()) {
         M_PREFS->setDisableStyleForTracks(cbDisableStyleForTracks->isChecked());
@@ -457,6 +475,7 @@ void PreferencesDialog::savePrefs()
     M_PREFS->setAutoSaveDoc(cbAutoSaveDoc->isChecked());
     M_PREFS->setAutoExtractTracks(cbAutoExtractTracks->isChecked());
     M_PREFS->setReadonlyTracksDefault(cbReadonlyTracksDefault->isChecked());
+    M_PREFS->setGdalConfirmProjection(cbGdalConfirmProjection->isChecked());
 
     ToolList* tl = M_PREFS->getTools();
     tl->clear();
@@ -474,7 +493,6 @@ void PreferencesDialog::savePrefs()
     M_PREFS->setMerkaartorStyle(cbCustomStyle->isChecked());
     M_PREFS->setMerkaartorStyleString(comboCustomStyle->currentText());
 
-    M_PREFS->setUseShapefileForBackground(cbUseShapefileForBackground->isChecked());
     M_PREFS->setAutoSourceTag(cbAutoSourceTag->isChecked());
 
     M_PREFS->save(OsmDataChanged);
