@@ -9,7 +9,7 @@
 // Copyright: See COPYING file that comes with this distribution
 //
 //
-#include "Preferences/TMSPreferencesDialog.h"
+#include "TMSPreferencesDialog.h"
 
 #include <QMessageBox>
 #include <QDir>
@@ -49,9 +49,7 @@ void TMSPreferencesDialog::on_btApplyTmsServer_clicked(void)
         return;
 
     QUrl theUrl(edTmsUrl->text());
-    QString theAdress = theUrl.host();
-    if (theUrl.port() != -1)
-        theAdress += ":" + QString::number(theUrl.port());
+    QString theAdress = theUrl.toString(QUrl::RemovePath | QUrl::RemoveQuery | QUrl::RemoveFragment);
 
     TmsServer& WS(theTmsServers[idx]);
 
@@ -86,9 +84,7 @@ void TMSPreferencesDialog::on_btApplyTmsServer_clicked(void)
 void TMSPreferencesDialog::on_btAddTmsServer_clicked(void)
 {
     QUrl theUrl(edTmsUrl->text());
-    QString theAdress = theUrl.host();
-    if (theUrl.port() != -1)
-        theAdress += ":" + QString::number(theUrl.port());
+    QString theAdress = theUrl.toString(QUrl::RemovePath | QUrl::RemoveQuery | QUrl::RemoveFragment);
 
     QString theBaseUrl;
     if (frOSGeo->isVisible()) {
@@ -131,7 +127,12 @@ void TMSPreferencesDialog::on_lvTmsServers_itemSelectionChanged()
 
     TmsServer& WS(theTmsServers[idx]);
     edTmsName->setText(WS.TmsName);
-    edTmsUrl->setText("http://" + WS.TmsAdress + WS.TmsPath);
+    
+    if (WS.TmsAdress.contains("://")) {
+        edTmsUrl->setText(WS.TmsAdress + WS.TmsPath);
+    } else {
+        edTmsUrl->setText("http://" + WS.TmsAdress + WS.TmsPath);
+    }
     sbTileSize->setValue(WS.TmsTileSize);
     sbMinZoom->setValue(WS.TmsMinZoom);
     sbMaxZoom->setValue(WS.TmsMaxZoom);
@@ -178,10 +179,10 @@ void TMSPreferencesDialog::setSelectedServer(QString theValue)
 
 void TMSPreferencesDialog::on_buttonBox_clicked(QAbstractButton * button)
 {
-    if ((button == buttonBox->button(QDialogButtonBox::Apply))) {
+    if (button == buttonBox->button(QDialogButtonBox::Apply)) {
         savePrefs();
     } else
-        if ((button == buttonBox->button(QDialogButtonBox::Ok))) {
+        if (button == buttonBox->button(QDialogButtonBox::Ok)) {
             savePrefs();
             this->accept();
         }
@@ -323,11 +324,11 @@ void TMSPreferencesDialog::httpRequestFinished(int /*id*/, bool error)
             } else if (c.nodeName() == "SRS") {
                 srs = c.firstChild().toText().nodeValue();
             } else if (c.nodeName() == "BoundingBox") {
-                Coord bl(angToCoord(c.attribute("miny").toDouble()), angToCoord(c.attribute("minx").toDouble()));
-                Coord tr(angToCoord(c.attribute("maxy").toDouble()), angToCoord(c.attribute("maxx").toDouble()));
+                Coord bl(c.attribute("minx").toDouble(), c.attribute("miny").toDouble());
+                Coord tr(c.attribute("maxx").toDouble(), c.attribute("maxy").toDouble());
                 bbox = CoordBox(bl, tr);
             } else if (c.nodeName() == "Origin") {
-                Coord pt(angToCoord(c.attribute("y").toDouble()), angToCoord(c.attribute("x").toDouble()));
+                Coord pt(c.attribute("x").toDouble(), c.attribute("y").toDouble());
                 origin = pt;
             } else if (c.nodeName() == "TileFormat") {
                 tilesize.setWidth(c.attribute("width").toInt());
@@ -354,7 +355,7 @@ void TMSPreferencesDialog::httpRequestFinished(int /*id*/, bool error)
         ts.BBox = bbox;
         ts.Format = tileformat;
         Coord center = bbox.center();
-        if (origin.lat() < center.lat())
+        if (origin.y() < center.y())
             ts.Origin = true;
         else
             ts.Origin = false;
