@@ -13,7 +13,7 @@
 #include <QApplication>
 
 #include "TmsServersList.h"
-#include "Preferences/MerkaartorPreferences.h"
+#include "MerkaartorPreferences.h"
 
 TmsServer::TmsServer()
 {
@@ -148,7 +148,7 @@ void TmsServersList::toXml(QDomElement parent)
 {
     QDomElement rt = parent.ownerDocument().createElement("TmsServers");
     parent.appendChild(rt);
-    rt.setAttribute("creator", QString("Merkaartor v%1%2").arg(STRINGIFY(VERSION)).arg(STRINGIFY(REVISION)));
+    rt.setAttribute("creator", QString("%1 v%2%3").arg(STRINGIFY(PRODUCT)).arg(STRINGIFY(VERSION)).arg(STRINGIFY(REVISION)));
 
     TmsServerListIterator it(theServers);
     while (it.hasNext()) {
