@@ -9,7 +9,7 @@
 // Copyright: See COPYING file that comes with this distribution
 //
 //
-#include "Preferences/WMSPreferencesDialog.h"
+#include "WMSPreferencesDialog.h"
 
 #include <QMessageBox>
 #include <QDir>
@@ -305,10 +305,10 @@ void WMSPreferencesDialog::setSelectedServer(QString theValue)
 
 void WMSPreferencesDialog::on_buttonBox_clicked(QAbstractButton * button)
 {
-    if ((button == buttonBox->button(QDialogButtonBox::Apply))) {
+    if (button == buttonBox->button(QDialogButtonBox::Apply)) {
         savePrefs();
     } else
-        if ((button == buttonBox->button(QDialogButtonBox::Ok))) {
+        if (button == buttonBox->button(QDialogButtonBox::Ok)) {
             savePrefs();
             this->accept();
         }
@@ -449,7 +449,7 @@ void WMSPreferencesDialog::httpRequestFinished(QNetworkReply * reply)
     styleList.clear();
     QDomElement layElem = capElem.firstChildElement("Layer");
     while(!layElem.isNull()) {
-        QTreeWidgetItem* it = parseLayers(layElem, NULL);
+        QTreeWidgetItem* it = parseLayer(layElem, NULL);
         tvWmsLayers->addTopLevelItem(it);
         tvWmsLayers->expandItem(it);
 
@@ -537,7 +537,7 @@ void WMSPreferencesDialog::parseTileSet(QDomElement &tilesetElem, WmscLayer &aLa
     }
 }
 
-QTreeWidgetItem * WMSPreferencesDialog::parseLayers(const QDomElement& aLayerElem,
+QTreeWidgetItem * WMSPreferencesDialog::parseLayer(const QDomElement& aLayerElem,
                               QTreeWidgetItem* aLayerItem)
 {
     if (aLayerElem.tagName() != "Layer")
@@ -565,11 +565,12 @@ QTreeWidgetItem * WMSPreferencesDialog::parseLayers(const QDomElement& aLayerEle
         theName = name.firstChild().nodeValue();
         newItem->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsSelectable | Qt::ItemIsEnabled);
         newItem->setData(0, Qt::UserRole, theName);
+        newItem->setCheckState(0, Qt::Unchecked);
         foreach (QString s, edWmsLayers->text().split(',')) {
-            if (theName == s)
+            if (theName == s) {
                 newItem->setCheckState(0, Qt::Checked);
-            else
-                newItem->setCheckState(0, Qt::Unchecked);
+                break;
+            }
         }
     } else {
 //        newItem->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsSelectable | Qt::ItemIsEnabled);
@@ -606,7 +607,7 @@ QTreeWidgetItem * WMSPreferencesDialog::parseLayers(const QDomElement& aLayerEle
 
     QDomElement layElem = aLayerElem.firstChildElement("Layer");
     while(!layElem.isNull()) {
-        parseLayers(layElem, newItem);
+        parseLayer(layElem, newItem);
         layElem = layElem.nextSiblingElement("Layer");
     }
 
@@ -644,7 +645,7 @@ void WMSPreferencesDialog::on_tvWmsLayers_itemChanged(QTreeWidgetItem *twi, int)
     refreshStyles();
 }
 
-void WMSPreferencesDialog::on_tvWmsLayers_currentItemChanged(QTreeWidgetItem *current, QTreeWidgetItem *previous)
+void WMSPreferencesDialog::on_tvWmsLayers_currentItemChanged(QTreeWidgetItem *current, QTreeWidgetItem */*previous*/)
 {
     if (current)
         curLayer = current->text(0);
