@@ -51,8 +51,8 @@ class BrowserWebPage : public QWebPage
         void launchRequest ( const QUrl & url );
 
     private:
-        double tllat, tllon;
-        double brlat, brlon;
+        qreal tllat, tllon;
+        qreal brlat, brlon;
 
         int ox, oy, sw, sh;
 };
@@ -77,15 +77,13 @@ class BrowserImageManager : public QObject, public IImageManager
          * @param path the path to the image
          * @return the pixmap of the asked image
          */
-        //QPixmap getImage(const QString& host, const QString& path);
-        QPixmap getPixmap(IMapAdapter* anAdapter, int x, int y, int z);
-        QPixmap getPixmap(IMapAdapter* anAdapter, QString url);
+        QImage getImage(IMapAdapter* anAdapter, QString url);
         QByteArray getData(IMapAdapter* anAdapter, QString url);
 
         //QPixmap prefetchImage(const QString& host, const QString& path);
-        QPixmap prefetchPixmap(IMapAdapter* anAdapter, int x, int y, int z);
+        QImage prefetchImage(IMapAdapter* anAdapter, int x, int y, int z);
 
-        void receivedData(const QByteArray& ba, const QString& url);
+        void receivedData(const QByteArray& ba, const QHash<QString, QString>& headers, const QString& url);
 
         /*!
          * This method is called by MapNetwork, after all images in its queue were loaded.
