@@ -26,57 +26,35 @@
 #define EQUATORIALMETERHALFCIRCUMFERENCE  20037508.34
 #define EQUATORIALMETERPERDEGREE    222638.981555556
 
-TileMapAdapter::TileMapAdapter(const QString& host, const QString& serverPath, const QString& projection, int theTilesize, int minZoom, int maxZoom, bool blOrigin)
-    :MapAdapter(host, serverPath, projection, minZoom, maxZoom)
-    , tilesize(theTilesize)
-    , BlOrigin(blOrigin)
+TileMapAdapter::TileMapAdapter(const TmsServer& ts)
+    :MapAdapter(ts.TmsAdress, ts.TmsPath, ts.TmsProjection, ts.TmsMinZoom, ts.TmsMaxZoom)
+    , theServer(ts)
+    , tilesize(ts.TmsTileSize)
+    , BlOrigin(ts.TmsBlOrigin)
 {
-    name = "tiles";
+    name = ts.TmsName;
 
-// 	qDebug() << "creating adapter: min, max, current: " << minZoom << ", " << maxZoom << ", " << current_zoom << ", " << (min_zoom < max_zoom);
+    serverPath.replace("%1", "%z");
+    serverPath.replace("%2", "%x");
+    serverPath.replace("%3", "%y");
 
-    /*
-        Initialize the "substring replace engine". First the string replacement
-        in getQuery was made by QString().arg() but this was very slow. So this
-        splits the servers path into substrings and when calling getQuery the
-        substrings get merged with the parameters of the URL.
-        Pretty complicated, but fast.
-    */
-    param1 = serverPath.indexOf("%1");
-    param2 = serverPath.indexOf("%2");
-    param3 = serverPath.indexOf("%3");
+    int paramz = serverPath.indexOf("%z");
+    int paramx = serverPath.indexOf("%x");
+    int paramy = serverPath.indexOf("%y");
 
-    int min = param1 < param2 ? param1 : param2;
-    min = param3 < min ? param3 : min;
+    if (paramx == -1 && paramy == -1 && paramz == -1) {
+        // Check for potlach-style url
+        paramz = serverPath.indexOf('!');
+        if (paramz)
+            serverPath.replace(paramz, 1, "%z");
+        paramx = serverPath.indexOf('!');
+        if (paramx)
+            serverPath.replace(paramx, 1, "%x");
+        paramy = serverPath.indexOf('!');
+        if (paramy)
+            serverPath.replace(paramy, 1, "%y");
 
-    int max = param1 > param2 ? param1 : param2;
-    max = param3 > max ? param3 : max;
-
-    int middle = param1+param2+param3-min-max;
-
-    order[0][0] = min;
-    if (min == param1)
-        order[0][1] = 0;
-    else if (min == param2)
-        order[0][1] = 1;
-    else
-        order[0][1] = 2;
-
-    order[1][0] = middle;
-    if (middle == param1)
-        order[1][1] = 0;
-    else if (middle == param2)
-        order[1][1] = 1;
-    else
-        order[1][1] = 2;
-
-    order[2][0] = max;
-    if (max == param1)
-        order[2][1] = 0;
-    else if(max == param2)
-        order[2][1] = 1;
-    else
-        order[2][1] = 2;
+    }
 
     isProj4326 = (Projection.contains(":4326"));
     loc.setNumberOptions(QLocale::OmitGroupSeparator);
@@ -104,7 +82,12 @@ QRectF TileMapAdapter::getBoundingbox() const
         return QRectF(QPointF(-EQUATORIALMETERHALFCIRCUMFERENCE, -EQUATORIALMETERHALFCIRCUMFERENCE), QPointF(EQUATORIALMETERHALFCIRCUMFERENCE, EQUATORIALMETERHALFCIRCUMFERENCE));
 }
 
-int TileMapAdapter::getTileSize() const
+int TileMapAdapter::getTileSizeW() const
+{
+    return tilesize;
+}
+
+int TileMapAdapter::getTileSizeH() const
 {
     return tilesize;
 }
@@ -129,11 +112,11 @@ QString TileMapAdapter::getQuery		(int x, int y, int z) const
 {
     if (BlOrigin)
         y = getTilesNS(current_zoom)-1 - y;
-    int a[3] = {z, x, y};
-    return QString(serverPath).replace(order[2][0],2, loc.toString(a[order[2][1]]))
-                                      .replace(order[1][0],2, loc.toString(a[order[1][1]]))
-                                      .replace(order[0][0],2, loc.toString(a[order[0][1]]));
-
+    QString str = serverPath;
+    str.replace("%z", QString::number(z));
+    str.replace("%y", QString::number(y));
+    str.replace("%x", QString::number(x));
+    return str;
 }
 
 bool TileMapAdapter::isValid(int x, int y, int z) const
@@ -159,3 +142,13 @@ int TileMapAdapter::getTilesNS(int zoomlevel) const
 {
     return int(pow(2, zoomlevel+0.0));
 }
+
+QString	TileMapAdapter::getSourceTag() const
+{
+    return theServer.TmsSourceTag;
+}
+
+QString TileMapAdapter::getLicenseUrl() const
+{
+    return theServer.TmsLicenseUrl;
+}
