@@ -14,110 +14,108 @@
 #include "Document.h"
 #include "Features.h"
 #include "MapView.h"
-#include "PaintStyle/MasPaintStyle.h"
+#include "MasPaintStyle.h"
 #include "ImageMapLayer.h"
-#include "Utils/LineF.h"
+#include "LineF.h"
 
 #define TEST_RFLAGS(x) theOptions.options.testFlag(x)
 #define TEST_RENDERER_RFLAGS(x) r->theOptions.options.testFlag(x)
 
 void BackgroundStyleLayer::draw(Way* R)
 {
-    const FeaturePainter* paintsel = R->getPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = R->getPainter(r->thePixelPerM);
     if (paintsel) {
-        paintsel->drawBackground(R,r->thePainter,r->theView);
-    } else if (!R->hasPainter() && !TEST_RENDERER_RFLAGS(RendererOptions::UnstyledHidden))
+        paintsel->drawBackground(R, r->thePainter, r);
+    } else if (!TEST_RENDERER_RFLAGS(RendererOptions::UnstyledHidden))
 //    if (/*!globalZoom(r->theProjection) && */!R->hasPainter()) //FIXME Untagged roads level of zoom?
     {
         QPen thePen(QColor(0,0,0),1);
 
         r->thePainter->setBrush(Qt::NoBrush);
-        if (dynamic_cast<ImageMapLayer*>(R->layer()) && M_PREFS->getUseShapefileForBackground()) {
+        if (R->layer()->classType() == Layer::ImageLayerType && M_PREFS->getUseShapefileForBackground()) {
             thePen = QPen(QColor(0xc0,0xc0,0xc0),1);
-            if (!R->isCoastline()) {
-                if (M_PREFS->getBackgroundOverwriteStyle() || !M_STYLE->getGlobalPainter().getDrawBackground())
-                    r->thePainter->setBrush(M_PREFS->getBgColor());
-                else
-                    r->thePainter->setBrush(QBrush(M_STYLE->getGlobalPainter().getBackgroundColor()));
-            }
+            if (M_PREFS->getBackgroundOverwriteStyle() || !M_STYLE->getGlobalPainter().getDrawBackground())
+                r->thePainter->setBrush(M_PREFS->getBgColor());
+            else
+                r->thePainter->setBrush(QBrush(M_STYLE->getGlobalPainter().getBackgroundColor()));
         } else {
-            if (r->theView->pixelPerM() < M_PREFS->getRegionalZoom())
+            if (r->thePixelPerM < M_PREFS->getRegionalZoom())
                 thePen = QPen(QColor(0x77,0x77,0x77),1);
         }
 
         r->thePainter->setPen(thePen);
-        r->thePainter->drawPath(r->theView->transform().map(R->getPath()));
+        r->thePainter->drawPath(r->theTransform.map(R->getPath()));
     }
 }
 
 void BackgroundStyleLayer::draw(Relation* R)
 {
-    const FeaturePainter* paintsel = R->getPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = R->getPainter(r->thePixelPerM);
     if (paintsel)
-        paintsel->drawBackground(R,r->thePainter,r->theView);
+        paintsel->drawBackground(R,r->thePainter,r);
 }
 
 
 void BackgroundStyleLayer::draw(Node* N)
 {
-    if ((N->isReadonly() || !N->isSelectable(r->theView)) && (!N->isPOI() && !N->isWaypoint()))
+    if ((N->isReadonly() || !N->isSelectable(r->thePixelPerM, r->theOptions)) && (!N->isPOI() && !N->isWaypoint()))
         return;
 
-    const FeaturePainter* paintsel = N->getPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = N->getPainter(r->thePixelPerM);
     if (paintsel)
-        paintsel->drawBackground(N,r->thePainter,r->theView);
+        paintsel->drawBackground(N,r->thePainter,r);
 }
 
 void ForegroundStyleLayer::draw(Way* R)
 {
-    const FeaturePainter* paintsel = R->getPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = R->getPainter(r->thePixelPerM);
     if (paintsel)
-        paintsel->drawForeground(R,r->thePainter,r->theView);
+        paintsel->drawForeground(R,r->thePainter,r);
 }
 
 void ForegroundStyleLayer::draw(Relation* R)
 {
-    const FeaturePainter* paintsel = R->getPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = R->getPainter(r->thePixelPerM);
     if (paintsel)
-        paintsel->drawForeground(R,r->thePainter,r->theView);
+        paintsel->drawForeground(R,r->thePainter,r);
 }
 
 void ForegroundStyleLayer::draw(Node* N)
 {
-    if ((N->isReadonly() || !N->isSelectable(r->theView)) && (!N->isPOI() && !N->isWaypoint()))
+    if ((N->isReadonly() || !N->isSelectable(r->thePixelPerM, r->theOptions)) && (!N->isPOI() && !N->isWaypoint()))
         return;
 
-    const FeaturePainter* paintsel = N->getPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = N->getPainter(r->thePixelPerM);
     if (paintsel)
-        paintsel->drawForeground(N,r->thePainter,r->theView);
+        paintsel->drawForeground(N,r->thePainter,r);
 }
 
 void TouchupStyleLayer::draw(Way* R)
 {
-    const FeaturePainter* paintsel = R->getPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = R->getPainter(r->thePixelPerM);
     if (paintsel)
-        paintsel->drawTouchup(R,r->thePainter,r->theView);
+        paintsel->drawTouchup(R,r->thePainter,r);
     else if (!R->hasPainter() && !TEST_RENDERER_RFLAGS(RendererOptions::UnstyledHidden)) {
         if ( r->theOptions.arrowOptions != RendererOptions::ArrowsNever )
         {
             Feature::TrafficDirectionType TT = trafficDirection(R);
             if ( (TT != Feature::UnknownDirection) || (r->theOptions.arrowOptions == RendererOptions::ArrowsAlways) )
             {
-                double theWidth = r->theView->pixelPerM()*R->widthOf()-4;
+                qreal theWidth = r->thePixelPerM*R->widthOf()-4;
                 if (theWidth > 8)
                     theWidth = 8;
-                double DistFromCenter = 2*(theWidth+4);
+                qreal DistFromCenter = 2*(theWidth+4);
                 if (theWidth > 0)
                 {
                     for (int i=1; i<R->size(); ++i)
                     {
-                        QPointF FromF(r->theView->transform().map(r->theView->projection().project(R->getNode(i-1))));
-                        QPointF ToF(r->theView->transform().map(r->theView->projection().project(R->getNode(i))));
+                        QPointF FromF(r->toView(R->getNode(i-1)));
+                        QPointF ToF(r->toView(R->getNode(i)));
                         if (distance(FromF,ToF) > (DistFromCenter*2+4))
                         {
                             QPointF H(FromF+ToF);
                             H *= 0.5;
-                            double A = angle(FromF-ToF);
+                            qreal A = angle(FromF-ToF);
                             QPointF T(DistFromCenter*cos(A),DistFromCenter*sin(A));
                             QPointF V1(theWidth*cos(A+M_PI/6),theWidth*sin(A+M_PI/6));
                             QPointF V2(theWidth*cos(A-M_PI/6),theWidth*sin(A-M_PI/6));
@@ -156,23 +154,30 @@ void TouchupStyleLayer::draw(Relation* /* R */)
 
 void TouchupStyleLayer::draw(Node* Pt)
 {
-    const FeaturePainter* paintsel = Pt->getPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = Pt->getPainter(r->thePixelPerM);
     if (paintsel)
-        paintsel->drawTouchup(Pt,r->thePainter,r->theView);
-    else if (!Pt->hasPainter() && !TEST_RENDERER_RFLAGS(RendererOptions::UnstyledHidden)) {
-        if (! ((Pt->isReadonly() || !Pt->isSelectable(r->theView)) && (!Pt->isPOI() && !Pt->isWaypoint())))
-//        if (!Pt->isReadonly() && Pt->isSelectable(r->theView))
+        paintsel->drawTouchup(Pt,r->thePainter,r);
+    else if (!TEST_RENDERER_RFLAGS(RendererOptions::UnstyledHidden)) {
+        if (! ((Pt->isReadonly() || !Pt->isSelectable(r->thePixelPerM, r->theOptions)) && (!Pt->isPOI() && !Pt->isWaypoint())))
+//        if (!Pt->isReadonly() && Pt->isSelectable(r))
         {
-            QPointF P(r->theView->toView(Pt));
-            double WW = r->theView->nodeWidth();
+            qreal WW = r->NodeWidth;
             if (WW >= 1) {
-                if (Pt->isWaypoint()) {
+                QColor theColor = QColor(0,0,0,128);
+                if (r->theGlobalPainter.DrawNodes) {
+                    theColor = r->theGlobalPainter.NodesColor;
+                }
+                QPointF P(r->toView(Pt));
+                if (Pt->layer()->classGroups() & Layer::Special) {
                     QRect R2(P.x()-(WW+4)/2, P.y()-(WW+4)/2, WW+4, WW+4);
-                    r->thePainter->fillRect(R2,QColor(255,0,0,128));
+                    r->thePainter->fillRect(R2,QColor(255,0,255,192));
+                } else if (Pt->isWaypoint()) {
+                    QRect R2(P.x()-(WW+4)/2, P.y()-(WW+4)/2, WW+4, WW+4);
+                    r->thePainter->fillRect(R2,QColor(255,0,0,192));
                 }
 
                 QRect R(P.x()-WW/2, P.y()-WW/2, WW, WW);
-                r->thePainter->fillRect(R,QColor(0,0,0,128));
+                r->thePainter->fillRect(R,theColor);
             }
         }
     }
@@ -180,9 +185,9 @@ void TouchupStyleLayer::draw(Node* Pt)
 
 void LabelStyleLayer::draw(Way* R)
 {
-    const FeaturePainter* paintsel = R->getPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = R->getPainter(r->thePixelPerM);
     if (paintsel)
-        paintsel->drawLabel(R,r->thePainter,r->theView);
+        paintsel->drawLabel(R,r->thePainter,r);
 }
 
 void LabelStyleLayer::draw(Relation* /* R */)
@@ -191,9 +196,9 @@ void LabelStyleLayer::draw(Relation* /* R */)
 
 void LabelStyleLayer::draw(Node* Pt)
 {
-    const FeaturePainter* paintsel = Pt->getPainter(r->theView->pixelPerM());
+    const FeaturePainter* paintsel = Pt->getPainter(r->thePixelPerM);
     if (paintsel)
-        paintsel->drawLabel(Pt,r->thePainter,r->theView);
+        paintsel->drawLabel(Pt,r->thePainter,r);
 }
 
 /*** MapRenderer ***/
@@ -241,13 +246,14 @@ void MapRenderer::render(
     {
         pix.fill(Qt::transparent);
         thePainter->begin(&pix);
-        thePainter->setRenderHint(QPainter::Antialiasing);
+        if (M_PREFS->getUseAntiAlias())
+            thePainter->setRenderHint(QPainter::Antialiasing);
         int curLayer = (itm.key()).layer();
         while (itm != theFeatures.constEnd() && (itm.key()).layer() == curLayer)
         {
             for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it)
             {
-                double alpha = (*it)->getAlpha();
+                qreal alpha = (*it)->getAlpha();
                 thePainter->setOpacity(alpha);
 
                 R = NULL;
@@ -262,7 +268,7 @@ void MapRenderer::render(
                     // If there is painter at the relation level, don't paint at the way level
                     bool draw = true;
                     for (int i=0; i<R->sizeParents(); ++i) {
-                        if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(theView->pixelPerM()))
+                        if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(PixelPerM))
                             draw = false;
                     }
                     if (!draw)
@@ -338,19 +344,58 @@ void MapRenderer::render(
     }
 }
 #else
+
+QPoint MapRenderer::toView(Node* aPt) const
+{
+    return theTransform.map(aPt->projected()).toPoint();
+}
+
+
 void MapRenderer::render(
         QPainter* P,
         const QMap<RenderPriority, QSet <Feature*> >& theFeatures,
-        const RendererOptions& options,
-        MapView* aView
+        const QRectF& pViewport,
+        const QRect& screen,
+        const qreal pixelPerM,
+        const RendererOptions& options
 )
 {
 //    #ifndef NDEBUG
 //        QTime Start(QTime::currentTime());
 //    #endif
 
-    theView = aView;
+    theViewport = pViewport;
+    theScreen = screen;
+    thePixelPerM = pixelPerM;
+
+    qreal Aspect = (double)screen.width() / screen.height();
+    qreal pAspect = fabs(pViewport.width() / pViewport.height());
+
+    qreal wv, hv;
+    if (pAspect > Aspect) {
+        wv = fabs(pViewport.width());
+        hv = fabs(pViewport.height() * pAspect / Aspect);
+    } else {
+        wv = fabs(pViewport.width() * Aspect / pAspect);
+        hv = fabs(pViewport.height());
+    }
+
+    qreal ScaleLon = screen.width() / wv;
+    qreal ScaleLat = screen.height() / hv;
+    theTransform.reset();
+    theTransform.scale(ScaleLon, -ScaleLat);
+    theTransform.translate(-pViewport.topLeft().x(), -pViewport.topLeft().y());
+//    qDebug() << "render transform: " << theTransform;
+
     theOptions = options;
+    theGlobalPainter = M_STYLE->getGlobalPainter();
+    if (theGlobalPainter.DrawNodes) {
+        NodeWidth = thePixelPerM*theGlobalPainter.NodesProportional+theGlobalPainter.NodesFixed;
+    } else {
+        NodeWidth = thePixelPerM * M_PREFS->getNodeSize();
+        if (NodeWidth > M_PREFS->getNodeSize())
+            NodeWidth = M_PREFS->getNodeSize();
+    }
 
     bool bgLayerVisible = TEST_RFLAGS(RendererOptions::BackgroundVisible);
     bool fgLayerVisible = TEST_RFLAGS(RendererOptions::ForegroundVisible);
@@ -361,8 +406,9 @@ void MapRenderer::render(
     QMap<RenderPriority, QSet<Feature*> >::const_iterator itmCur;
     QSet<Feature*>::const_iterator it;
 
-    P->setRenderHint(QPainter::Antialiasing);
     thePainter = P;
+    thePainter->save();
+    thePainter->translate(screen.left(), screen.top());
 
     itm = theFeatures.constBegin();
     while (itm != theFeatures.constEnd())
@@ -374,7 +420,7 @@ void MapRenderer::render(
             if (bgLayerVisible)
             {
                 for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it) {
-                    double alpha = (*it)->getAlpha();
+                    qreal alpha = (*it)->getAlpha();
                     if ((*it)->isReadonly() && !TEST_RFLAGS(RendererOptions::ForPrinting))
                         alpha /= 2.0;
                     if (alpha != 1.) {
@@ -385,7 +431,7 @@ void MapRenderer::render(
                     if (CHECK_WAY(*it)) {
                         Way * R = STATIC_CAST_WAY(*it);
                         for (int i=0; i<R->sizeParents(); ++i)
-                            if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(theView->pixelPerM()))
+                            if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(thePixelPerM))
                                 continue;
                         bglayer.draw(R);
                     } else if (CHECK_NODE(*it))
@@ -405,7 +451,7 @@ void MapRenderer::render(
             if (fgLayerVisible)
             {
                 for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it) {
-                    double alpha = (*it)->getAlpha();
+                    qreal alpha = (*it)->getAlpha();
                     if ((*it)->isReadonly() && !TEST_RFLAGS(RendererOptions::ForPrinting))
                         alpha /= 2.0;
                     if (alpha != 1.) {
@@ -416,7 +462,7 @@ void MapRenderer::render(
                     if (CHECK_WAY(*it)) {
                         Way * R = STATIC_CAST_WAY(*it);
                         for (int i=0; i<R->sizeParents(); ++i)
-                            if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(theView->pixelPerM()))
+                            if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(thePixelPerM))
                                 continue;
                         fglayer.draw(R);
                     } else if (CHECK_NODE(*it))
@@ -435,7 +481,7 @@ void MapRenderer::render(
     {
         for (itm = theFeatures.constBegin() ;itm != theFeatures.constEnd(); ++itm) {
             for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it) {
-                double alpha = (*it)->getAlpha();
+                qreal alpha = (*it)->getAlpha();
                 if ((*it)->isReadonly() && !TEST_RFLAGS(RendererOptions::ForPrinting))
                     alpha /= 2.0;
                 if (alpha != 1.) {
@@ -446,7 +492,7 @@ void MapRenderer::render(
                 if (CHECK_WAY(*it)) {
                     Way * R = STATIC_CAST_WAY(*it);
                     for (int i=0; i<R->sizeParents(); ++i)
-                        if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(theView->pixelPerM()))
+                        if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(thePixelPerM))
                             continue;
                     tchuplayer.draw(R);
                 } else if (CHECK_NODE(*it))
@@ -459,12 +505,13 @@ void MapRenderer::render(
             }
         }
     }
+
     if (lblLayerVisible)
     {
         for (itm = theFeatures.constBegin() ;itm != theFeatures.constEnd(); ++itm) {
             for (it = itm.value().constBegin(); it != itm.value().constEnd(); ++it) {
                 P->save();
-                double alpha = (*it)->getAlpha();
+                qreal alpha = (*it)->getAlpha();
                 if ((*it)->isReadonly() && !TEST_RFLAGS(RendererOptions::ForPrinting))
                     alpha /= 2.0;
                 P->setOpacity(alpha);
@@ -472,7 +519,7 @@ void MapRenderer::render(
                 if (CHECK_WAY(*it)) {
                     Way * R = STATIC_CAST_WAY(*it);
                     for (int i=0; i<R->sizeParents(); ++i)
-                        if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(theView->pixelPerM()))
+                        if (!R->getParent(i)->isDeleted() && R->getParent(i)->hasPainter(thePixelPerM))
                             continue;
                     lbllayer.draw(R);
                 } else if (CHECK_NODE(*it))
@@ -483,20 +530,8 @@ void MapRenderer::render(
             }
         }
     }
+    thePainter->restore();
 
-    for (itm = theFeatures.constBegin() ;itm != theFeatures.constEnd(); ++itm)
-    {
-        for (it = itm.value().constBegin() ;it != itm.value().constEnd(); ++it)
-        {
-            double alpha = (*it)->getAlpha();
-            if ((*it)->isReadonly() && !TEST_RFLAGS(RendererOptions::ForPrinting))
-                alpha /= 2.0;
-            if (alpha != 1.)
-                P->setOpacity(alpha);
-
-            (*it)->draw(*P, aView);
-        }
-    }
 //    #ifndef NDEBUG
 //        QTime Stop(QTime::currentTime());
 //        qDebug() << Start.msecsTo(Stop) << "ms";
