@@ -14,17 +14,18 @@
 #include "MainWindow.h"
 #include "Document.h"
 #include "MapView.h"
-#include "Maps/Projection.h"
+#include "Projection.h"
 #include "Layer.h"
 #include "Features.h"
-#include "Preferences/MerkaartorPreferences.h"
-#include "PaintStyle/MasPaintStyle.h"
-#include "Utils/PictureViewerDialog.h"
+#include "MerkaartorPreferences.h"
+#include "MasPaintStyle.h"
+#include "PictureViewerDialog.h"
 
 #include <QPrinter>
 #include <QPrintPreviewDialog>
 #include <QPrintPreviewWidget>
 
+#include <QMainWindow>
 #include <QProgressDialog>
 #include <QPainter>
 #include <QSvgGenerator>
@@ -104,21 +105,21 @@ void NativeRenderDialog::setOptions(RendererOptions aOpt)
 CoordBox NativeRenderDialog::boundingBox()
 {
     CoordBox VP(Coord(
-                    angToCoord(ui.sbMinLat->value()),
-                    angToCoord(ui.sbMinLon->value())
+                    ui.sbMinLon->value(),
+                    ui.sbMinLat->value()
             ), Coord(
-                    angToCoord(ui.sbMaxLat->value()),
-                    angToCoord(ui.sbMaxLon->value())
+                    ui.sbMaxLon->value(),
+                    ui.sbMaxLat->value()
                     ));
     return VP;
 }
 
 void NativeRenderDialog::setBoundingBox(CoordBox aBBox)
 {
-    ui.sbMinLat->setValue(coordToAng(aBBox.bottomLeft().lat()));
-    ui.sbMaxLat->setValue(coordToAng(aBBox.topLeft().lat()));
-    ui.sbMinLon->setValue(coordToAng(aBBox.topLeft().lon()));
-    ui.sbMaxLon->setValue(coordToAng(aBBox.topRight().lon()));
+    ui.sbMinLat->setValue(aBBox.bottomLeft().y());
+    ui.sbMaxLat->setValue(aBBox.topLeft().y());
+    ui.sbMinLon->setValue(aBBox.topLeft().x());
+    ui.sbMaxLon->setValue(aBBox.topRight().x());
 
     prtW->updatePreview();
 }
@@ -141,14 +142,13 @@ void NativeRenderDialog::render(QPainter& P, QRect theR, RendererOptions opt)
 {
     P.setClipRect(theR);
     P.setClipping(true);
+    P.setRenderHint(QPainter::Antialiasing);
 
     mapview->setGeometry(theR);
     mapview->setViewport(boundingBox(), theR);
     mapview->setRenderOptions(opt);
-    mapview->invalidate(true, false);
-    mapview->buildFeatureSet();
-    mapview->drawCoastlines(P);
-    mapview->printFeatures(P);
+    mapview->invalidate(true, true, false);
+    mapview->drawFeatures(P);
     if (opt.options & RendererOptions::ScaleVisible)
         mapview->drawScale(P);
     if (opt.options & RendererOptions::LatLonGridVisible)
