@@ -2,17 +2,17 @@
 
 #include "MainWindow.h"
 #include "MapView.h"
-#include "Maps/Coord.h"
+#include "Coord.h"
 #include "ImportGPX.h"
+#include "ImportExportGdal.h"
 #include "ImportOSM.h"
 #include "Document.h"
 #include "Layer.h"
 #include "Feature.h"
 #include "TrackSegment.h"
-#include "Utils/SlippyMapWidget.h"
-#include "Preferences/MerkaartorPreferences.h"
-#include "ImportExport/ImportExportOsmBin.h"
-#include "Utils/OsmLink.h"
+#include "SlippyMapWidget.h"
+#include "MerkaartorPreferences.h"
+#include "OsmLink.h"
 
 #include "IProgressWindow.h"
 
@@ -252,8 +252,9 @@ bool Downloader::request(const QString& Method, const QUrl& url, const QString&
 
     QString sReq = url.toString(QUrl::RemoveScheme | QUrl::RemoveAuthority);
     QHttpRequestHeader Header(Method,sReq);
-    Header.setValue("Host",url.host()+':'+url.port(80));
+    Header.setValue("Host",url.host()+':'+QString::number(url.port(80)));
     Header.setValue("User-Agent", USER_AGENT);
+    Header.setValue("Content-Type", "text/xml");
 
     QString auth = QString("%1:%2").arg(User).arg(Password);
     QByteArray ba_auth = auth.toUtf8().toBase64();
@@ -464,7 +465,7 @@ bool downloadOSM(QWidget* aParent, const QUrl& theUrl, const QString& aUser, con
     }
     if (!Rcv.go(theUrl))
     {
-#ifndef Q_OS_SYMBIAN
+#ifndef _MOBILE
         aParent->setCursor(QCursor(Qt::ArrowCursor));
 #endif
         return false;
@@ -517,31 +518,12 @@ bool downloadOSM(QWidget* aParent, const QString& aWeb, const QString& aUser, co
     }
     Downloader Rcv(aUser, aPassword);
     QString URL = Rcv.getURLToMap();
-    URL = URL.arg(coordToAng(aBox.bottomLeft().lon()), 0, 'f').arg(coordToAng(aBox.bottomLeft().lat()), 0, 'f').arg(coordToAng(aBox.topRight().lon()), 0, 'f').arg(coordToAng(aBox.topRight().lat()), 0, 'f');
+    URL = URL.arg(aBox.bottomLeft().x(), 0, 'f').arg(aBox.bottomLeft().y(), 0, 'f').arg(aBox.topRight().x(), 0, 'f').arg(aBox.topRight().y(), 0, 'f');
 
     QUrl theUrl(aWeb+URL);
     return downloadOSM(aParent, theUrl, aUser, aPassword, theDocument, theLayer);
 }
 
-bool downloadOSM(QWidget* Main, const QString& aUser, const QString& aPassword, const quint32 region , Document* theDocument, Layer* theLayer)
-{
-    Q_UNUSED(aUser)
-    Q_UNUSED(aPassword)
-    int y = int(region / NUM_REGIONS); //2565024
-    int x = (region % NUM_REGIONS);
-    CoordBox Clip = CoordBox(
-        Coord (y * REGION_WIDTH - COORD_MAX, x * REGION_WIDTH - COORD_MAX  ),
-        Coord ((y+1) * REGION_WIDTH - COORD_MAX, (x+1) * REGION_WIDTH - COORD_MAX )
-        );
-
-    QString osmWebsite, osmUser, osmPwd;
-    osmWebsite = M_PREFS->getOsmWebsite();
-    osmUser = M_PREFS->getOsmUser();
-    osmPwd = M_PREFS->getOsmPassword();
-
-    return downloadOSM(Main,osmWebsite,osmUser,osmPwd,Clip,theDocument,theLayer);
-}
-
 bool downloadTracksFromOSM(QWidget* Main, const QString& aWeb, const QString& aUser, const QString& aPassword, const CoordBox& aBox , Document* theDocument)
 {
     Downloader theDownloader(aUser, aPassword);
@@ -571,10 +553,10 @@ bool downloadTracksFromOSM(QWidget* Main, const QString& aWeb, const QString& aU
     {
         Lbl->setText(QApplication::translate("Downloader","Downloading trackpoints %1-%2").arg(Page*5000+1).arg(Page*5000+5000));
         QString URL = theDownloader.getURLToTrackPoints();
-        URL = URL.arg(coordToAng(aBox.bottomLeft().lon())).
-                arg(coordToAng(aBox.bottomLeft().lat())).
-                arg(coordToAng(aBox.topRight().lon())).
-                arg(coordToAng(aBox.topRight().lat())).
+        URL = URL.arg(aBox.bottomLeft().x()).
+                arg(aBox.bottomLeft().y()).
+                arg(aBox.topRight().x()).
+                arg(aBox.topRight().y()).
                 arg(Page);
         QUrl theUrl(aWeb+URL);
         if (!theDownloader.go(theUrl))
@@ -633,11 +615,10 @@ bool downloadFeatures(MainWindow* Main, const QList<IFeature::FId>& idList , Doc
         } else
             theLayer = (Layer*)theDocument->getLastDownloadLayer();
     }
-    theLayer->blockIndexing(true);
 
     QString osmWebsite, osmUser, osmPwd;
 
-    osmWebsite = M_PREFS->getOsmWebsite();
+    osmWebsite = M_PREFS->getOsmApiUrl();
     osmUser = M_PREFS->getOsmUser();
     osmPwd = M_PREFS->getOsmPassword();
 
@@ -656,10 +637,8 @@ bool downloadFeatures(MainWindow* Main, const QList<IFeature::FId>& idList , Doc
 
     if (Main)
         Main->view()->setUpdatesEnabled(true);
-    theLayer->blockIndexing(false);
     if (OK)
     {
-        theLayer->reIndex();
         if (Main)
             Main->invalidateView();
     } else
@@ -672,24 +651,23 @@ bool downloadFeatures(MainWindow* Main, const QList<IFeature::FId>& idList , Doc
     return OK;
 }
 
-bool downloadOpenstreetbugs(MainWindow* Main, const CoordBox& aBox, Document* theDocument)
+bool downloadMapdust(MainWindow* Main, const CoordBox& aBox, Document* theDocument, SpecialLayer* theLayer)
 {
-    QUrl osbUrl;
+    QUrl url;
 
-    osbUrl.setUrl(M_PREFS->getOpenStreetBugsUrl());
-    osbUrl.setPath(osbUrl.path() + "getGPX");
+    url.setUrl(M_PREFS->getMapdustUrl());
 
     if (Main)
         Main->view()->setUpdatesEnabled(false);
 
     Downloader theDownloader("", "");
-    QList<TrackLayer*> theTracklayers;
-    TrackLayer* trackLayer = new TrackLayer(QApplication::translate("Downloader","OpenStreetBugs"));
-    trackLayer->setUploadable(false);
-    theDocument->add(trackLayer);
-    theTracklayers << trackLayer;
 
-    trackLayer->blockIndexing(true);
+    SpecialLayer* trackLayer = theLayer;
+    if (!trackLayer) {
+        trackLayer = new SpecialLayer(QApplication::translate("Downloader","MapDust"), Layer::MapDustLayer);
+        trackLayer->setUploadable(false);
+        theDocument->add(trackLayer);
+    }
 
     IProgressWindow* aProgressWindow = dynamic_cast<IProgressWindow*>(Main);
     if (!aProgressWindow)
@@ -710,10 +688,72 @@ bool downloadOpenstreetbugs(MainWindow* Main, const CoordBox& aBox, Document* th
 
     theDownloader.setAnimator(dlg,Lbl,Bar,true);
     Lbl->setText(QApplication::translate("Downloader","Downloading points"));
-    osbUrl.addQueryItem("t", COORD2STRING(coordToAng(aBox.topRight().lat())));
-    osbUrl.addQueryItem("l", COORD2STRING(coordToAng(aBox.bottomLeft().lon())));
-    osbUrl.addQueryItem("b", COORD2STRING(coordToAng(aBox.bottomLeft().lat())));
-    osbUrl.addQueryItem("r", COORD2STRING(coordToAng(aBox.topRight().lon())));
+    url.addQueryItem("t", COORD2STRING(aBox.topRight().y()));
+    url.addQueryItem("l", COORD2STRING(aBox.bottomLeft().x()));
+    url.addQueryItem("b", COORD2STRING(aBox.bottomLeft().y()));
+    url.addQueryItem("r", COORD2STRING(aBox.topRight().x()));
+
+    if (!theDownloader.go(url))
+        return false;
+    if (theDownloader.resultCode() != 200)
+        return false;
+    QByteArray Ar(theDownloader.content());
+    ImportExportGdal gdal(theDocument);
+    bool OK = gdal.import(trackLayer, Ar, false);
+
+    if (Main)
+        Main->view()->setUpdatesEnabled(true);
+    if (OK) {
+        if (Main)
+            Main->invalidateView();
+    }
+    return OK;
+}
+
+bool downloadOpenstreetbugs(MainWindow* Main, const CoordBox& aBox, Document* theDocument, SpecialLayer* theLayer)
+{
+    QUrl osbUrl;
+
+    osbUrl.setUrl(M_PREFS->getOpenStreetBugsUrl());
+    osbUrl.setPath(osbUrl.path() + "getGPX");
+
+    if (Main)
+        Main->view()->setUpdatesEnabled(false);
+
+    Downloader theDownloader("", "");
+
+    QList<TrackLayer*> theTracklayers;
+    SpecialLayer* trackLayer = theLayer;
+    if (!trackLayer) {
+        SpecialLayer* trackLayer = new SpecialLayer(QApplication::translate("Downloader","OpenStreetBugs"),Layer::OsmBugsLayer);
+        trackLayer->setUploadable(false);
+        theDocument->add(trackLayer);
+    }
+    theTracklayers << trackLayer;
+
+    IProgressWindow* aProgressWindow = dynamic_cast<IProgressWindow*>(Main);
+    if (!aProgressWindow)
+        return false;
+
+    QProgressDialog* dlg = aProgressWindow->getProgressDialog();
+    dlg->setWindowTitle(QApplication::translate("Downloader","Parsing..."));
+
+    QProgressBar* Bar = aProgressWindow->getProgressBar();
+    Bar->setTextVisible(false);
+    Bar->setMaximum(11);
+
+    QLabel* Lbl = aProgressWindow->getProgressLabel();
+    Lbl->setText(QApplication::translate("Downloader","Parsing XML"));
+
+    if (dlg)
+        dlg->show();
+
+    theDownloader.setAnimator(dlg,Lbl,Bar,true);
+    Lbl->setText(QApplication::translate("Downloader","Downloading points"));
+    osbUrl.addQueryItem("t", COORD2STRING(aBox.topRight().y()));
+    osbUrl.addQueryItem("l", COORD2STRING(aBox.bottomLeft().x()));
+    osbUrl.addQueryItem("b", COORD2STRING(aBox.bottomLeft().y()));
+    osbUrl.addQueryItem("r", COORD2STRING(aBox.topRight().x()));
     osbUrl.addQueryItem("open", "yes");
 
     if (!theDownloader.go(osbUrl))
@@ -725,11 +765,10 @@ bool downloadOpenstreetbugs(MainWindow* Main, const CoordBox& aBox, Document* th
 
     if (Main)
         Main->view()->setUpdatesEnabled(true);
-    trackLayer->blockIndexing(false);
-    trackLayer->reIndex();
-    if (Main)
-        Main->invalidateView();
-
+    if (OK) {
+        if (Main)
+            Main->invalidateView();
+    }
     return OK;
 }
 
@@ -741,11 +780,10 @@ bool downloadMoreOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocum
         theDocument->add(theLayer);
     } else
         theLayer = (Layer*)theDocument->getLastDownloadLayer();
-    theLayer->blockIndexing(true);
 
     QString osmWebsite, osmUser, osmPwd;
 
-    osmWebsite = M_PREFS->getOsmWebsite();
+    osmWebsite = M_PREFS->getOsmApiUrl();
     osmUser = M_PREFS->getOsmUser();
     osmPwd = M_PREFS->getOsmPassword();
 
@@ -754,10 +792,8 @@ bool downloadMoreOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocum
     bool OK = true;
     OK = downloadOSM(Main,osmWebsite,osmUser,osmPwd,aBox,theDocument,theLayer);
     Main->view()->setUpdatesEnabled(true);
-    theLayer->blockIndexing(false);
     if (OK)
     {
-        theLayer->reIndex();
         theDocument->setLastDownloadLayer(theLayer);
         theDocument->addDownloadBox(theLayer, aBox);
         // Don't jump around on Download More
@@ -780,7 +816,7 @@ bool downloadOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocument)
 
     QDialog * dlg = new QDialog(Main);
 
-    osmWebsite = M_PREFS->getOsmWebsite();
+    osmWebsite = M_PREFS->getOsmApiUrl();
     osmUser = M_PREFS->getOsmUser();
     osmPwd = M_PREFS->getOsmPassword();
 
@@ -791,7 +827,7 @@ bool downloadOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocument)
     SlippyMap->setMinimumHeight(256);
 #endif
     CoordBox Clip(aBox);
-    SlippyMap->setViewportArea(Clip.toRectF());
+    SlippyMap->setViewportArea(Clip);
     ui.verticalLayout->addWidget(SlippyMap);
     QObject::connect(SlippyMap, SIGNAL(redraw()), ui.FromMap, SLOT(toggle()));
     BookmarkListIterator i(*(M_PREFS->getBookmarks()));
@@ -800,10 +836,10 @@ bool downloadOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocument)
         if (i.value().deleted == false)
             ui.Bookmarks->addItem(i.key());
     }
-    ui.edXapiUrl->setText(QString("*[bbox=%1,%2,%3,%4]").arg(coordToAng(aBox.bottomLeft().lon()), 0, 'f').arg(coordToAng(aBox.bottomLeft().lat()), 0, 'f').arg(coordToAng(aBox.topRight().lon()), 0, 'f').arg(coordToAng(aBox.topRight().lat()), 0, 'f'));
+    ui.edXapiUrl->setText(QString("*[bbox=%1,%2,%3,%4]").arg(aBox.bottomLeft().x(), 0, 'f').arg(aBox.bottomLeft().y(), 0, 'f').arg(aBox.topRight().x(), 0, 'f').arg(aBox.topRight().y(), 0, 'f'));
     ui.IncludeTracks->setChecked(DownloadRaw);
     ui.ResolveRelations->setChecked(M_PREFS->getResolveRelations());
-    bool OK = true, retry = true, directAPI = false, Regional=false;
+    bool OK = true, retry = true, directAPI = false;
     QString directUrl;
     while (retry) {
         retry = false;
@@ -827,54 +863,63 @@ bool downloadOSM(MainWindow* Main, const CoordBox& aBox , Document* theDocument)
                 if (link.contains("/api/")) {
                     directAPI=true;
                     directUrl = link;
+                } else if (link.contains("/browse/")) {
+                    QString tag("/browse/");
+                    int ix = link.lastIndexOf(tag) + tag.length();
+                    directUrl = M_PREFS->getOsmApiUrl();
+                    if (!directUrl.endsWith("/")) directUrl += "/";
+                    directUrl += link.right(link.length() - ix);
+                    if (!directUrl.endsWith("/")) directUrl += "/";
+                    directUrl += "full";
+                    directAPI=true;
+                } else if (link.startsWith("way") || link.startsWith("node") || link.startsWith("relation")) {
+                    directUrl = M_PREFS->getOsmApiUrl();
+                    if (!directUrl.endsWith("/")) directUrl += "/";
+                    directUrl += link;
+                    directAPI=true;
                 } else {
-                    link.toUInt(&Regional);
-                    if (!Regional) {
-                        OsmLink ol(link);
-                        Clip = ol.getCoordBox();
-                        if (Clip.isNull() || Clip.isEmpty())
-                            retry = true;
-                    }
+                    OsmLink ol(link);
+                    Clip = ol.getCoordBox();
+                    if (Clip.isNull() || Clip.isEmpty())
+                        retry = true;
                 }
             }
             else if (ui.FromXapi->isChecked())
             {
                 directAPI = true;
-                directUrl = M_PREFS->getXapiUrl() + ui.edXapiUrl->text();
+                directUrl = M_PREFS->getXapiUrl();
+                if (!directUrl.endsWith("/")) directUrl += "/";
+                directUrl += ui.edXapiUrl->text();
             }
             else if (ui.FromMap->isChecked())
             {
                 QRectF R(SlippyMap->viewArea());
-                Clip = CoordBox(Coord(R.y(), R.x()), Coord(R.y()+R.height(), R.x()+R.width()));
+                Clip = CoordBox(Coord(R.x(), R.y()), Coord(R.x()+R.width(), R.y()+R.height()));
             }
             if (retry) continue;
             Main->view()->setUpdatesEnabled(false);
             Layer* theLayer = new DrawingLayer(QApplication::translate("Downloader","%1 download").arg(QDateTime::currentDateTime().toString(Qt::ISODate)));
             theDocument->add(theLayer);
-            theLayer->blockIndexing(true);
             M_PREFS->setResolveRelations(ui.ResolveRelations->isChecked());
             if (directAPI) {
                 if (ui.FromXapi->isChecked())
                     theLayer->setUploadable(false);
                 OK = downloadOSM(Main,QUrl(QUrl::fromEncoded(directUrl.toAscii())),osmUser,osmPwd,theDocument,theLayer);
             }
-            else
-            if (Regional)
-                OK = downloadOSM(Main,osmUser,osmPwd,ui.Link->text().toUInt(),theDocument,theLayer);
             else
                 OK = downloadOSM(Main,osmWebsite,osmUser,osmPwd,Clip,theDocument,theLayer);
             if (OK && ui.IncludeTracks->isChecked())
                 OK = downloadTracksFromOSM(Main,osmWebsite,osmUser,osmPwd, Clip,theDocument);
             Main->view()->setUpdatesEnabled(true);
-            theLayer->blockIndexing(false);
             if (OK)
             {
-                theLayer->reIndex();
                 theDocument->setLastDownloadLayer(theLayer);
                 theDocument->addDownloadBox(theLayer, Clip);
+#ifndef _MOBILE
                 if (directAPI)
                     Main->on_viewZoomAllAction_triggered();
                 else
+#endif
                     Main->view()->setViewport(Clip,Main->view()->rect());
                 Main->invalidateView();
             } else {
