@@ -36,47 +36,47 @@ QString OsmLink::parseUrl(QUrl theUrl)
     }
     else if (theUrl.hasQueryItem("lat") && theUrl.hasQueryItem("lon") && theUrl.hasQueryItem("zoom"))
     {
-        double lat = theUrl.queryItemValue("lat").toDouble(&parseOk);   ARG_VALID(lat);
-        double lon = theUrl.queryItemValue("lon").toDouble(&parseOk);   ARG_VALID(lon);
-        double zoom = theUrl.queryItemValue("zoom").toInt(&parseOk);    ARG_VALID(zoom);
+        qreal lat = theUrl.queryItemValue("lat").toDouble(&parseOk);   ARG_VALID(lat);
+        qreal lon = theUrl.queryItemValue("lon").toDouble(&parseOk);   ARG_VALID(lon);
+        qreal zoom = theUrl.queryItemValue("zoom").toInt(&parseOk);    ARG_VALID(zoom);
 
         setLatLonZoom(lat, lon, zoom);
     }
     else if (theUrl.hasQueryItem("mlat") && theUrl.hasQueryItem("mlon") && theUrl.hasQueryItem("zoom"))
     {
-        double lat = theUrl.queryItemValue("mlat").toDouble(&parseOk);   ARG_VALID(lat);
-        double lon = theUrl.queryItemValue("mlon").toDouble(&parseOk);   ARG_VALID(lon);
-        double zoom = theUrl.queryItemValue("zoom").toInt(&parseOk);     ARG_VALID(zoom);
+        qreal lat = theUrl.queryItemValue("mlat").toDouble(&parseOk);   ARG_VALID(lat);
+        qreal lon = theUrl.queryItemValue("mlon").toDouble(&parseOk);   ARG_VALID(lon);
+        qreal zoom = theUrl.queryItemValue("zoom").toInt(&parseOk);     ARG_VALID(zoom);
 
         setLatLonZoom(lat, lon, zoom);
     }
     else if (theUrl.hasQueryItem("minlon") && theUrl.hasQueryItem("maxlon") &&
          theUrl.hasQueryItem("minlat") && theUrl.hasQueryItem("maxlat"))
     {
-        double bottom = theUrl.queryItemValue("minlat").toDouble(&parseOk);   ARG_VALID(minlat);
-        double left = theUrl.queryItemValue("minlon").toDouble(&parseOk);     ARG_VALID(minlon);
-        double top = theUrl.queryItemValue("maxlat").toDouble(&parseOk);      ARG_VALID(maxlat);
-        double right = theUrl.queryItemValue("maxlon").toDouble(&parseOk);    ARG_VALID(maxlon);
+        qreal bottom = theUrl.queryItemValue("minlat").toDouble(&parseOk);   ARG_VALID(minlat);
+        qreal left = theUrl.queryItemValue("minlon").toDouble(&parseOk);     ARG_VALID(minlon);
+        qreal top = theUrl.queryItemValue("maxlat").toDouble(&parseOk);      ARG_VALID(maxlat);
+        qreal right = theUrl.queryItemValue("maxlon").toDouble(&parseOk);    ARG_VALID(maxlon);
 
         setMinMax(bottom, left, top, right);
     }
     else if (theUrl.hasQueryItem("left") && theUrl.hasQueryItem("right") &&
          theUrl.hasQueryItem("bottom") && theUrl.hasQueryItem("top"))
     {
-        double bottom = theUrl.queryItemValue("bottom").toDouble(&parseOk);  ARG_VALID(minlat);
-        double left = theUrl.queryItemValue("left").toDouble(&parseOk);      ARG_VALID(minlon);
-        double top = theUrl.queryItemValue("top").toDouble(&parseOk);        ARG_VALID(maxlat);
-        double right = theUrl.queryItemValue("right").toDouble(&parseOk);    ARG_VALID(maxlon);
+        qreal bottom = theUrl.queryItemValue("bottom").toDouble(&parseOk);  ARG_VALID(minlat);
+        qreal left = theUrl.queryItemValue("left").toDouble(&parseOk);      ARG_VALID(minlon);
+        qreal top = theUrl.queryItemValue("top").toDouble(&parseOk);        ARG_VALID(maxlat);
+        qreal right = theUrl.queryItemValue("right").toDouble(&parseOk);    ARG_VALID(maxlon);
 
         setMinMax(bottom, left, top, right);
     }
     else if (theUrl.hasQueryItem("left") && theUrl.hasQueryItem("right") &&
          theUrl.hasQueryItem("bottom") && theUrl.hasQueryItem("top"))
     {
-        double bottom = theUrl.queryItemValue("bottom").toDouble(&parseOk);  ARG_VALID(minlat);
-        double left = theUrl.queryItemValue("left").toDouble(&parseOk);      ARG_VALID(minlon);
-        double top = theUrl.queryItemValue("top").toDouble(&parseOk);        ARG_VALID(maxlat);
-        double right = theUrl.queryItemValue("right").toDouble(&parseOk);    ARG_VALID(maxlon);
+        qreal bottom = theUrl.queryItemValue("bottom").toDouble(&parseOk);  ARG_VALID(minlat);
+        qreal left = theUrl.queryItemValue("left").toDouble(&parseOk);      ARG_VALID(minlon);
+        qreal top = theUrl.queryItemValue("top").toDouble(&parseOk);        ARG_VALID(maxlat);
+        qreal right = theUrl.queryItemValue("right").toDouble(&parseOk);    ARG_VALID(maxlon);
 
         setMinMax(bottom, left, top, right);
     }
@@ -84,11 +84,11 @@ QString OsmLink::parseUrl(QUrl theUrl)
          theUrl.hasQueryItem("ll") && theUrl.hasQueryItem("spn"))
     {
         QStringList ll = theUrl.queryItemValue("ll").split(",");     if (ll.count() != 2) return QString("Unsplit=\"%2\" (%1 elements)").arg(ll.count()).arg(theUrl.queryItemValue("ll"));
-        double lat = ll[0].toDouble(&parseOk);                       PARSE_ERROR(ll[0]);
-        double lon = ll[1].toDouble(&parseOk);                       PARSE_ERROR(ll[1]);
+        qreal lat = ll[0].toDouble(&parseOk);                       PARSE_ERROR(ll[0]);
+        qreal lon = ll[1].toDouble(&parseOk);                       PARSE_ERROR(ll[1]);
         QStringList spn = theUrl.queryItemValue("spn").split(",");   if (spn.count() != 2) return QString("Unsplit=\"%2\" (%1 elements)").arg(spn.count()).arg(theUrl.queryItemValue("spn"));
-        double spanLat = spn[0].toDouble(&parseOk);                  PARSE_ERROR(spn[0]);
-        double spanLon = spn[1].toDouble(&parseOk);                  PARSE_ERROR(spn[1]);
+        qreal spanLat = spn[0].toDouble(&parseOk);                  PARSE_ERROR(spn[0]);
+        qreal spanLon = spn[1].toDouble(&parseOk);                  PARSE_ERROR(spn[1]);
 
         setMinMax(lat-spanLat, lon-spanLon/2, lat+spanLat, lon+spanLon/2);
     }
@@ -98,7 +98,7 @@ QString OsmLink::parseUrl(QUrl theUrl)
 void OsmLink::parseShortUrl(QString code)
 {
     // TWO_32 defined in terms of 1<<16 because 1L<<32 may overflow if long is only 32 bits
-    static const double TWO_32 = (double)(1<<16) * (double)(1<<16);
+    static const qreal TWO_32 = (double)(1<<16) * (double)(1<<16);
     static const QByteArray possibleChar("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_@");
         int x = 0;
         int y = 0;
@@ -129,8 +129,8 @@ void OsmLink::parseShortUrl(QString code)
         x <<= (32 - z);
         y <<= (32 - z);
 
-        double lon = (x * 360. / TWO_32) - 180.;
-        double lat = (y * 180. / TWO_32) - 90.;
+        qreal lon = (x * 360. / TWO_32) - 180.;
+        qreal lat = (y * 180. / TWO_32) - 90.;
         if (z_offset)
         z_offset = (3 - (z_offset % 3));
         int zoom = z - 8  - z_offset;
@@ -143,7 +143,7 @@ void OsmLink::parseShortUrl(QString code)
     setLatLonZoom(lat, lon, zoom);
 }
 
-void OsmLink::setLatLonZoom(double lat, double lon, int zoom)
+void OsmLink::setLatLonZoom(qreal lat, qreal lon, int zoom)
 {
     // Constrain zoom
     if (zoom > 20)
@@ -152,18 +152,18 @@ void OsmLink::setLatLonZoom(double lat, double lon, int zoom)
         zoom = 15;
 
     /* term to calculate the angle from the zoom-value */
-    double zoomLat = 360.0 / (double)(1 << zoom);
-    double zoomLon = zoomLat / fabs(cos(angToRad(lat)));
+    qreal zoomLat = 360.0 / (double)(1 << zoom);
+    qreal zoomLon = zoomLat / fabs(cos(angToRad(lat)));
     /* the following line is equal to the line above. (just for explanation) */
-    //double zoomLon = zoomLat / aParent->view()->projection().latAnglePerM() * aParent->view()->projection().lonAnglePerM(angToRad(lat));
+    //qreal zoomLon = zoomLat / aParent->view()->projection().latAnglePerM() * aParent->view()->projection().lonAnglePerM(angToRad(lat));
 
     /* the OSM link contains the coordinates from the middle of the visible map so we have to add and sub zoomLon/zoomLat */
     setMinMax(lat-zoomLat, lon-zoomLon, lat+zoomLat, lon+zoomLon);
 }
 
-void OsmLink::setMinMax(double bottom, double left, double top, double right)
+void OsmLink::setMinMax(qreal bottom, qreal left, qreal top, qreal right)
 {
-    const double minwidth = 0.0001;
+    const qreal minwidth = 0.0001;
     if (right-left < minwidth) {
         left = (left + right - minwidth) / 2.0;
         right = left + minwidth;
@@ -172,6 +172,6 @@ void OsmLink::setMinMax(double bottom, double left, double top, double right)
         top = (top + bottom + minwidth) / 2.0;
         bottom = top - minwidth;
     }
-    m_Box = CoordBox(Coord(angToCoord(bottom), angToCoord(left)), Coord(angToCoord(top), angToCoord(right)));
+    m_Box = CoordBox(Coord(left, bottom), Coord(right, top));
     m_IsValid = true;
 }
