@@ -22,14 +22,44 @@ bool canParseSymbol(const QString& Expression, int& idx, char Symbol)
     return false;
 }
 
+bool canParseString(const QString& Expression, int& idx, QString& Key)
+{
+    Key = "";
+    skipWhite(Expression,idx);
+    if (idx >= Expression.length())
+        return false;
+
+    if (Expression[idx] != '/' && Expression[idx] != '"')
+        return false;
+    Key += Expression[idx++];
+    while (idx < Expression.length())
+    {
+        if (Expression[idx] != '/' && Expression[idx] != '"')
+            Key += Expression[idx++];
+        else
+            break;
+    }
+    if (Expression[idx] == '/' || Expression[idx] == '"') {
+        Key += Expression[idx++];
+        return Key.length() > 0;
+    }
+    return false;
+}
+
+
 bool canParseValue(const QString& Expression, int& idx, QString& Key)
 {
     Key = "";
     skipWhite(Expression,idx);
+    if (idx >= Expression.length())
+        return false;
+
     unsigned short opened =0;
+    if (Expression[idx] == '/' || Expression[idx] == '"')  //Constants
+        return false;
     while (idx < Expression.length())
     {
-        if ( ((Expression[idx] == '_') || (Expression[idx].isLetterOrNumber()) /*|| (Expression[idx].isPunct())*/ || (Expression[idx] == '*') || (Expression[idx] == ':') || (Expression[idx] == '?'))
+        if ( ((Expression[idx] == '_') || (Expression[idx].isLetterOrNumber()) || (Expression[idx] == '-') /*|| (Expression[idx].isPunct())*/ || (Expression[idx] == '*') || (Expression[idx] == ':') || (Expression[idx] == '?'))
                 &&  ( (Expression[idx] != '[') && (Expression[idx] != ']') && (Expression[idx] != ',') && (Expression[idx] != '(')&& (Expression[idx] != ')')) )
             Key += Expression[idx++];
         else if ( Expression[idx] == '[' )
@@ -51,6 +81,9 @@ bool canParseValue(const QString& Expression, int& idx, QString& Key)
 
 bool canParseKey(const QString& Expression, int& idx, QString& Key)
 {
+    if (idx >= Expression.length())
+        return false;
+
     if (!canParseSymbol(Expression,idx,'['))
         return false;
     if (!canParseValue(Expression,idx,Key))
@@ -62,6 +95,9 @@ bool canParseKey(const QString& Expression, int& idx, QString& Key)
 bool canParseLiteral(const QString& Expression, int& idx, const QString& Literal)
 {
     skipWhite(Expression,idx);
+    if (idx >= Expression.length())
+        return false;
+
     if (Expression.indexOf(Literal, idx, Qt::CaseInsensitive) == idx) {
         idx += Literal.length();
         return true;
@@ -92,7 +128,7 @@ TagSelectorOperator* parseTagSelectorOperator(const QString& Expression, int& id
     if (Oper.isNull())
         return 0;
 
-    if (!canParseValue(Expression, idx, Value))
+    if (!canParseString(Expression, idx, Value) && !canParseValue(Expression, idx, Value))
         return 0;
     return new TagSelectorOperator(Key, Oper, Value);
 }
@@ -242,6 +278,11 @@ TagSelector* parseFactor(const QString& Expression, int& idx)
             Current = parseFactor("not(" + Key + " is _NULL_)", TmpIdx);
         }
     }
+    if (!Current)
+    {
+        ++idx;
+        return NULL;
+    }
     return Current;
 }
 
@@ -284,7 +325,7 @@ TagSelector* parseTagSelector(const QString& Expression, int& idx)
         return Terms[0];
     else if (Terms.size() > 1)
         return new TagSelectorOr(Terms);
-    return new TagSelectorTrue();
+    return new TagSelectorFalse();
 }
 
 TagSelector* TagSelector::parse(const QString& Expression)
@@ -306,7 +347,7 @@ TagSelector::~TagSelector()
 /* TAGSELECTOROPERATOR */
 
 TagSelectorOperator::TagSelectorOperator(const QString& key, const QString& oper, const QString& value)
-    : Key(key), Oper(oper), Value(value), UseRegExp(false)
+    : Key(key), Oper(oper), Value(value), UseSimpleRegExp(false), UseFullRegExp(false)
     , specialKey(TagSelectKey_None)
     , specialValue(TagSelectValue_None)
 {
@@ -332,13 +373,28 @@ TagSelectorOperator::TagSelectorOperator(const QString& key, const QString& oper
     else if (key.toLower() == ":uploaded")
         specialKey = TagSelectKey_Uploaded;
 
+    boolVal = false;
     if (value.toUpper() == "_NULL_") {
         specialValue = TagSelectValue_Empty;
+    } else if (value.toUpper() == "TRUE") {
+        boolVal = true;
+        valB = true;
+    } else if (Value.toUpper() == "FALSE") {
+        boolVal = true;
+        valB = false;
+    } else if (value.startsWith("/") && value.endsWith("/")) {
+        UseFullRegExp = true;
+        QString r = value.mid(1);
+        r.chop(1);
+        rx = QRegExp(r, Qt::CaseInsensitive);
+        rx.setPatternSyntax(QRegExp::RegExp2);
     } else if (value.contains(QRegExp("[][*?]"))) {
-        UseRegExp = true;
+        UseSimpleRegExp = true;
         rx = QRegExp(value, Qt::CaseInsensitive);
         rx.setPatternSyntax(QRegExp::Wildcard);
     }
+    valN = Value.toDouble(&okval);
+
     // Else exact match against ->Value only
 
     if (Oper == ">")
@@ -362,17 +418,110 @@ TagSelector* TagSelectorOperator::copy() const
 
 static const QString emptyString("__EMPTY__");
 
-TagSelectorMatchResult TagSelectorOperator::matches(const IFeature* F, double PixelPerM) const
+TagSelectorMatchResult TagSelectorOperator::evaluateVal(const QString& val) const
 {
-    bool boolVal = false, valB;
-    if (Value.toLower() == "true") {
-        boolVal = true;
-        valB = true;
-    } else if (Value.toLower() == "false") {
-        boolVal = true;
-        valB = false;
-    }
+    if (val == emptyString && specialValue != TagSelectValue_Empty)
+        return TagSelect_NoMatch;
+    if (specialValue == TagSelectValue_Empty) {
+        if (theOp == EQ) {
+            if (val.toUpper() == emptyString) return TagSelect_Match;
+        } else {
+            if (val.toUpper() != emptyString) return TagSelect_Match;
+        }
+    } else if (UseSimpleRegExp) {
+        if (rx.exactMatch(val))  {
+            if (theOp == EQ)
+                return TagSelect_Match;
+        } else {
+            if (theOp != EQ)
+                return TagSelect_Match;
+        }
 
+    } else if (UseFullRegExp) {
+        if (rx.indexIn(val) != -1)  {
+            if (theOp == EQ)
+                return TagSelect_Match;
+        } else {
+            if (theOp != EQ)
+                return TagSelect_Match;
+        }
+
+    } else {
+        bool okkey;
+        qreal keyN = val.toDouble(&okkey);
+        if (boolVal)
+            switch (theOp) {
+            case EQ:
+                if (valB) {
+                    if (val.toLower() == "true" || val.toLower() == "yes" || val == "1")
+                        return TagSelect_Match;
+                } else {
+                    if (val.toLower() == "false" || val.toLower() == "no" || val == "0")
+                        return TagSelect_Match;
+                }
+                break;
+
+            case NE:
+                if (valB) {
+                    if (val.toLower() == "false" || val.toLower() == "no" || val == "0")
+                        return TagSelect_Match;
+                } else {
+                    if (val.toLower() == "true" || val.toLower() == "yes" || val == "1")
+                        return TagSelect_Match;
+                }
+                break;
+
+            default:
+                break;
+            }
+        else if (okkey && okval)
+            switch (theOp) {
+            case EQ:
+                if (keyN == valN) return TagSelect_Match;
+                break;
+            case NE:
+                if (keyN != valN) return TagSelect_Match;
+                break;
+            case GT:
+                if (keyN > valN) return TagSelect_Match;
+                break;
+            case LT:
+                if (keyN < valN) return TagSelect_Match;
+                break;
+            case GE:
+                if (keyN >= valN) return TagSelect_Match;
+                break;
+            case LE:
+                if (keyN <= valN) return TagSelect_Match;
+                break;
+            }
+        else
+            switch (theOp) {
+            case EQ:
+                if ((QString::compare(val, Value, Qt::CaseInsensitive)) == 0) return TagSelect_Match;
+                break;
+            case NE:
+                if ((QString::compare(val, Value, Qt::CaseInsensitive)) != 0 ) return TagSelect_Match;
+                break;
+            case GT:
+                if ((QString::compare(val, Value, Qt::CaseInsensitive)) > 0) return TagSelect_Match;
+                break;
+            case LT:
+                if ((QString::compare(val, Value, Qt::CaseInsensitive)) < 0) return TagSelect_Match;
+                break;
+            case GE:
+                if ((QString::compare(val, Value, Qt::CaseInsensitive)) >= 0) return TagSelect_Match;
+                break;
+            case LE:
+                if ((QString::compare(val, Value, Qt::CaseInsensitive)) <= 0) return TagSelect_Match;
+                break;
+            }
+    }
+    return TagSelect_NoMatch;
+}
+
+TagSelectorMatchResult TagSelectorOperator::matches(const IFeature* F, qreal PixelPerM) const
+{
     if (specialKey != TagSelectKey_None) {
         switch (specialKey) {
         case TagSelectKey_Id:
@@ -398,6 +547,7 @@ TagSelectorMatchResult TagSelectorOperator::matches(const IFeature* F, double Pi
             }
             break;
 
+#ifndef FRISIUS_BUILD
         case TagSelectKey_User:
             switch (theOp) {
             case EQ:
@@ -493,12 +643,13 @@ TagSelectorMatchResult TagSelectorOperator::matches(const IFeature* F, double Pi
             }
 
             break;
+#endif
 
         case TagSelectKey_PixelPerM: {
             if (!PixelPerM)
                 return TagSelect_Match;
             bool okval;
-            double valN = Value.toDouble(&okval);
+            qreal valN = Value.toDouble(&okval);
             if (!okval)
                 return TagSelect_NoMatch;
             switch (theOp) {
@@ -607,96 +758,12 @@ TagSelectorMatchResult TagSelectorOperator::matches(const IFeature* F, double Pi
             break;
         }
     } else {
-        QList<QString> valueList;
         if (Key != "*")
-            valueList << F->tagValue(Key, emptyString);
+            return evaluateVal(F->tagValue(Key, emptyString));
         else {
             for (int i=0; i<F->tagSize(); ++i)
-                valueList << F->tagValue(i);
-        }
-        foreach (QString val, valueList) {
-            if (val == emptyString && specialValue != TagSelectValue_Empty)
-                return TagSelect_NoMatch;
-            if (specialValue == TagSelectValue_Empty) {
-                if (theOp == EQ) {
-                    if (val.toUpper() == emptyString) return TagSelect_Match;
-                } else {
-                    if (val.toUpper() != emptyString) return TagSelect_Match;
-                }
-            } else if (UseRegExp) {
-                if (rx.exactMatch(val)) return TagSelect_Match;
-            } else {
-                bool okkey, okval;
-                double keyN = val.toDouble(&okkey);
-                double valN = Value.toDouble(&okval);
-                if (boolVal)
-                    switch (theOp) {
-                    case EQ:
-                    if (valB) {
-                        if (val.toLower() == "true" || val.toLower() == "yes" || val == "1")
-                            return TagSelect_Match;
-                    } else {
-                        if (val.toLower() == "false" || val.toLower() == "no" || val == "0")
-                            return TagSelect_Match;
-                    }
-                    break;
-
-                case NE:
-                    if (valB) {
-                        if (val.toLower() == "false" || val.toLower() == "no" || val == "0")
-                            return TagSelect_Match;
-                    } else {
-                        if (val.toLower() == "true" || val.toLower() == "yes" || val == "1")
-                            return TagSelect_Match;
-                    }
-                    break;
-
-                default:
-                    break;
-                }
-                else if (okkey && okval)
-                    switch (theOp) {
-                    case EQ:
-                        if (keyN == valN) return TagSelect_Match;
-                        break;
-                    case NE:
-                        if (keyN != valN) return TagSelect_Match;
-                        break;
-                    case GT:
-                        if (keyN > valN) return TagSelect_Match;
-                        break;
-                    case LT:
-                        if (keyN < valN) return TagSelect_Match;
-                        break;
-                    case GE:
-                        if (keyN >= valN) return TagSelect_Match;
-                        break;
-                    case LE:
-                        if (keyN <= valN) return TagSelect_Match;
-                        break;
-                    }
-                else
-                    switch (theOp) {
-                    case EQ:
-                        if ((QString::compare(val, Value, Qt::CaseInsensitive)) == 0) return TagSelect_Match;
-                        break;
-                    case NE:
-                        if ((QString::compare(val, Value, Qt::CaseInsensitive)) != 0 ) return TagSelect_Match;
-                        break;
-                    case GT:
-                        if ((QString::compare(val, Value, Qt::CaseInsensitive)) > 0) return TagSelect_Match;
-                        break;
-                    case LT:
-                        if ((QString::compare(val, Value, Qt::CaseInsensitive)) < 0) return TagSelect_Match;
-                        break;
-                    case GE:
-                        if ((QString::compare(val, Value, Qt::CaseInsensitive)) >= 0) return TagSelect_Match;
-                        break;
-                     case LE:
-                        if ((QString::compare(val, Value, Qt::CaseInsensitive)) <= 0) return TagSelect_Match;
-                        break;
-                    }
-            }
+                if (evaluateVal(F->tagValue(i)) == TagSelect_Match)
+                    return TagSelect_Match;
         }
     }
     return TagSelect_NoMatch;
@@ -748,7 +815,7 @@ TagSelector* TagSelectorIsOneOf::copy() const
     return new TagSelectorIsOneOf(Key,Values);
 }
 
-TagSelectorMatchResult TagSelectorIsOneOf::matches(const IFeature* F, double /*PixelPerM*/) const
+TagSelectorMatchResult TagSelectorIsOneOf::matches(const IFeature* F, qreal /*PixelPerM*/) const
 {
     if (specialKey != TagSelectKey_None) {
         foreach (QString Value, exactMatchv) {
@@ -758,6 +825,7 @@ TagSelectorMatchResult TagSelectorIsOneOf::matches(const IFeature* F, double /*P
                     return TagSelect_Match;
                 break;
 
+#ifndef FRISIUS_BUILD
             case TagSelectKey_User:
                 if (QString::compare(F->user(), Value, Qt::CaseInsensitive) == 0)
                     return TagSelect_Match;
@@ -782,6 +850,7 @@ TagSelectorMatchResult TagSelectorIsOneOf::matches(const IFeature* F, double /*P
                 if (F->versionNumber() == Value.toInt())
                     return TagSelect_Match;
                 break;
+#endif
 
             default:
                 break;
@@ -830,7 +899,7 @@ TagSelector* TagSelectorTypeIs::copy() const
     return new TagSelectorTypeIs(Type);
 }
 
-TagSelectorMatchResult TagSelectorTypeIs::matches(const IFeature* F, double /*PixelPerM*/) const
+TagSelectorMatchResult TagSelectorTypeIs::matches(const IFeature* F, qreal /*PixelPerM*/) const
 {
     QString t = Type.toLower();
     if (t == "node")
@@ -867,7 +936,7 @@ TagSelector* TagSelectorHasTags::copy() const
     return new TagSelectorHasTags();
 }
 
-TagSelectorMatchResult TagSelectorHasTags::matches(const IFeature* F, double /*PixelPerM*/) const
+TagSelectorMatchResult TagSelectorHasTags::matches(const IFeature* F, qreal /*PixelPerM*/) const
 {
     for (int i=0; i<F->tagSize(); ++i) {
         if (!TechnicalTags.contains(F->tagKey(i))) {
@@ -905,7 +974,7 @@ TagSelector* TagSelectorOr::copy() const
     return new TagSelectorOr(Copied);
 }
 
-TagSelectorMatchResult TagSelectorOr::matches(const IFeature* F, double PixelPerM) const
+TagSelectorMatchResult TagSelectorOr::matches(const IFeature* F, qreal PixelPerM) const
 {
     for (int i=0; i<Terms.size(); ++i)
         if (Terms[i]->matches(F,PixelPerM) == TagSelect_Match)
@@ -951,7 +1020,7 @@ TagSelector* TagSelectorAnd::copy() const
     return new TagSelectorAnd(Copied);
 }
 
-TagSelectorMatchResult TagSelectorAnd::matches(const IFeature* F, double PixelPerM) const
+TagSelectorMatchResult TagSelectorAnd::matches(const IFeature* F, qreal PixelPerM) const
 {
     for (int i=0; i<Terms.size(); ++i)
         if (Terms[i]->matches(F,PixelPerM) == TagSelect_NoMatch)
@@ -990,7 +1059,7 @@ TagSelector* TagSelectorNot::copy() const
     return new TagSelectorNot(Term->copy());
 }
 
-TagSelectorMatchResult TagSelectorNot::matches(const IFeature* F, double PixelPerM) const
+TagSelectorMatchResult TagSelectorNot::matches(const IFeature* F, qreal PixelPerM) const
 {
     if (!Term)
         return TagSelect_NoMatch;
@@ -1027,7 +1096,7 @@ TagSelector* TagSelectorParent::copy() const
     return new TagSelectorParent(Term->copy());
 }
 
-TagSelectorMatchResult TagSelectorParent::matches(const IFeature* F, double PixelPerM) const
+TagSelectorMatchResult TagSelectorParent::matches(const IFeature* F, qreal PixelPerM) const
 {
     if (!Term)
         return TagSelect_NoMatch;
@@ -1064,7 +1133,7 @@ TagSelector* TagSelectorFalse::copy() const
     return new TagSelectorFalse();
 }
 
-TagSelectorMatchResult TagSelectorFalse::matches(const IFeature* /* F */, double /*PixelPerM*/) const
+TagSelectorMatchResult TagSelectorFalse::matches(const IFeature* /* F */, qreal /*PixelPerM*/) const
 {
     return TagSelect_NoMatch;
 }
@@ -1087,7 +1156,7 @@ TagSelector* TagSelectorTrue::copy() const
     return new TagSelectorFalse();
 }
 
-TagSelectorMatchResult TagSelectorTrue::matches(const IFeature* /* F */, double /*PixelPerM*/) const
+TagSelectorMatchResult TagSelectorTrue::matches(const IFeature* /* F */, qreal /*PixelPerM*/) const
 {
     return TagSelect_Match;
 }
@@ -1116,7 +1185,7 @@ TagSelector* TagSelectorDefault::copy() const
     return new TagSelectorDefault(Term->copy());
 }
 
-TagSelectorMatchResult TagSelectorDefault::matches(const IFeature* F, double PixelPerM) const
+TagSelectorMatchResult TagSelectorDefault::matches(const IFeature* F, qreal PixelPerM) const
 {
     //return (Term->matches(F) == TagSelect_Match) ? TagSelect_DefaultMatch : TagSelect_NoMatch;
     if (Term->matches(F,PixelPerM) == TagSelect_Match)
